@@ -1,15 +1,15 @@
 # -*- coding: iso-8859-1 -*-
 """
     MoinMoin - Page class
-    
+
     Page is used for read-only access to a wiki page. For r/w access see PageEditor.
     A Page object is used to access a wiki page (in general) as well as to access
     some specific revision of a wiki page.
-    
-    The RootPage is some virtual page located at / and is mainly used to do namespace
-    operations like getting the page list. 
 
-    Currently, this is all a big mixture between high-level page code, intermediate 
+    The RootPage is some virtual page located at / and is mainly used to do namespace
+    operations like getting the page list.
+
+    Currently, this is all a big mixture between high-level page code, intermediate
     data/underlay layering code, caching code and low-level filesystem storage code.
     To see the filesystem storage layout we use, best is to look into data/pages/
     (underlay dir uses the same format).
@@ -28,13 +28,16 @@
       The mimetype hierarchy should be modelled by a MimeTypeItem class hierarchy.
 
     @copyright: 2000-2004 by Juergen Hermann <jh@web.de>,
-                2005-2007 by MoinMoin:ThomasWaldmann,
+                2005-2008 by MoinMoin:ThomasWaldmann,
                 2006 by MoinMoin:FlorianFesti,
                 2007 by MoinMoin:ReimarBauer
     @license: GNU GPL, see COPYING for details.
 """
 
-import os, re, codecs, logging
+import os, re, codecs
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin import config, caching, user, util, wikiutil
 from MoinMoin.logfile import eventlog
@@ -201,7 +204,7 @@ class Page(object):
         self._pagepath = [normalpath, underlaypath]
 
     # now we define some properties to lazy load some attributes on first access:
-    
+
     def get_body(self):
         if self.__body is None:
             # try to open file
@@ -224,7 +227,8 @@ class Page(object):
                 self.__body = text
             finally:
                 f.close()
-        return self.__body    
+        return self.__body
+
     def set_body(self, newbody):
         self.__body = newbody
         self.__meta = None
@@ -248,7 +252,7 @@ class Page(object):
             self.__pi = self.parse_processing_instructions()
         return self.__pi
     pi = property(fget=get_pi) # processed meta stuff
-    
+
     def getlines(self):
         """ Return a list of all lines in body.
 
@@ -304,7 +308,7 @@ class Page(object):
 
     def get_rev_dir(self, pagedir, rev=0):
         """ Get a revision of a page from an arbitrary pagedir.
-        
+
         Does not modify page object's state, uncached, direct disk access.
 
         @param pagedir: the path to the page storage area
@@ -329,21 +333,21 @@ class Page(object):
 
     def _setRealPageName(self, pagedir):
         """ Set page_name to the real case of page name
-        
+
         On case insensitive file system, "pagename" exists even if the
         real page name is "PageName" or "PAGENAME". This leads to
         confusion in urls, links and logs.
         See MoinMoinBugs/MacHfsPlusCaseInsensitive
-        
+
         Correct the case of the page name. Elements created from the
         page name in reset() are not updated because it's too messy, and
         this fix seems to be enough for now.
-        
+
         Problems to fix later:
-        
+
          - ["helponnavigation"] link to HelpOnNavigation but not
            considered as backlink.
-        
+
         @param pagedir: the storage path to the page directory
         """
         if self._text_filename_force is None:
@@ -405,9 +409,9 @@ class Page(object):
 
     def current_rev(self):
         """ Return number of current revision.
-        
+
         This is the same as get_rev()[1].
-        
+
         @return: int revision
         """
         pagefile, rev, exists = self.get_rev()
@@ -478,7 +482,7 @@ class Page(object):
         isfile = kw.get('isfile', 0)
         use_underlay = kw.get('use_underlay', -1)
         underlay, path = self.getPageBasePath(use_underlay)
-        fullpath = os.path.join(*((path,) + args))
+        fullpath = os.path.join(*((path, ) + args))
         if check_create:
             if isfile:
                 dirname, filename = os.path.split(fullpath)
@@ -510,87 +514,83 @@ class Page(object):
         fname, rev, exists = self.get_rev(-1, rev)
         return fname
 
-    # XXX TODO clean up the mess, rewrite _last_edited, last_edit, lastEditInfo for new logs,
-    # XXX TODO do not use mtime() calls any more
-    def _last_edited(self, request):
-        # as it is implemented now, this is rather a _last_changed as it just uses
-        # the last log entry, which could be not only from an edit, but also from 
-        # an attachment operation. See different semantics in .mtime().
-        cache_name = self.page_name
-        cache_key = 'lastlog'
-        log = request.cfg.cache.meta.getItem(request, cache_name, cache_key)
-        if log is None:
+    def editlog_entry(self):
+        """ Return the edit-log entry for this Page object (can be an old revision).
+        """
+        request = self.request
+        use_cache = self.rev == 0 # use the cache for current rev
+        if use_cache:
+            cache_name, cache_key = self.page_name, 'lastlog'
+            entry = request.cfg.cache.meta.getItem(request, cache_name, cache_key)
+        else:
+            entry = None
+        if entry is None:
             from MoinMoin.logfile import editlog
-            try:
-                logfile = editlog.EditLog(request, rootpagename=self.page_name)
-                logfile.to_end()
-                log = logfile.previous()
-            except StopIteration:
-                log = () # don't use None!
-            request.cfg.cache.meta.putItem(request, cache_name, cache_key, log)
-        return log
+            wanted_rev = "%08d" % self.get_real_rev()
+            edit_log = editlog.EditLog(request, rootpagename=self.page_name)
+            for entry in edit_log.reverse():
+                if entry.rev == wanted_rev:
+                    break
+            else:
+                entry = () # don't use None
+            if use_cache:
+                request.cfg.cache.meta.putItem(request, cache_name, cache_key, entry)
+        return entry
+
+    def edit_info(self):
+        """ Return timestamp/editor info for this Page object (can be an old revision).
+
+            Note: if you ask about a deleted revision, it will report timestamp and editor
+                  for the delete action (in the edit-log, this is just a SAVE).
 
-    def last_edit(self, request):
-        """ Return the last edit.
         This is used by MoinMoin/xmlrpc/__init__.py.
 
-        @param request: the request object
         @rtype: dict
         @return: timestamp and editor information
         """
-        if not self.exists():
-            return None
-
-        result = None
-        if not self.rev:
-            log = self._last_edited(request)
-            if log:
-                editordata = log.getInterwikiEditorData(request)
-                editor = editordata[1]
-                if editordata[0] == 'interwiki':
-                    editor = "%s:%s" % editordata[1]
-                else: # 'ip'
-                    editor = editordata[1]
-                result = {
-                    'timestamp': log.ed_time_usecs,
-                    'editor': editor,
-                }
-                del log
-        if not result:
-            version = self.mtime_usecs()
+        line = self.editlog_entry()
+        if line:
+            editordata = line.getInterwikiEditorData(self.request)
+            if editordata[0] == 'interwiki':
+                editor = "%s:%s" % editordata[1]
+            else:
+                editor = editordata[1] # ip or email
             result = {
-                'timestamp': version,
-                'editor': '?',
+                'timestamp': line.ed_time_usecs,
+                'editor': editor,
             }
-
+            del line
+        else:
+            result = {}
         return result
 
+    def last_edit(self, request):
+        # XXX usage of last_edit is DEPRECATED - use edit_info()
+        if not self.exists(): # XXX doesn't make much sense, but still kept
+            return None       # XXX here until we remove last_edit()
+        return self.edit_info()
+
     def lastEditInfo(self, request=None):
         """ Return the last edit info.
 
-        @param request: the request object
+            Note: if you ask about a deleted revision, it will report timestamp and editor
+                  for the delete action (in the edit-log, this is just a SAVE).
+
+        @param request: the request object (DEPRECATED, unused)
         @rtype: dict
         @return: timestamp and editor information
         """
-        if not self.exists():
-            return {}
-        if request is None:
-            request = self.request
-
-        # Try to get data from log
-        log = self._last_edited(request)
+        log = self.editlog_entry()
         if log:
+            request = self.request
             editor = log.getEditor(request)
             time = wikiutil.version2timestamp(log.ed_time_usecs)
+            time = request.user.getFormattedDateTime(time) # Use user time format
+            result = {'editor': editor, 'time': time}
             del log
-        # Or from the file system
         else:
-            editor = ''
-            time = os.path.getmtime(self._text_filename())
-
-        # Use user time format
-        time = request.user.getFormattedDateTime(time)
-        return {'editor': editor, 'time': time}
+            result = {}
+        return result
 
     def isWritable(self):
         """ Can this page be changed?
@@ -685,32 +685,13 @@ class Page(object):
             raise
 
     def mtime_usecs(self):
-        """ Get modification timestamp of this page.
+        """ Get modification timestamp of this page (from edit-log, can be for an old revision).
 
-        @rtype: long
-        @return: mtime of page (or 0 if page does not exist)
+        @rtype: int
+        @return: mtime of page (or 0 if page / edit-log entry does not exist)
         """
-        request = self.request
-        cache_name = self.page_name
-        cache_key = 'lastpagechange'
-        mtime = request.cfg.cache.meta.getItem(request, cache_name, cache_key)
-        current_wanted = (self.rev == 0) # True if we search for the current revision
-        if mtime is None or not current_wanted:
-            from MoinMoin.logfile import editlog
-            wanted_rev = "%08d" % self.rev
-            mtime = 0L
-            try:
-                logfile = editlog.EditLog(self.request, rootpagename=self.page_name)
-                for line in logfile.reverse():
-                    if (current_wanted and line.rev != 99999999) or line.rev == wanted_rev:
-                        mtime = line.ed_time_usecs
-                        break
-            except StopIteration:
-                pass
-            if current_wanted:
-                request.cfg.cache.meta.putItem(request, cache_name, cache_key, mtime)
-
-        return mtime
+        entry = self.editlog_entry()
+        return entry and entry.ed_time_usecs or 0
 
     def mtime_printable(self, request):
         """ Get printable (as per user's preferences) modification timestamp of this page.
@@ -739,11 +720,10 @@ class Page(object):
 
         # look for the end of words and the start of a new word,
         # and insert a space there
-        split_re = re.compile('([%s])([%s])' % (config.chars_lower, config.chars_upper))
-        splitted = split_re.sub(r'\1 \2', self.page_name)
+        splitted = config.split_regex.sub(r'\1 \2', self.page_name)
         return splitted
 
-    def url(self, request, querystr=None, anchor=None, relative=True, **kw):
+    def url(self, request, querystr=None, anchor=None, relative=False, **kw):
         """ Return complete URL for this page, including scriptname.
             The URL is NOT escaped, if you write it to HTML, use wikiutil.escape
             (at least if you have a querystr, to escape the & chars).
@@ -752,6 +732,8 @@ class Page(object):
         @param querystr: the query string to add after a "?" after the url
             (str or dict, see wikiutil.makeQueryString)
         @param anchor: if specified, make a link to this anchor
+        @param relative: create a relative link (default: False), note that this
+                         changed in 1.7, in 1.6, the default was True.
         @rtype: str
         @return: complete url of this page, including scriptname
         """
@@ -781,7 +763,7 @@ class Page(object):
 
     def link_to_raw(self, request, text, querystr=None, anchor=None, **kw):
         """ core functionality of link_to, without the magic """
-        url = self.url(request, querystr, anchor=anchor)
+        url = self.url(request, querystr, anchor=anchor, relative=True) # scriptName is added by link_tag
         # escaping is done by link_tag -> formatter.url -> ._open()
         link = wikiutil.link_tag(request, url, text,
                                  formatter=getattr(self, 'formatter', None), **kw)
@@ -831,13 +813,11 @@ class Page(object):
         @param request: the request object
         @keyword include_self: if 1, include current user (default: 0)
         @keyword return_users: if 1, return user instances (default: 0)
-        @keyword trivial: if 1, only include users who want trivial changes (default: 0)
         @rtype: dict
         @return: lists of subscribed email addresses in a dict by language key
         """
         include_self = kw.get('include_self', self.include_self)
         return_users = kw.get('return_users', 0)
-        trivial = kw.get('trivial', 0)
 
         # extract categories of this page
         pageList = self.getCategories(request)
@@ -853,8 +833,6 @@ class Page(object):
         # get email addresses of the all wiki user which have a profile stored;
         # add the address only if the user has subscribed to the page and
         # the user is not the current editor
-        # Also, if the change is trivial (send email isn't ticked) only send email to users
-        # who want_trivial changes (typically Admins on public sites)
         userlist = user.getUserList(request)
         subscriber_list = {}
         for uid in userlist:
@@ -862,25 +840,33 @@ class Page(object):
                 continue # no self notification
             subscriber = user.User(request, uid)
 
+            # The following tests should be ordered in order of
+            # decreasing computation complexity, in particular
+            # the permissions check may be expensive; see the bug
+            # MoinMoinBugs/GetSubscribersPerformanceProblem
+
             # This is a bit wrong if return_users=1 (which implies that the caller will process
             # user attributes and may, for example choose to send an SMS)
             # So it _should_ be "not (subscriber.email and return_users)" but that breaks at the moment.
             if not subscriber.email:
                 continue # skip empty email addresses
-            if trivial and not subscriber.want_trivial:
-                continue # skip uninterested subscribers
 
+            # skip people not subscribed
+            if not subscriber.isSubscribedTo(pageList):
+                continue
+
+            # skip people who can't read the page
             if not UserPerms(subscriber).read(self.page_name):
                 continue
 
-            if subscriber.isSubscribedTo(pageList):
-                lang = subscriber.language or request.cfg.language_default
-                if not lang in subscriber_list:
-                    subscriber_list[lang] = []
-                if return_users:
-                    subscriber_list[lang].append(subscriber)
-                else:
-                    subscriber_list[lang].append(subscriber.email)
+            # add the user to the list
+            lang = subscriber.language or request.cfg.language_default
+            if not lang in subscriber_list:
+                subscriber_list[lang] = []
+            if return_users:
+                subscriber_list[lang].append(subscriber)
+            else:
+                subscriber_list[lang].append(subscriber.email)
 
         return subscriber_list
 
@@ -892,11 +878,11 @@ class Page(object):
         from MoinMoin import security
         request = self.request
         pi = {} # we collect the processing instructions here
+
         # default language from cfg
         pi['language'] = self.cfg.language_default or "en"
 
         body = self.body
-
         # TODO: remove this hack once we have separate metadata and can use mimetype there
         if body.startswith('<?xml'): # check for XML content
             pi['lines'] = 0
@@ -912,7 +898,7 @@ class Page(object):
         pi['formatargs'] = ''
         pi['lines'] = len(meta)
         acl = []
-        
+
         for verb, args in meta:
             if verb == "format": # markup format
                 format, formatargs = (args + ' ').split(' ', 1)
@@ -946,7 +932,7 @@ class Page(object):
                         else:
                             url = Page(request, target).url(request)
                         pi['refresh'] = (delay, url)
-                    except (ValueError,):
+                    except (ValueError, ):
                         pass
 
             elif verb == "redirect":
@@ -955,6 +941,10 @@ class Page(object):
             elif verb == "deprecated":
                 pi['deprecated'] = True
 
+            elif verb == "openiduser":
+                if request.cfg.openid_server_enable_user:
+                    pi['openid.user'] = args
+
             elif verb == "pragma":
                 try:
                     key, val = args.split(' ', 1)
@@ -970,7 +960,7 @@ class Page(object):
         """ Output the raw page data (action=raw).
             With no content_disposition, the browser usually just displays the
             data on the screen, with content_disposition='attachment', it will
-            offer a dialogue to save it to disk (used by Save action).            
+            offer a dialogue to save it to disk (used by Save action).
         """
         request = self.request
         request.setHttpHeader("Content-type: text/plain; charset=%s" % config.charset)
@@ -995,16 +985,15 @@ class Page(object):
         request.emit_http_headers()
         request.write(text)
 
-    def send_page(self, msg=None, **keywords):
+    def send_page(self, **keywords):
         """ Output the formatted page.
 
         TODO: "kill send_page(), quick" (since 2002 :)
 
-        @param msg: if given, display message in header area
         @keyword content_only: if 1, omit http headers, page header and footer
         @keyword content_id: set the id of the enclosing div
         @keyword count_hit: if 1, add an event to the log
-        @keyword send_missing_page: if 1, assume that page to be sent is MissingPage
+        @keyword send_special: if True, this is a special page send
         @keyword omit_footnotes: if True, do not send footnotes (used by include macro)
         """
         request = self.request
@@ -1015,7 +1004,7 @@ class Page(object):
         omit_footnotes = keywords.get('omit_footnotes', 0)
         content_id = keywords.get('content_id', 'content')
         do_cache = keywords.get('do_cache', 1)
-        send_missing_page = keywords.get('send_missing_page', 0)
+        send_special = keywords.get('send_special', False)
         print_mode = keywords.get('print_mode', 0)
         if print_mode:
             media = 'media' in request.form and request.form['media'][0] or 'print'
@@ -1023,8 +1012,6 @@ class Page(object):
             media = 'screen'
         self.hilite_re = (keywords.get('hilite_re') or
                           request.form.get('highlight', [None])[0])
-        if msg is None:
-            msg = ""
 
         # count hit?
         if keywords.get('count_hit', 0):
@@ -1042,7 +1029,7 @@ class Page(object):
             request.http_redirect('%s/%s?action=show&redirect=%s' % (
                 request.getScriptname(),
                 wikiutil.quoteWikinameURL(pi['redirect']),
-                wikiutil.url_quote_plus(self.page_name, ''),))
+                wikiutil.url_quote_plus(self.page_name, ''), ))
             return
 
         # if necessary, load the formatter
@@ -1065,19 +1052,16 @@ class Page(object):
             except re.error, err:
                 if 'highlight' in request.form:
                     del request.form['highlight']
-                msg = _('Invalid highlighting regular expression "%(regex)s": %(error)s') % {
-                        'regex': self.hilite_re,
-                        'error': str(err),
-                }
+                request.theme.add_msg(_('Invalid highlighting regular expression "%(regex)s": %(error)s') % {
+                                          'regex': self.hilite_re,
+                                          'error': str(err),
+                                      }, "warning")
                 self.hilite_re = None
 
         if 'deprecated' in pi:
             # deprecated page, append last backup version to current contents
             # (which should be a short reason why the page is deprecated)
-            msg = '%s<strong>%s</strong><br>%s' % (
-                self.formatter.smiley('/!\\'),
-                _('The backed up content of this page is deprecated and will not be included in search results!', formatted=False),
-                msg)
+            request.theme.add_msg(_('The backed up content of this page is deprecated and will not be included in search results!'), "warning")
 
             revisions = self.getRevList()
             if len(revisions) >= 2: # XXX shouldn't that be ever the case!? Looks like not.
@@ -1088,13 +1072,16 @@ class Page(object):
         lang = self.pi.get('language', request.cfg.language_default)
         request.setContentLanguage(lang)
 
-        # start document output        
+        # start document output
         page_exists = self.exists()
         if not content_only:
             if emit_headers:
                 request.setHttpHeader("Content-Type: %s; charset=%s" % (self.output_mimetype, self.output_charset))
                 if page_exists:
-                    request.setHttpHeader('Status: 200 OK')
+                    if not request.user.may.read(self.page_name):
+                        request.setHttpHeader('Status: 403 Permission Denied')
+                    else:
+                        request.setHttpHeader('Status: 200 OK')
                     if not request.cacheable:
                         # use "nocache" headers if we're using a method that is not simply "display"
                         request.disableHttpCaching(level=2)
@@ -1121,24 +1108,22 @@ class Page(object):
             # send the page header
             if self.default_formatter:
                 if self.rev:
-                    msg = "<strong>%s</strong><br>%s" % (
-                        _('Revision %(rev)d as of %(date)s', formatted=False) % {
+                    request.theme.add_msg("<strong>%s</strong><br>" % (
+                        _('Revision %(rev)d as of %(date)s') % {
                             'rev': self.rev,
                             'date': self.mtime_printable(request)
-                        }, msg)
+                        }), "info")
 
                 # This redirect message is very annoying.
                 # Less annoying now without the warning sign.
                 if 'redirect' in request.form:
                     redir = request.form['redirect'][0]
-                    msg = '<strong>%s</strong><br>%s' % (
-                        _('Redirected from page "%(page)s"', formatted=False) % {'page':
-                            wikiutil.link_tag(request, wikiutil.quoteWikinameURL(redir) + "?action=show", self.formatter.text(redir))},
-                        msg)
+                    request.theme.add_msg('<strong>%s</strong><br>' % (
+                        _('Redirected from page "%(page)s"') % {'page':
+                            wikiutil.link_tag(request, wikiutil.quoteWikinameURL(redir) + "?action=show", self.formatter.text(redir))}), "info")
                 if 'redirect' in pi:
-                    msg = '<strong>%s</strong><br>%s' % (
-                        _('This page redirects to page "%(page)s"', formatted=False) % {'page': wikiutil.escape(pi['redirect'])},
-                        msg)
+                    request.theme.add_msg('<strong>%s</strong><br>' % (
+                        _('This page redirects to page "%(page)s"') % {'page': wikiutil.escape(pi['redirect'])}), "info")
 
                 # Page trail
                 trail = None
@@ -1148,28 +1133,59 @@ class Page(object):
 
                 title = self.split_title()
 
-                request.theme.send_title(title, page=self, msg=msg,
+                html_head = ''
+                if request.cfg.openid_server_enabled:
+                    openid_username = self.page_name
+                    userid = user.getUserId(request, openid_username)
+
+                    if userid is None and 'openid.user' in self.pi:
+                        openid_username = self.pi['openid.user']
+                        userid = user.getUserId(request, openid_username)
+
+                    if request.cfg.openid_server_restricted_users_group:
+                        request.dicts.addgroup(request,
+                                               request.cfg.openid_server_restricted_users_group)
+
+                    if userid is not None and not request.cfg.openid_server_restricted_users_group or \
+                      request.dicts.has_member(request.cfg.openid_server_restricted_users_group, openid_username):
+                        html_head = '<link rel="openid2.provider" href="%s">' % \
+                                        wikiutil.escape(request.getQualifiedURL(self.url(request,
+                                                                                querystr={'action': 'serveopenid'})), True)
+                        html_head += '<link rel="openid.server" href="%s">' % \
+                                        wikiutil.escape(request.getQualifiedURL(self.url(request,
+                                                                                querystr={'action': 'serveopenid'})), True)
+                        html_head += '<meta http-equiv="x-xrds-location" content="%s">' % \
+                                        wikiutil.escape(request.getQualifiedURL(self.url(request,
+                                                                                querystr={'action': 'serveopenid', 'yadis': 'ep'})), True)
+                    elif self.page_name == request.cfg.page_front_page:
+                        html_head = '<meta http-equiv="x-xrds-location" content="%s">' % \
+                                        wikiutil.escape(request.getQualifiedURL(self.url(request,
+                                                                                querystr={'action': 'serveopenid', 'yadis': 'idp'})), True)
+
+                request.theme.send_title(title, page=self,
                                     print_mode=print_mode,
                                     media=media, pi_refresh=pi.get('refresh'),
                                     allow_doubleclick=1, trail=trail,
+                                    html_head=html_head,
                                     )
 
-        # new page?
-        if not page_exists and (not content_only or (content_only
-                                                     and send_missing_page)):
-            if self.default_formatter and not content_only:
-                self._emptyPageText(request) # this recursively calls send_page
-            elif content_only and send_missing_page:
-                # We should send MissingPage but it is not there
-                import warnings
-                warnings.warn("Error - The page MissingPage or MissingHomePage could not be found."
-                              " Check your underlay directory setting.")
-                url = '%s?action=edit' % wikiutil.quoteWikinameURL(self.page_name)
-                request.write(wikiutil.link_tag(request, url, text=_("Create New Page", formatted=False),
-                                                formatter=self.formatter, rel='nofollow'))
-        elif not request.user.may.read(self.page_name):
-            request.write("<strong>%s</strong><br>" % _("You are not allowed to view this page.", formatted=False))
-        else:
+        # special pages handling, including denying access
+        special = None
+
+        if not send_special:
+            if not page_exists and not body:
+                special = 'missing'
+            elif not request.user.may.read(self.page_name):
+                special = 'denied'
+
+            # if we have a special page, output it, unless
+            #  - we should only output content (this is for say the pagelinks formatter)
+            #  - we have a non-default formatter
+            if special and not content_only and self.default_formatter:
+                self._specialPageText(request, special) # this recursively calls send_page
+
+        # if we didn't short-cut to a special page, output this page
+        if not special:
             # start wiki content div
             request.write(self.formatter.startContent(content_id))
 
@@ -1276,7 +1292,7 @@ class Page(object):
                 except Exception, e:
                     if not is_cache_exception(e):
                         raise
-                    request.log('page cache failed after creation')
+                    logging.error('page cache failed after creation')
                     self.format(parser)
 
         request.clock.stop('send_page_content')
@@ -1318,7 +1334,7 @@ class Page(object):
             # See http://docs.python.org/lib/module-marshal.html
             raise Exception('CacheNeedsUpdate')
         except Exception, err:
-            request.log('fail to load "%s" cache: %s' %
+            logging.info('failed to load "%s" cache: %s' %
                         (self.page_name, str(err)))
             raise Exception('CacheNeedsUpdate')
 
@@ -1340,22 +1356,40 @@ class Page(object):
                        self.page_name.encode(config.charset), 'exec')
         cache = caching.CacheEntry(request, self, self.getFormatterName(), scope='item')
         cache.update(marshal.dumps(code))
-        self.cache_mtime = cache.mtime()
         return code
 
-    def _emptyPageText(self, request):
+    def _specialPageText(self, request, special_type):
         """ Output the default page content for new pages.
 
         @param request: the request object
         """
-        if request.user.valid and request.user.name == self.page_name:
-            missingpage = wikiutil.getLocalizedPage(request, 'MissingHomePage')
+        _ = request.getText
+
+        if special_type == 'missing':
+            if request.user.valid and request.user.name == self.page_name and \
+               request.cfg.user_homewiki in ('Self', request.cfg.interwikiname):
+                page = wikiutil.getLocalizedPage(request, 'MissingHomePage')
+            else:
+                page = wikiutil.getLocalizedPage(request, 'MissingPage')
+
+            alternative_text = u"'''<<Action(action=edit, text=\"%s\")>>'''" % _('Create New Page')
+        elif special_type == 'denied':
+            page = wikiutil.getLocalizedPage(request, 'PermissionDeniedPage')
+            alternative_text = u"'''%s'''" % _('You are not allowed to view this page.')
         else:
-            missingpage = wikiutil.getLocalizedPage(request, 'MissingPage')
-        missingpagefn = missingpage._text_filename()
-        missingpage.page_name = self.page_name
-        missingpage._text_filename_force = missingpagefn
-        missingpage.send_page(content_only=1, send_missing_page=1)
+            assert False
+
+        special_exists = page.exists()
+
+        if special_exists:
+            page._text_filename_force = page._text_filename()
+        else:
+            page.body = alternative_text
+            logging.warn('The page "%s" could not be found. Check your'
+                         ' underlay directory setting.' % page.page_name)
+        page.page_name = self.page_name
+
+        page.send_page(content_only=True, do_cache=not special_exists, send_special=True)
 
 
     def getRevList(self):
@@ -1442,12 +1476,12 @@ class Page(object):
         return links
 
     def parsePageLinks(self, request):
-        """ Parse page links by formatting with a pagelinks formatter 
-        
+        """ Parse page links by formatting with a pagelinks formatter
+
         This is a old hack to get the pagelinks by rendering the page
         with send_page. We can remove this hack after factoring
         send_page and send_page_content into small reuseable methods.
-        
+
         More efficient now by using special pagelinks formatter and
         redirecting possible output into null file.
         """
@@ -1476,8 +1510,7 @@ class Page(object):
                 page = Page(request, pagename, formatter=formatter)
                 page.send_page(content_only=1)
             except:
-                import traceback
-                traceback.print_exc(200)
+                logging.exception("pagelinks formatter failed, traceback follows")
         finally:
             request.mode_getpagelinks -= 1
             #logging.debug("mode_getpagelinks == %r" % request.mode_getpagelinks)
@@ -1512,7 +1545,7 @@ class Page(object):
 
     def getACL(self, request):
         """ Get cached ACLs of this page.
-        
+
         Return cached ACL or invoke parseACL and update the cache.
 
         @param request: the request object
@@ -1540,7 +1573,7 @@ class Page(object):
                 if currentRevision != 99999999:
                     # don't use cache for non existing pages
                     # otherwise in the process of creating copies by filesys.copytree (PageEditor.copyPage)
-                    # the first may test will create a cache entry with the default_acls for a non existing page 
+                    # the first may test will create a cache entry with the default_acls for a non existing page
                     # At the time the page is created acls on that page would be ignored until the process
                     # is completed by adding a log entry into edit-log
                     cache_data = (currentRevision, acl)
@@ -1550,8 +1583,8 @@ class Page(object):
             return acl
 
     def parseACL(self):
-        """ Return ACLs parsed from the last available revision 
-        
+        """ Return ACLs parsed from the last available revision
+
         The effective ACL is always from the last revision, even if
         you access an older revision.
         """
@@ -1604,7 +1637,7 @@ class Page(object):
 
     def isConflict(self):
         """ Returns true if there is a known editing conflict for that page.
-        
+
         @return: true if there is a known conflict.
         """
 
@@ -1613,7 +1646,7 @@ class Page(object):
 
     def setConflict(self, state):
         """ Sets the editing conflict flag.
-        
+
         @param state: bool, true if there is a conflict.
         """
         cache = caching.CacheEntry(self.request, self, 'conflict', scope='item')
