@@ -24,7 +24,8 @@ from MoinMoin.parser.text_moin_wiki import Parser as WikiParser
 def execute(pagename, request):
     if not request.user.may.write(pagename):
         _ = request.getText
-        Page(request, pagename).send_page(msg=_('You are not allowed to edit this page.', formatted=False))
+        request.theme.add_msg_('You are not allowed to edit this page.', "error")
+        Page(request, pagename).send_page()
         return
 
     PageGraphicalEditor(request, pagename).sendEditor()
@@ -45,10 +46,7 @@ class PageGraphicalEditor(PageEditor.PageEditor):
         @keyword comment: comment field (when preview is true)
         """
         from MoinMoin import i18n
-        try:
-            from MoinMoin.action import SpellCheck
-        except ImportError:
-            SpellCheck = None
+        from MoinMoin.action import SpellCheck
 
         request = self.request
         form = request.form
@@ -65,27 +63,28 @@ class PageGraphicalEditor(PageEditor.PageEditor):
 
         # check edit permissions
         if not request.user.may.write(self.page_name):
-            msg = _('You are not allowed to edit this page.', formatted=False)
+            msg = _('You are not allowed to edit this page.')
         elif not self.isWritable():
-            msg = _('Page is immutable!', formatted=False)
+            msg = _('Page is immutable!')
         elif self.rev:
             # Trying to edit an old version, this is not possible via
             # the web interface, but catch it just in case...
-            msg = _('Cannot edit old revisions!', formatted=False)
+            msg = _('Cannot edit old revisions!')
         else:
             # try to acquire edit lock
             ok, edit_lock_message = self.lock.acquire()
             if not ok:
                 # failed to get the lock
                 if preview is not None:
-                    edit_lock_message = _('The lock you held timed out. Be prepared for editing conflicts!', formatted=False
+                    edit_lock_message = _('The lock you held timed out. Be prepared for editing conflicts!'
                         ) + "<br>" + edit_lock_message
                 else:
                     msg = edit_lock_message
 
         # Did one of the prechecks fail?
         if msg:
-            self.send_page(msg=msg)
+            request.theme.add_msg(msg, "error")
+            self.send_page()
             return
 
         # check if we want to load a draft
@@ -101,15 +100,15 @@ class PageGraphicalEditor(PageEditor.PageEditor):
 
         # Check for draft / normal / preview submit
         if use_draft is not None:
-            title = _('Draft of "%(pagename)s"', formatted=False)
+            title = _('Draft of "%(pagename)s"')
             # Propagate original revision
             rev = int(form['draft_rev'][0])
             self.set_raw_body(use_draft, modified=1)
             preview = use_draft
         elif preview is None:
-            title = _('Edit "%(pagename)s"', formatted=False)
+            title = _('Edit "%(pagename)s"')
         else:
-            title = _('Preview of "%(pagename)s"', formatted=False)
+            title = _('Preview of "%(pagename)s"')
             # Propagate original revision
             rev = request.rev
             self.set_raw_body(preview, modified=1)
@@ -117,9 +116,9 @@ class PageGraphicalEditor(PageEditor.PageEditor):
         # send header stuff
         lock_timeout = self.lock.timeout / 60
         lock_page = wikiutil.escape(self.page_name, quote=1)
-        lock_expire = _("Your edit lock on %(lock_page)s has expired!", formatted=False) % {'lock_page': lock_page}
-        lock_mins = _("Your edit lock on %(lock_page)s will expire in # minutes.", formatted=False) % {'lock_page': lock_page}
-        lock_secs = _("Your edit lock on %(lock_page)s will expire in # seconds.", formatted=False) % {'lock_page': lock_page}
+        lock_expire = _("Your edit lock on %(lock_page)s has expired!") % {'lock_page': lock_page}
+        lock_mins = _("Your edit lock on %(lock_page)s will expire in # minutes.") % {'lock_page': lock_page}
+        lock_secs = _("Your edit lock on %(lock_page)s will expire in # seconds.") % {'lock_page': lock_page}
 
         # get request parameters
         try:
@@ -134,12 +133,12 @@ class PageGraphicalEditor(PageEditor.PageEditor):
             if not self.exists():
                 # page does not exist, are we creating it?
                 if rev:
-                    conflict_msg = _('Someone else deleted this page while you were editing!', formatted=False)
+                    conflict_msg = _('Someone else deleted this page while you were editing!')
             elif rev != self.current_rev():
-                conflict_msg = _('Someone else changed this page while you were editing!', formatted=False)
+                conflict_msg = _('Someone else changed this page while you were editing!')
                 if self.mergeEditConflict(rev):
                     conflict_msg = _("""Someone else saved this page while you were editing!
-Please review the page and save then. Do not save this page as it is!""", formatted=False)
+Please review the page and save then. Do not save this page as it is!""")
                     rev = self.current_rev()
             if conflict_msg:
                 # We don't show preview when in conflict
@@ -159,26 +158,25 @@ Please review the page and save then. Do not save this page as it is!""", format
 
         # Get the text body for the editor field.
         # TODO: what about deleted pages? show the text of the last revision or use the template?
-        template_msg = None
         if preview is not None:
             raw_body = self.get_raw_body()
             if use_draft:
-                template_msg = _("[Content loaded from draft]", formatted=False) + '<br>'
+                request.write(_("[Content loaded from draft]"), '<br>')
         elif self.exists():
             # If the page exists, we get the text from the page.
             # TODO: maybe warn if template argument was ignored because the page exists?
             raw_body = self.get_raw_body()
         elif 'template' in form:
-            # If the page does not exists, we try to get the content from the template parameter.
+            # If the page does not exist, we try to get the content from the template parameter.
             template_page = wikiutil.unquoteWikiname(form['template'][0])
             if request.user.may.read(template_page):
                 raw_body = Page(request, template_page).get_raw_body()
                 if raw_body:
-                    template_msg = _("[Content of new page loaded from %s]", formatted=False) % (template_page, ) + '<br>'
+                    request.write(_("[Content of new page loaded from %s]") % (template_page, ), '<br>')
                 else:
-                    template_msg = _("[Template %s not found]", formatted=False) % (template_page, ) + '<br>'
+                    request.write(_("[Template %s not found]") % (template_page, ), '<br>')
             else:
-                template_msg = _("[You may not read %s]", formatted=False) % (template_page, ) + '<br>'
+                request.write(_("[You may not read %s]") % (template_page, ), '<br>')
 
         # Make backup on previews - but not for new empty pages
         if not use_draft and preview and raw_body:
@@ -194,18 +192,18 @@ Please review the page and save then. Do not save this page as it is!""", format
                     loadable_draft = True
                     page_rev = rev
                     draft_timestamp_str = request.user.getFormattedDateTime(draft_timestamp)
-                    draft_message = _(u"'''<<BR>>Your draft based on revision %(draft_rev)d (saved %(draft_timestamp_str)s) can be loaded instead of the current revision %(page_rev)d by using the load draft button - in case you lost your last edit somehow without saving it.''' A draft gets saved for you when you do a preview, cancel an edit or unsuccessfully save.", formatted=True, percent=True) % locals()
+                    draft_message = _(u"'''<<BR>>Your draft based on revision %(draft_rev)d (saved %(draft_timestamp_str)s) can be loaded instead of the current revision %(page_rev)d by using the load draft button - in case you lost your last edit somehow without saving it.''' A draft gets saved for you when you do a preview, cancel an edit or unsuccessfully save.", wiki=True, percent=True) % locals()
 
         # Setup status message
-        status = [kw.get('msg', ''), template_msg, conflict_msg, edit_lock_message, draft_message]
+        status = [kw.get('msg', ''), conflict_msg, edit_lock_message, draft_message]
         status = [msg for msg in status if msg]
         status = ' '.join(status)
         status = Status(request, content=status)
 
+        request.theme.add_msg(status, "error")
         request.theme.send_title(
             title % {'pagename': self.split_title(), },
             page=self,
-            msg=status,
             html_head=self.lock.locktype and (
                 PageEditor._countdown_js % {
                      'countdown_script': request.theme.externalScript('countdown'),
@@ -221,7 +219,7 @@ Please review the page and save then. Do not save this page as it is!""", format
 
         # Generate default content for new pages
         if not raw_body:
-            raw_body = _('Describe %s here.', formatted=False) % (self.page_name,)
+            raw_body = _('Describe %s here.') % (self.page_name, )
 
         # send form
         request.write('<form id="editor" method="post" action="%s/%s#preview">' % (
@@ -237,12 +235,12 @@ Please review the page and save then. Do not save this page as it is!""", format
         request.write(unicode(html.INPUT(type="hidden", name="action", value="edit")))
 
         # Send revision of the page our edit is based on
-        request.write('<input type="hidden" name="rev" value="%d">' % (rev,))
+        request.write('<input type="hidden" name="rev" value="%d">' % (rev, ))
 
         # Add src format (e.g. 'wiki') into a hidden form field, so that
         # we can load the correct converter after POSTing.
         request.write('<input type="hidden" name="format" value="%s">' % self.pi['format'])
-        
+
         # Create and send a ticket, so we can check the POST
         request.write('<input type="hidden" name="ticket" value="%s">' % wikiutil.createTicket(request))
 
@@ -252,17 +250,15 @@ Please review the page and save then. Do not save this page as it is!""", format
             request.write(unicode(html.INPUT(type="hidden", name="backto", value=backto)))
 
         # button bar
-        button_spellcheck = (SpellCheck and
-            '<input class="button" type="submit" name="button_spellcheck" value="%s">'
-                % _('Check Spelling', formatted=False)) or ''
+        button_spellcheck = '<input class="button" type="submit" name="button_spellcheck" value="%s">' % _('Check Spelling')
 
-        save_button_text = _('Save Changes', formatted=False)
-        cancel_button_text = _('Cancel', formatted=False)
+        save_button_text = _('Save Changes')
+        cancel_button_text = _('Cancel')
 
         if self.cfg.page_license_enabled:
             request.write('<p><em>', _(
 """By hitting '''%(save_button_text)s''' you put your changes under the %(license_link)s.
-If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.""", formatted=True, percent=True) % {
+If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.""", wiki=True) % {
                 'save_button_text': save_button_text,
                 'cancel_button_text': cancel_button_text,
                 'license_link': wikiutil.getLocalizedPage(request, self.cfg.page_license_page).link_to(request),
@@ -272,20 +268,20 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <input class="button" type="submit" name="button_save" value="%s">
 <input class="button" type="submit" name="button_preview" value="%s">
 <input class="button" type="submit" name="button_switch" value="%s">
-''' % (save_button_text, _('Preview', formatted=False), _('Text mode', formatted=False),))
+''' % (save_button_text, _('Preview'), _('Text mode'), ))
 
         if loadable_draft:
             request.write('''
 <input class="button" type="submit" name="button_load_draft" value="%s" onClick="flgChange = false;">
 <input type="hidden" name="draft_ts" value="%d">
 <input type="hidden" name="draft_rev" value="%d">
-''' % (_('Load Draft', formatted=False), draft_timestamp, draft_rev))
+''' % (_('Load Draft'), draft_timestamp, draft_rev))
 
         request.write('''
 %s
 <input class="button" type="submit" name="button_cancel" value="%s">
 <input type="hidden" name="editor" value="gui">
-''' % (button_spellcheck, cancel_button_text,))
+''' % (button_spellcheck, cancel_button_text, ))
         if self.cfg.mail_enabled:
             request.write('''
 <script type="text/javascript">
@@ -300,9 +296,10 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <input type="checkbox" name="trivial" id="chktrivialtop" value="1" %(checked)s onclick="toggle_trivial(this)">
 <label for="chktrivialtop">%(label)s</label>
 ''' % {
-                'checked': ('', 'checked')[form.get('trivial', ['0'])[0] == '1'],
-                'label': _("Trivial change", formatted=False),
-            })
+          'checked': ('', 'checked')[form.get('trivial', ['0'])[0] == '1'],
+          'label': _("Trivial change"),
+       })
+
         from MoinMoin.security.textcha import TextCha
         request.write(TextCha(request).render())
 
@@ -358,19 +355,19 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 </script>
 """)
         request.write("<p>")
-        request.write(_("Comment:", formatted=False),
+        request.write(_("Comment:"),
             ' <input id="editor-comment" type="text" name="comment" value="%s" size="80" maxlength="200">' % (
                 wikiutil.escape(kw.get('comment', ''), 1), ))
         request.write("</p>")
 
         # Category selection
-        filterfn = self.cfg.cache.page_category_regex.search
+        filterfn = self.cfg.cache.page_category_regexact.search
         cat_pages = request.rootpage.getPageList(filter=filterfn)
         cat_pages.sort()
         cat_pages = [wikiutil.pagelinkmarkup(p) for p in cat_pages]
-        cat_pages.insert(0, ('', _('<No addition>', formatted=False)))
+        cat_pages.insert(0, ('', _('<No addition>')))
         request.write("<p>")
-        request.write(_('Add to: %(category)s', formatted=False) % {
+        request.write(_('Add to: %(category)s') % {
             'category': unicode(web.makeSelection('category', cat_pages)),
         })
         if self.cfg.mail_enabled:
@@ -379,7 +376,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <input type="checkbox" name="trivial" id="chktrivial" value="1" %(checked)s onclick="toggle_trivial(this)">
 <label for="chktrivial">%(label)s</label> ''' % {
                 'checked': ('', 'checked')[form.get('trivial', ['0'])[0] == '1'],
-                'label': _("Trivial change", formatted=False),
+                'label': _("Trivial change"),
                 })
 
         request.write('''
@@ -388,16 +385,14 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <label for="chkrstrip">%(label)s</label>
 </p> ''' % {
             'checked': ('', 'checked')[form.get('rstrip', ['0'])[0] == '1'],
-            'label': _('Remove trailing whitespace from each line', formatted=False)
+            'label': _('Remove trailing whitespace from each line')
             })
 
         request.write("</p>")
 
         badwords_re = None
         if preview is not None:
-            if SpellCheck and (
-                    'button_spellcheck' in form or
-                    'button_newwords' in form):
+            if 'button_spellcheck' in form or 'button_newwords' in form:
                 badwords, badwords_re, msg = SpellCheck.checkSpelling(self, request, own_form=0)
                 request.write("<p>%s</p>" % msg)
         request.write('</fieldset>')
