@@ -12,7 +12,9 @@ import py
 from MoinMoin import wikiutil
 from MoinMoin.Page import Page
 from MoinMoin.PageEditor import PageEditor
-from MoinMoin._tests.common import gain_superuser_rights
+
+# TODO: check if and where we can use the helpers:
+from MoinMoin._tests import become_trusted, create_page, nuke_page
 
 class TestExpandVars(object):
     """PageEditor: testing page editor"""
@@ -168,6 +170,40 @@ class TestExpandPrivateVariables(TestExpandUserName):
         return page.getPagePath(use_underlay=0, check_create=0)
 
 
+class TestSave(object):
+
+    def setup_method(self, method):
+        self.old_handlers = self.request.cfg.event_handlers
+        become_trusted(self.request)
+
+    def teardown_method(self, method):
+        self.request.cfg.event_handlers = self.old_handlers
+
+    def testSaveAbort(self):
+        """Test if saveText() is interrupted if PagePreSave event handler returns Abort"""
+
+        def handler(event):
+            from MoinMoin.events import Abort
+            return Abort("This is just a test")
+
+        pagename = u'AutoCreatedMoinMoinTemporaryTestPageFortestSave'
+        testtext = u'ThisIsSomeStupidTestPageText!'
+
+        self.request.cfg.event_handlers = [handler]
+
+        page = Page(self.request, pagename)
+        if page.exists():
+            deleter = PageEditor(self.request, pagename)
+            deleter.deletePage()
+
+        editor = PageEditor(self.request, pagename)
+        editor.saveText(testtext, 0)
+
+        print "PageEditor can't save a page if Abort is returned from PreSave event handlers"
+        page = Page(self.request, pagename)
+        assert page.body != testtext
+
+
 class TestDictPageDeletion(object):
 
     def testCreateDictAndDeleteDictPage(self):
@@ -175,11 +211,10 @@ class TestDictPageDeletion(object):
         simple test if it is possible to delete a Dict page after creation
         """
         pagename = u'SomeDict'
-        gain_superuser_rights(self.request)
         page = PageEditor(self.request, pagename, do_editor_backup=0)
         body = u"This is an example text"
         page.saveText(body, 0)
-        
+
         success_i, result = page.deletePage()
 
         expected = u'Page "SomeDict" was successfully deleted!'
@@ -200,7 +235,7 @@ class TestCopyPage(object):
     def teardown_method(self, method):
         self.request.user.valid = self.savedValid
         self.deleteTestPage()
-        
+
     def createTestPage(self):
         """ Create temporary page, bypass logs, notification and backups
 
@@ -211,7 +246,7 @@ class TestCopyPage(object):
         import os
         path = Page(self.request, self.pagename).getPagePath(check_create=0)
         copy_path = Page(self.request, self.copy_pagename).getPagePath(check_create=0)
-        
+
         if os.path.exists(path) or os.path.exists(copy_path):
             self.shouldDeleteTestPage = False
             py.test.skip("%s or %s exists. Won't overwrite exiting page" % (self.pagename, self.copy_pagename))
@@ -221,7 +256,7 @@ class TestCopyPage(object):
             os.mkdir(revisionsDir)
             current = '00000001'
             file(os.path.join(path, 'current'), 'w').write('%s\n' % current)
-           
+
             file(os.path.join(revisionsDir, current), 'w').write(self.text)
         except Exception, err:
             py.test.skip("Can not be create test page: %s" % err)
@@ -241,7 +276,6 @@ class TestCopyPage(object):
         result, msg = PageEditor(self.request, self.pagename).copyPage(self.copy_pagename)
         revision = Page(self.request, self.copy_pagename).current_rev()
         assert result and revision is 2
-        
 
     def test_copy_page_acl_read(self):
         """
@@ -252,8 +286,7 @@ class TestCopyPage(object):
         result, msg = PageEditor(self.request, self.pagename).copyPage(self.copy_pagename)
         revision = Page(self.request, self.copy_pagename).current_rev()
         assert result and revision is 2
-        
-        
+
     def test_copy_page_acl_no_read(self):
         """
         Tests copying a page without read rights
@@ -263,6 +296,5 @@ class TestCopyPage(object):
         result, msg = PageEditor(self.request, self.pagename).copyPage(self.copy_pagename)
         revision = Page(self.request, self.copy_pagename).current_rev()
         assert result and revision is 2
-        
-coverage_modules = ['MoinMoin.PageEditor']
 
+coverage_modules = ['MoinMoin.PageEditor']
