@@ -1,13 +1,15 @@
 # -*- coding: iso-8859-1 -*-
 """
     MoinMoin - Despam action
-    
+
     Mass revert changes done by some specific author / bot.
 
     @copyright: 2005 by ???, Thomas Waldmann
     @license: GNU GPL, see COPYING for details.
 """
 
+DAYS = 30 # we look for spam edits in the last x days
+
 import time
 
 from MoinMoin.logfile import editlog
@@ -15,11 +17,24 @@ from MoinMoin.util.dataset import TupleDataset, Column
 from MoinMoin.widget.browser import DataBrowserWidget
 from MoinMoin import wikiutil, Page, PageEditor
 from MoinMoin.macro import RecentChanges
-from MoinMoin.formatter.text_html import Formatter
+
+def render(editor_tuple):
+    etype, evalue = editor_tuple
+    if etype in ('ip', 'email', ):
+        ret = evalue
+    elif etype == 'interwiki':
+        ewiki, euser = evalue
+        if ewiki == 'Self':
+            ret = euser
+        else:
+            ret = '%s:%s' % evalue
+    else:
+        ret = repr(editor_tuple)
+    return ret
 
 def show_editors(request, pagename, timestamp):
-    _ =  request.getText
-    
+    _ = request.getText
+
     timestamp = int(timestamp * 1000000)
     log = editlog.EditLog(request)
     editors = {}
@@ -27,17 +42,18 @@ def show_editors(request, pagename, timestamp):
     for line in log.reverse():
         if line.ed_time_usecs < timestamp:
             break
-        
+
         if not request.user.may.read(line.pagename):
             continue
-        
-        editor = line.getEditor(request)
+
+        editor = line.getInterwikiEditorData(request)
         if not line.pagename in pages:
             pages[line.pagename] = 1
             editors[editor] = editors.get(editor, 0) + 1
-            
+
     editors = [(nr, editor) for editor, nr in editors.iteritems()]
     editors.sort()
+    editors.reverse()
 
     pg = Page.Page(request, pagename)
 
@@ -46,8 +62,13 @@ def show_editors(request, pagename, timestamp):
                        Column('pages', label=_("Pages"), align='right'),
                        Column('link', label='', align='left')]
     for nr, editor in editors:
-        dataset.addRow((editor, unicode(nr), pg.link_to(request, text=_("Select Author"), querystr="action=Despam&editor=%s" % wikiutil.url_quote_plus(editor))))
-    
+        dataset.addRow((render(editor), unicode(nr),
+            pg.link_to(request, text=_("Select Author"),
+                querystr={
+                    'action': 'Despam',
+                    'editor': editor, # was: url_quote_plus()
+                })))
+
     table = DataBrowserWidget(request)
     table.setData(dataset)
     table.render()
@@ -56,28 +77,27 @@ class tmp:
     pass
 
 def show_pages(request, pagename, editor, timestamp):
-    _ =  request.getText
-    
+    _ = request.getText
+
     timestamp = int(timestamp * 1000000)
     log = editlog.EditLog(request)
-    lines = []
     pages = {}
     #  mimic macro object for use of RecentChanges subfunctions
     macro = tmp()
     macro.request = request
-    macro.formatter = Formatter(request)
+    macro.formatter = request.html_formatter
 
     request.write("<table>")
     for line in log.reverse():
         if line.ed_time_usecs < timestamp:
             break
-        
+
         if not request.user.may.read(line.pagename):
             continue
 
         if not line.pagename in pages:
             pages[line.pagename] = 1
-            if line.getEditor(request) == editor:
+            if repr(line.getInterwikiEditorData(request)) == editor:
                 line.time_tuple = request.user.getTime(wikiutil.version2timestamp(line.ed_time_usecs))
                 request.write(RecentChanges.format_page_edits(macro, [line], timestamp))
 
@@ -95,7 +115,7 @@ def show_pages(request, pagename, editor, timestamp):
 
 def revert_page(request, pagename, editor):
     if not request.user.may.revert(pagename):
-        return 
+        return
 
     log = editlog.EditLog(request, rootpagename=pagename)
 
@@ -104,14 +124,14 @@ def revert_page(request, pagename, editor):
     for line in log.reverse():
         if first:
             first = False
-            if line.getEditor(request) != editor:
+            if repr(line.getInterwikiEditorData(request)) != editor:
                 return
         else:
-            if line.getEditor(request) != editor:
+            if repr(line.getInterwikiEditorData(request)) != editor:
                 rev = line.rev
                 break
 
-    if rev == u"00000000": # page created by spammer 
+    if rev == u"00000000": # page created by spammer
         comment = u"Page deleted by Despam action"
         pg = PageEditor.PageEditor(request, pagename, do_editor_backup=0)
         try:
@@ -126,9 +146,9 @@ def revert_page(request, pagename, editor):
         except pg.SaveError, msg:
             savemsg = unicode(msg)
     return savemsg
-    
+
 def revert_pages(request, editor, timestamp):
-    _ =  request.getText
+    _ = request.getText
 
     editor = wikiutil.url_unquote(editor, want_unicode=False)
     timestamp = int(timestamp * 1000000)
@@ -144,37 +164,34 @@ def revert_pages(request, editor, timestamp):
 
         if not line.pagename in pages:
             pages[line.pagename] = 1
-            if line.getEditor(request) == editor:
+            if repr(line.getInterwikiEditorData(request)) == editor:
                 revertpages.append(line.pagename)
 
-    request.write("Debug: Pages to revert:<br>%s" % "<br>".join(revertpages))
+    request.write("Pages to revert:<br>%s" % "<br>".join(revertpages))
     for pagename in revertpages:
-        request.write("Debug: Begin reverting %s ...<br>" % pagename)
+        request.write("Begin reverting %s ...<br>" % pagename)
         msg = revert_page(request, pagename, editor)
         if msg:
             request.write("<p>%s: %s</p>" % (
                 Page.Page(request, pagename).link_to(request), msg))
-        request.write("Debug: Finished reverting %s.<br>" % pagename)
+        request.write("Finished reverting %s.<br>" % pagename)
 
 def execute(pagename, request):
     _ = request.getText
-    # be extra paranoid in dangerous actions
-    actname = __name__.split('.')[-1]
-    if actname in request.cfg.actions_excluded or \
-       not request.user.isSuperUser():
-        return Page.Page(request, pagename).send_page(request,
-            msg = _('You are not allowed to use this action.'))
+    # check for superuser
+    if not request.user.isSuperUser():
+        request.theme.add_msg(_('You are not allowed to use this action.'), "error")
+        return Page.Page(request, pagename).send_page()
 
     editor = request.form.get('editor', [None])[0]
-    timestamp = time.time() - 24 * 3600
-       # request.form.get('timestamp', [None])[0]
+    timestamp = time.time() - DAYS * 24 * 3600
     ok = request.form.get('ok', [0])[0]
 
-    request.http_headers()
-    wikiutil.send_title(request, "Despam", pagename=pagename)    
+    request.emit_http_headers()
+    request.theme.send_title("Despam", pagename=pagename)
     # Start content (important for RTL support)
     request.write(request.formatter.startContent("content"))
-    
+
     if ok:
         revert_pages(request, editor, timestamp)
     elif editor:
@@ -184,5 +201,6 @@ def execute(pagename, request):
 
     # End content and send footer
     request.write(request.formatter.endContent())
-    wikiutil.send_footer(request, pagename)
+    request.theme.send_footer(pagename)
+    request.theme.send_closing_html()
 
