@@ -52,13 +52,15 @@ class PackagePages:
 
         try:
             if not self.allowed():
-                raise ActionError(_('You are not allowed to edit this page.', formatted=False))
+                self.request.theme.add_msg(_('You are not allowed to edit this page.'), "error")
+                raise ActionError
             elif not self.page.exists():
-                raise ActionError(_('This page is already deleted or was never created!', formatted=False))
+                self.request.theme.add_msg(_('This page is already deleted or was never created!'))
+                raise ActionError
 
             self.package()
         except ActionError, e:
-            return self.page.send_page(msg=e.args[0])
+            return self.page.send_page()
 
     def package(self):
         """ Calls collectpackage() with the arguments specified. """
@@ -70,50 +72,57 @@ class PackagePages:
         packagename = form.get('packagename', [u''])[0]
 
         if not form.get('submit', [None])[0]:
-            raise ActionError(self.makeform())
+            self.request.theme.add_msg(self.makeform(), "dialog")
+            raise ActionError
 
         target = wikiutil.taintfilename(packagename)
 
         if not target:
-            raise ActionError(self.makeform(_('Invalid filename "%s"!', formatted=False) % wikiutil.escape(packagename)))
+            self.request.theme.add_msg(self.makeform(_('Invalid filename "%s"!') % wikiutil.escape(packagename)), "error")
+            raise ActionError
 
         # get directory, and possibly create it
         attach_dir = Page(self.request, self.page.page_name).getPagePath("attachments", check_create=1)
         fpath = os.path.join(attach_dir, target).encode(config.charset)
         if os.path.exists(fpath):
-            raise ActionError(_("Attachment '%(target)s' (remote name '%(filename)s') already exists.", formatted=False) % {
-                'target': wikiutil.escape(target), 'filename': wikiutil.escape(target)})
+            self.request.theme.add_msg(_("Attachment '%(target)s' (remote name '%(filename)s') already exists.") % {
+                'target': wikiutil.escape(target), 'filename': wikiutil.escape(target)}, "error")
+            raise ActionError
 
         # Generate a package
         output = open(fpath, "wb")
         package = self.collectpackage(unpackLine(pagelist, ","), output, target)
-        
+
         if package:
-            raise ActionError(self.makeform(package))
-        
+            self.request.theme.add_msg(self.makeform(), "dialog")
+            raise ActionError
+
         _addLogEntry(self.request, 'ATTNEW', self.pagename, target)
-        raise ActionError(_("Created the package %s containing the pages %s.", formatted=False) % (wikiutil.escape(target), wikiutil.escape(pagelist)))
+
+        self.request.theme.add_msg(_("Created the package %s containing the pages %s.") % (wikiutil.escape(target), wikiutil.escape(pagelist)))
+        raise ActionError
 
     def makeform(self, error=""):
-        """ Display a rename page form
+        """ Display a package page form
 
-        The form might contain an error that happened when trying to rename.
+        The form might contain an error that happened when package file was not given.
         """
         from MoinMoin.widget.dialog import Dialog
         _ = self.request.getText
 
-        error = u'<p class="error">%s</p>\n' % error
+        if error:
+            error = u'<p class="error">%s</p>\n' % error
 
         d = {
             'baseurl': self.request.getScriptname(),
             'error': error,
             'action': self.__class__.__name__,
-            'pagename': wikiutil.escape(self.pagename),
+            'pagename': wikiutil.escape(self.pagename, True),
             'pagename_quoted': wikiutil.quoteWikinameURL(self.pagename),
-            'package': _('Package pages', formatted=False),
-            'cancel': _('Cancel', formatted=False),
-            'newname_label': _("Package name", formatted=False),
-            'list_label': _("List of page names - separated by a comma", formatted=False),
+            'package': _('Package pages'),
+            'cancel': _('Cancel'),
+            'newname_label': _("Package name"),
+            'list_label': _("List of page names - separated by a comma"),
         }
         form = '''
 %(error)s
@@ -129,7 +138,7 @@ class PackagePages:
     <tr>
         <td class="label"><label>%(list_label)s</label></td>
         <td class="content">
-            <input type="text" name="pagelist" size="80" maxlength="200" value=%(pagename)s>
+            <input type="text" name="pagelist" size="80" maxlength="200" value="%(pagename)s">
         </td>
     </tr>
     <tr>
@@ -148,7 +157,7 @@ class PackagePages:
         """ Search MoinMoin for the string specified and return a list of
         matching pages, provided they are not system pages and not
         present in the underlay.
-        
+
         @param request: current request
         @param searchkey: string to search for
         @rtype: list
@@ -156,7 +165,7 @@ class PackagePages:
         """
 
         pagelist = searchPages(request, searchkey)
-        
+
         titles = []
         for title in pagelist.hits:
             if not wikiutil.isSystemPage(request, title.page_name) or not title.page.getPageStatus()[0]:
@@ -166,7 +175,7 @@ class PackagePages:
     def collectpackage(self, pagelist, fileobject, pkgname=""):
         """ Expects a list of pages as an argument, and fileobject to be an open
         file object, which a zipfile will get written to.
-        
+
         @param pagelist: pages to package
         @param fileobject: open file object to write to
         @param pkgname: optional file name, to prevent self packaging
@@ -174,7 +183,6 @@ class PackagePages:
         @return: error message, if one happened
         """
         _ = self.request.getText
-        form = self.request.form
         COMPRESSION_LEVEL = zipfile.ZIP_DEFLATED
 
         pages = []
@@ -185,7 +193,7 @@ class PackagePages:
                 if page.exists() and self.request.user.may.read(pagename):
                     pages.append(page)
         if not pages:
-            return (_('No pages like "%s"!', formatted=False) % wikiutil.escape(pagelist))
+            return (_('No pages like "%s"!') % wikiutil.escape(pagelist))
 
         # Set zipfile output
         zf = zipfile.ZipFile(fileobject, "w", COMPRESSION_LEVEL)
@@ -209,7 +217,7 @@ class PackagePages:
                     zipname = "%d_attachment" % cnt
                     script.append(packLine(["AddAttachment", zipname, attname, page.page_name, userid, "Created by the PackagePages action."]))
                     filename = AttachFile.getFilename(self.request, page.page_name, attname)
-                    zf.write(filename.encode("cp437"), zipname)
+                    zf.write(filename, zipname)
         script += [packLine(['Print', 'Thank you for using PackagePages!'])]
 
         zf.writestr(MOIN_PACKAGE_FILE, u"\n".join(script).encode("utf-8"))
