@@ -12,12 +12,6 @@ import re
 import traceback
 import StringIO # not relevant for speed, so we do not need cStringIO
 
-# Compatiblity to Python 2.3
-try:
-    set
-except NameError:
-    from sets import Set as set
-
 
 from MoinMoin import wikiutil
 from MoinMoin.packages import unpackLine, packLine
@@ -26,6 +20,7 @@ from MoinMoin.Page import Page
 from MoinMoin.wikidicts import Dict
 from MoinMoin.wikisync import TagStore, UnsupportedWikiException, SyncPage, NotAllowedException
 from MoinMoin.wikisync import MoinLocalWiki, MoinRemoteWiki, UP, DOWN, BOTH, MIMETYPE_MOIN
+from MoinMoin.support.python_compatibility import set
 from MoinMoin.util.bdiff import decompress, patch, compress, textdiff
 from MoinMoin.util import diff3, rpc_aggregator
 
@@ -107,7 +102,7 @@ class ActionClass(object):
             "password": None,
         }
 
-        options.update(Dict(self.request, self.pagename).get_dict())
+        options.update(Dict(self.request, self.pagename))
 
         # Convert page and group list strings to lists
         if options["pageList"] is not None:
@@ -142,16 +137,14 @@ class ActionClass(object):
 
     def show_password_form(self):
         _ = self.request.getText
-        d = {"message": _(r"Please enter your password of your account at the remote wiki below. <<BR>> /!\ You should trust both wikis because the password could be read by the particular administrators.", formatted=True),
-             "passwordlabel": _("Password", formatted=False),
-             "submit": _("Login", formatted=False),
-             "cancel": _("Cancel", formatted=False),
-             "baseurl": self.request.getScriptname(),
-             "pagename": wikiutil.quoteWikinameURL(self.pagename)
+        d = {"message": _(r"Please enter your password of your account at the remote wiki below. <<BR>> /!\ You should trust both wikis because the password could be read by the particular administrators.", wiki=True),
+             "passwordlabel": _("Password"),
+             "submit": _("Login"),
+             "cancel": _("Cancel"),
         }
         html_form = """
 %(message)s
-<form method="post" action="%(baseurl)s/%(pagename)s">
+<form method="post">
 <div>
 <input type="hidden" name="action" value="SyncPages">
 <label for="iPassword" style="font-weight: bold;">%(passwordlabel)s:</label>
@@ -167,7 +160,8 @@ class ActionClass(object):
 </div>
 </form>
 """ % d
-        self.page.send_page(msg=html_form)
+        self.request.theme.add_msg(html_form, "dialog")
+        self.page.send_page()
 
     def render(self):
         """ Render action
@@ -180,30 +174,30 @@ class ActionClass(object):
 
         try:
             if "cancel" in self.request.form:
-                raise ActionStatus(_("Operation was canceled.", formatted=False))
+                raise ActionStatus(_("Operation was canceled."), "error")
 
             if params["direction"] == UP:
-                raise ActionStatus(_("The only supported directions are BOTH and DOWN.", formatted=False))
+                raise ActionStatus(_("The only supported directions are BOTH and DOWN."), "error")
 
             if not self.request.cfg.interwikiname:
-                raise ActionStatus(_("Please set an interwikiname in your wikiconfig (see HelpOnConfiguration) to be able to use this action.", formatted=True))
+                raise ActionStatus(_("Please set an interwikiname in your wikiconfig (see HelpOnConfiguration) to be able to use this action.", wiki=True), "error")
 
             if not params["remoteWiki"]:
-                raise ActionStatus(_("Incorrect parameters. Please supply at least the ''remoteWiki'' parameter. Refer to HelpOnSynchronisation for help.", formatted=True))
+                raise ActionStatus(_("Incorrect parameters. Please supply at least the ''remoteWiki'' parameter. Refer to HelpOnSynchronisation for help.", wiki=True), "error")
 
             local = MoinLocalWiki(self.request, params["localPrefix"], params["pageList"])
             try:
                 remote = MoinRemoteWiki(self.request, params["remoteWiki"], params["remotePrefix"], params["pageList"], params["user"], params["password"], verbose=debug)
             except (UnsupportedWikiException, NotAllowedException), (msg, ):
-                raise ActionStatus(msg)
+                raise ActionStatus(msg, "error")
 
             if not remote.valid:
-                raise ActionStatus(_("The ''remoteWiki'' is unknown.", formatted=True))
+                raise ActionStatus(_("The ''remoteWiki'' is unknown.", wiki=True), "error")
             # if only the username is supplied, we ask for the password
             if params["user"] and not params["password"]:
                 return self.show_password_form()
         except ActionStatus, e:
-            msg = u'<p class="error">%s</p>\n' % (e.args[0], )
+            self.request.theme.add_msg(*e.args)
         else:
             try:
                 try:
@@ -211,19 +205,19 @@ class ActionClass(object):
                 except Exception, e:
                     temp_file = StringIO.StringIO()
                     traceback.print_exc(file=temp_file)
-                    self.log_status(self.ERROR, _("A severe error occured:", formatted=False), raw_suffix=temp_file.getvalue())
+                    self.log_status(self.ERROR, _("A severe error occurred:"), raw_suffix=temp_file.getvalue())
                     raise
                 else:
-                    msg = u"%s" % (_("Synchronisation finished. Look below for the status messages.", formatted=False), )
+                    self.request.theme.add_msg(u"%s" % (_("Synchronisation finished. Look below for the status messages."), ), "info")
             finally:
                 self.call_rollback_funcs()
                 # XXX aquire readlock on self.page
                 self.page.saveText(self.page.get_raw_body() + "\n\n" + self.generate_log_table(), 0)
                 # XXX release readlock on self.page
 
-                remote.destroy()
+                remote.delete_auth_token()
 
-        return self.page.send_page(msg=msg)
+        return self.page.send_page()
 
     def sync(self, params, local, remote):
         """ This method does the synchronisation work.
@@ -261,7 +255,7 @@ class ActionClass(object):
             match_direction = None
 
         local_full_iwid = packLine([local.get_iwid(), local.get_interwiki_name()])
-        remote_full_iwid = packLine([remote.get_iwid(), remote.get_interwiki_name()])
+        remote_full_iwid = remote.iwid_full
 
         self.log_status(self.INFO, _("Synchronisation started -"), raw_suffix=" <<DateTime(%s)>>" % self.page._get_local_timestamp())
 
@@ -275,7 +269,7 @@ class ActionClass(object):
 
         m_pages = [elem.add_missing_pagename(local, remote) for elem in SyncPage.merge(l_pages, r_pages)]
 
-        self.log_status(self.INFO, _("Got a list of %s local and %s remote pages. This results in %s different pages over-all."),
+        self.log_status(self.INFO, _("Got a list of %s local and %s remote pages. This results in %s pages to process."),
                         (str(len(l_pages)), str(len(r_pages)), str(len(m_pages))))
 
         if params["pageMatch"]:
