@@ -3,13 +3,18 @@
     MoinMoin caching module
 
     @copyright: 2001-2004 by Juergen Hermann <jh@web.de>,
-                2006-2007 MoinMoin:ThomasWaldmann
+                2006-2008 MoinMoin:ThomasWaldmann,
+                2008 MoinMoin:ThomasPfaff
     @license: GNU GPL, see COPYING for details.
 """
 
 import os
+import shutil
 import tempfile
 
+from MoinMoin import log
+logging = log.getLogger(__name__)
+
 from MoinMoin import config
 from MoinMoin.util import filesys, lock, pickle, PICKLE_PROTOCOL
 
@@ -18,6 +23,7 @@ class CacheError(Exception):
     """ raised if we have trouble reading or writing to the cache """
     pass
 
+
 def get_arena_dir(request, arena, scope):
     if scope == 'page_or_wiki': # XXX DEPRECATED, remove later
         if isinstance(arena, str):
@@ -66,30 +72,55 @@ class CacheEntry:
         self.arena_dir = get_arena_dir(request, arena, scope)
         if not os.path.exists(self.arena_dir):
             os.makedirs(self.arena_dir)
+        self._fname = os.path.join(self.arena_dir, key)
+
         if self.locking:
             self.lock_dir = os.path.join(self.arena_dir, '__lock__')
             self.rlock = lock.LazyReadLock(self.lock_dir, 60.0)
             self.wlock = lock.LazyWriteLock(self.lock_dir, 60.0)
 
+        # used by file-like api:
+        self._lock = None  # either self.rlock or self.wlock
+        self._fileobj = None  # open cache file object
+        self._tmp_fname = None  # name of temporary file (used for write)
+        self._mode = None  # mode of open file object
+
+
     def _filename(self):
-        return os.path.join(self.arena_dir, self.key)
+        # DEPRECATED - please use file-like api
+        return self._fname
 
     def exists(self):
-        return os.path.exists(self._filename())
+        return os.path.exists(self._fname)
 
     def mtime(self):
+        # DEPRECATED for checking a changed on-disk cache, please use
+        # self.uid() for this, see below
         try:
-            return os.path.getmtime(self._filename())
+            return os.path.getmtime(self._fname)
         except (IOError, OSError):
             return 0
 
+    def size(self):
+        try:
+            return os.path.getsize(self._fname)
+        except (IOError, OSError):
+            return 0
+
+    def uid(self):
+        """ Return a value that likely changes when the on-disk cache was updated.
+
+            See docstring of MoinMoin.util.filesys.fuid for details.
+        """
+        return filesys.fuid(self._fname)
+
     def needsUpdate(self, filename, attachdir=None):
         # following code is not necessary. will trigger exception and give same result
         #if not self.exists():
         #    return 1
 
         try:
-            ctime = os.path.getmtime(self._filename())
+            ctime = os.path.getmtime(self._fname)
             ftime = os.path.getmtime(filename)
         except os.error:
             return 1
@@ -106,73 +137,124 @@ class CacheEntry:
 
         return needsupdate
 
-#    def copyto(self, filename):
-#        # currently unused function
-#        import shutil
-#        tmpfname = self._tmpfilename()
-#        fname = self._filename()
-#        if not self.locking or self.locking and self.wlock.acquire(1.0):
-#            try:
-#                shutil.copyfile(filename, tmpfname)
-#                # this is either atomic or happening with real locks set:
-#                filesys.rename(tmpfname, fname)
-#            finally:
-#                if self.locking:
-#                    self.wlock.release()
-#        else:
-#            self.request.log("Can't acquire write lock in %s" % self.lock_dir)
+    def _determine_locktype(self, mode):
+        """ return the correct lock object for a specific file access mode """
+        if self.locking:
+            if 'r' in mode:
+                lock = self.rlock
+            if 'w' in mode or 'a' in mode:
+                lock = self.wlock
+        else:
+            lock = None
+        return lock
 
-    def update(self, content):
-        try:
-            fname = self._filename()
-            if self.use_pickle:
-                content = pickle.dumps(content, PICKLE_PROTOCOL)
-            elif self.use_encode:
-                content = content.encode(config.charset)
-            if not self.locking or self.locking and self.wlock.acquire(1.0):
-                try:
+    # file-like interface ----------------------------------------------------
+
+    def open(self, filename=None, mode='r', bufsize=-1):
+        """ open the cache for reading/writing
+
+        @param filename: must be None (default - automatically determine filename)
+        @param mode: 'r' (read, default), 'w' (write)
+                     Note: if mode does not include 'b' (binary), it will be
+                           automatically changed to include 'b'.
+        @param bufsize: size of read/write buffer (default: -1 meaning automatic)
+        @return: None (the opened file object is kept in self._fileobj and used
+                 implicitely by read/write/close functions of CacheEntry object.
+        """
+        assert self._fileobj is None, 'caching: trying to open an already opened cache'
+        assert filename is None, 'caching: giving a filename is not supported (yet?)'
+
+        self._lock = self._determine_locktype(mode)
+
+        if 'b' not in mode:
+            mode += 'b'  # we want to use binary mode, ever!
+        self._mode = mode  # for self.close()
+
+        if not self.locking or self.locking and self._lock.acquire(1.0):
+            try:
+                if 'r' in mode:
+                    self._fileobj = open(self._fname, mode, bufsize)
+                elif 'w' in mode:
                     # we do not write content to old inode, but to a new file
                     # so we don't need to lock when we just want to read the file
                     # (at least on POSIX, this works)
-                    tmp_handle, tmp_fname = tempfile.mkstemp('.tmp', self.key, self.arena_dir)
-                    os.write(tmp_handle, content)
-                    os.close(tmp_handle)
-                    # this is either atomic or happening with real locks set:
-                    filesys.rename(tmp_fname, fname)
-                    filesys.chmod(fname, 0666 & config.umask) # fix mode that mkstemp chose
+                    fd, self._tmp_fname = tempfile.mkstemp('.tmp', self.key, self.arena_dir)
+                    self._fileobj = os.fdopen(fd, mode, bufsize)
+                else:
+                    raise ValueError("caching: mode does not contain 'r' or 'w'")
+            finally:
+                if self.locking:
+                    self._lock.release()
+                    self._lock = None
+        else:
+            logging.error("Can't acquire read/write lock in %s" % self.lock_dir)
+
+
+    def read(self, size=-1):
+        """ read data from cache file
+
+        @param size: how many bytes to read (default: -1 == everything)
+        @return: read data (str)
+        """
+        return self._fileobj.read(size)
+
+    def write(self, data):
+        """ write data to cache file
+
+        @param data: write data (str)
+        """
+        self._fileobj.write(data)
+
+    def close(self):
+        """ close cache file (and release lock, if any) """
+        if self._fileobj:
+            self._fileobj.close()
+            self._fileobj = None
+            if 'w' in self._mode:
+                filesys.chmod(self._tmp_fname, 0666 & config.umask) # fix mode that mkstemp chose
+                # this is either atomic or happening with real locks set:
+                filesys.rename(self._tmp_fname, self._fname)
+
+        if self._lock:
+            if self.locking:
+                self._lock.release()
+            self._lock = None
+
+    # ------------------------------------------------------------------------
+
+    def update(self, content):
+        try:
+            if hasattr(content, 'read'):
+                # content is file-like
+                assert not (self.use_pickle or self.use_encode), 'caching: use_pickle and use_encode not supported with file-like api'
+                try:
+                    self.open(mode='w')
+                    shutil.copyfileobj(content, self)
                 finally:
-                    if self.locking:
-                        self.wlock.release()
+                    self.close()
             else:
-                self.request.log("Can't acquire write lock in %s" % self.lock_dir)
+                # content is a string
+                if self.use_pickle:
+                    content = pickle.dumps(content, PICKLE_PROTOCOL)
+                elif self.use_encode:
+                    content = content.encode(config.charset)
+
+                try:
+                    self.open(mode='w')
+                    self.write(content)
+                finally:
+                    self.close()
         except (pickle.PicklingError, OSError, IOError, ValueError), err:
             raise CacheError(str(err))
 
-    def remove(self):
-        if not self.locking or self.locking and self.wlock.acquire(1.0):
-            try:
-                try:
-                    os.remove(self._filename())
-                except OSError:
-                    pass
-            finally:
-                if self.locking:
-                    self.wlock.release()
-        else:
-            self.request.log("Can't acquire write lock in %s" % self.lock_dir)
-
     def content(self):
+        # no file-like api yet, we implement it when we need it
         try:
-            if not self.locking or self.locking and self.rlock.acquire(1.0):
-                try:
-                    f = open(self._filename(), 'rb')
-                    data = f.read()
-                    f.close()
-                finally:
-                    if self.locking:
-                        self.rlock.release()
-            else:
-                self.request.log("Can't acquire read lock in %s" % self.lock_dir)
+            try:
+                self.open(mode='r')
+                data = self.read()
+            finally:
+                self.close()
             if self.use_pickle:
                 data = pickle.loads(data)
             elif self.use_encode:
@@ -181,3 +263,17 @@ class CacheEntry:
         except (pickle.UnpicklingError, IOError, EOFError, ValueError), err:
             raise CacheError(str(err))
 
+    def remove(self):
+        if not self.locking or self.locking and self.wlock.acquire(1.0):
+            try:
+                try:
+                    os.remove(self._fname)
+                except OSError:
+                    pass
+            finally:
+                if self.locking:
+                    self.wlock.release()
+        else:
+            logging.error("Can't acquire write lock in %s" % self.lock_dir)
+
+
