@@ -3,7 +3,7 @@
     MoinMoin - Multiple configuration handler and Configuration defaults class
 
     @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
-                2005-2006 MoinMoin:ThomasWaldmann.
+                2005-2008 MoinMoin:ThomasWaldmann.
     @license: GNU GPL, see COPYING for details.
 """
 
@@ -11,10 +11,17 @@ import re
 import os
 import sys
 import time
-import logging
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin import config, error, util, wikiutil
 import MoinMoin.auth as authmodule
+import MoinMoin.events as events
+from MoinMoin.events import PageChangedEvent, PageRenamedEvent
+from MoinMoin.events import PageDeletedEvent, PageCopiedEvent
+from MoinMoin.events import PageRevertedEvent, FileAttachedEvent
+from MoinMoin import session
 from MoinMoin.packages import packLine
 from MoinMoin.security import AccessControlList
 from MoinMoin.support.python_compatibility import set
@@ -26,10 +33,10 @@ _config_cache = {}
 
 def _importConfigModule(name):
     """ Import and return configuration module and its modification time
-    
+
     Handle all errors except ImportError, because missing file is not
     always an error.
-    
+
     @param name: module name
     @rtype: tuple
     @return: module, modification time
@@ -40,6 +47,7 @@ def _importConfigModule(name):
     except ImportError:
         raise
     except IndentationError, err:
+        logging.exception('Your source code / config file is not correctly indented!')
         msg = '''IndentationError: %(err)s
 
 The configuration files are python modules. Therefore, whitespace is
@@ -50,6 +58,7 @@ You have to use four spaces at the beginning of the line mostly.
 }
         raise error.ConfigurationError(msg)
     except Exception, err:
+        logging.exception('An exception happened.')
         msg = '%s: %s' % (err.__class__.__name__, str(err))
         raise error.ConfigurationError(msg)
     return module, mtime
@@ -68,17 +77,24 @@ def _url_re_list():
     if _url_re_cache is None:
         try:
             farmconfig, _farmconfig_mtime = _importConfigModule('farmconfig')
-        except ImportError:
-            # Default to wikiconfig for all urls.
-            _farmconfig_mtime = 0
-            _url_re_cache = [('wikiconfig', re.compile(r'.')), ] # matches everything
+        except ImportError, err:
+            if 'farmconfig' in str(err):
+                # we failed importing farmconfig
+                logging.debug("could not import farmconfig, mapping all URLs to wikiconfig")
+                _farmconfig_mtime = 0
+                _url_re_cache = [('wikiconfig', re.compile(r'.')), ] # matches everything
+            else:
+                # maybe there was a failing import statement inside farmconfig
+                raise
         else:
+            logging.info("using farm config: %s" % os.path.abspath(farmconfig.__file__))
             try:
                 cache = []
                 for name, regex in farmconfig.wikis:
                     cache.append((name, re.compile(regex)))
                 _url_re_cache = cache
             except AttributeError:
+                logging.error("required 'wikis' list missing in farmconfig")
                 msg = """
 Missing required 'wikis' list in 'farmconfig.py'.
 
@@ -90,7 +106,7 @@ use wikiconfig.py.
 
 
 def _makeConfig(name):
-    """ Create and return a config instance 
+    """ Create and return a config instance
 
     Timestamp config with either module mtime or farmconfig mtime. This
     mtime can be used later to invalidate older caches.
@@ -105,7 +121,9 @@ def _makeConfig(name):
         configClass = getattr(module, 'Config')
         cfg = configClass(name)
         cfg.cfg_mtime = max(mtime, _farmconfig_mtime)
+        logging.info("using wiki config: %s" % os.path.abspath(module.__file__))
     except ImportError, err:
+        logging.exception('Could not import.')
         msg = '''ImportError: %(err)s
 
 Check that the file is in the same directory as the server script. If
@@ -121,6 +139,7 @@ module name does not include the ".py" suffix.
 }
         raise error.ConfigurationError(msg)
     except AttributeError, err:
+        logging.exception('An exception occured.')
         msg = '''AttributeError: %(err)s
 
 Could not find required "Config" class in "%(name)s.py".
@@ -140,6 +159,25 @@ use the wikiconfig.py file from the distribution.
     'err': err,
 }
         raise error.ConfigurationError(msg)
+
+    # postprocess configuration
+    # 'setuid' special auth method auth method can log out
+    cfg.auth_can_logout = ['setuid']
+    cfg.auth_login_inputs = []
+    found_names = []
+    for auth in cfg.auth:
+        if not auth.name:
+            raise error.ConfigurationError("Auth methods must have a name.")
+        if auth.name in found_names:
+            raise error.ConfigurationError("Auth method names must be unique.")
+        found_names.append(auth.name)
+        if auth.logout_possible and auth.name:
+            cfg.auth_can_logout.append(auth.name)
+        for input in auth.login_inputs:
+            if not input in cfg.auth_login_inputs:
+                cfg.auth_login_inputs.append(input)
+    cfg.auth_have_login = len(cfg.auth_login_inputs) > 0
+
     return cfg
 
 
@@ -183,24 +221,38 @@ class CacheClass:
     pass
 
 
-class DefaultConfig:
-    """ default config values """
+class DefaultConfig(object):
+    """ default config values
 
-    # setting DesktopEdition = True gives all local users special powers - ONLY use for MMDE style usage!
-    DesktopEdition = False
+        When adding new config attributes, PLEASE use a name with the TOPIC as prefix,
+        so it will sort naturally. E.g. use "actions_excluded", not "excluded_actions".
 
+        Also, please keep it (roughly) sorted (except if you have good reasons to group otherwise).
+    """
+
+    DesktopEdition = False # True gives all local users special powers - ONLY use for MMDE style usage!
+
+    SecurityPolicy = None
+
+    acl_hierarchic = False # True to use hierarchical ACLs
     # All acl_rights_* lines must use unicode!
     acl_rights_default = u"Trusted:read,write,delete,revert Known:read,write,delete,revert All:read,write"
     acl_rights_before = u""
     acl_rights_after = u""
     acl_rights_valid = ['read', 'write', 'delete', 'revert', 'admin']
-    acl_hierarchic = False
 
-    actions_excluded = ['xmlrpc'] # ['DeletePage', 'AttachFile', 'RenamePage', 'test', ]
+    actions_excluded = ['xmlrpc',  # we do not want wiki admins unknowingly offering xmlrpc service
+                        'MyPages',  # only works when used with a non-default SecurityPolicy (e.g. autoadmin)
+                        'CopyPage',  # has questionable behaviour regarding subpages a user can't read, but can copy
+                       ]
     allow_xslt = False
     antispam_master_url = "http://master.moinmo.in/?action=xmlrpc2"
-    attachments = None # {'dir': path, 'url': url-prefix}
-    auth = [authmodule.moin_login, authmodule.moin_session, ]
+
+    auth = [authmodule.MoinAuth()]
+    # default to http and xmlrpc_applytoken to get old semantics
+    # xmlrpc_applytoken shall be removed once that code is changed
+    # to have proper session handling and use request.handle_auth()
+    auth_methods_trusted = ['http', 'xmlrpc_applytoken']
 
     backup_compression = 'gz'
     backup_users = []
@@ -239,16 +291,16 @@ class DefaultConfig:
     default_markup = 'wiki'
     docbook_html_dir = r"/usr/share/xml/docbook/stylesheet/nwalsh/html/" # correct for debian sarge
 
-    edit_bar = ['Edit', 'Comments', 'Discussion', 'Info', 'Subscribe', 'Quicklink', 'Attachments', 'ActionsMenu'] 
+    edit_bar = ['Edit', 'Comments', 'Discussion', 'Info', 'Subscribe', 'Quicklink', 'Attachments', 'ActionsMenu']
     editor_default = 'text' # which editor is called when nothing is specified
+    editor_force = False # force using the default editor
     editor_ui = 'freechoice' # which editor links are shown on user interface
-    editor_force = False
     editor_quickhelp = {
         # editor markup hints quickhelp
         # MUST be in wiki markup, even if the help is not for the wiki parser!
-        'wiki': _("""\
+        'wiki': _(u"""\
  Emphasis:: <<Verbatim('')>>''italics''<<Verbatim('')>>; <<Verbatim(''')>>'''bold'''<<Verbatim(''')>>; <<Verbatim(''''')>>'''''bold italics'''''<<Verbatim(''''')>>; <<Verbatim('')>>''mixed ''<<Verbatim(''')>>'''''bold'''<<Verbatim(''')>> and italics''<<Verbatim('')>>; <<Verbatim(----)>> horizontal rule.
- Headings:: <<Verbatim(=)>> Title 1 <<Verbatim(=)>>; <<Verbatim(==)>> Title 2 <<Verbatim(==)>>; <<Verbatim(===)>> Title 3 <<Verbatim(===)>>;   <<Verbatim(====)>> Title 4 <<Verbatim(====)>>; <<Verbatim(=====)>> Title 5 <<Verbatim(=====)>>.
+ Headings:: = Title 1 =; == Title 2 ==; === Title 3 ===; ==== Title 4 ====; ===== Title 5 =====.
  Lists:: space and one of: * bullets; 1., a., A., i., I. numbered items; 1.#n start numbering at n; space alone indents.
  Links:: <<Verbatim(JoinCapitalizedWords)>>; <<Verbatim([[target|linktext]])>>.
  Tables:: || cell text |||| cell text spanning 2 columns ||;    no trailing white space allowed after tables or titles.
@@ -262,16 +314,27 @@ Emphasis: *italic* **bold** ``monospace``
 Headings: Heading 1  Heading 2  Heading 3
           =========  ---------  ~~~~~~~~~
 
-Horizontal rule: ---- 
+Horizontal rule: ----
 
-Links: TrailingUnderscore_ `multi word with backticks`_ external_ 
+Links: TrailingUnderscore_ `multi word with backticks`_ external_
 
 .. _external: http://external-site.example.org/foo/
 
 Lists: * bullets; 1., a. numbered items.
 }}}
-(!) For more help, see the 
+(!) For more help, see the
 [[http://docutils.sourceforge.net/docs/user/rst/quickref.html|reStructuredText Quick Reference]].
+"""),
+        'creole': _(u"""\
+ Emphasis:: <<Verbatim(//)>>''italics''<<Verbatim(//)>>; <<Verbatim(**)>>'''bold'''<<Verbatim(**)>>; <<Verbatim(**//)>>'''''bold italics'''''<<Verbatim(//**)>>; <<Verbatim(//)>>''mixed ''<<Verbatim(**)>>'''''bold'''<<Verbatim(**)>> and italics''<<Verbatim(//)>>;
+ Horizontal Rule:: <<Verbatim(----)>>
+ Force Linebreak:: <<Verbatim(\\\\)>>
+ Headings:: = Title 1 =; == Title 2 ==; === Title 3 ===; ==== Title 4 ====; ===== Title 5 =====.
+ Lists:: * bullets; ** sub-bullets; # numbered items; ## numbered sub items.
+ Links:: <<Verbatim([[target]])>>; <<Verbatim([[target|linktext]])>>.
+ Tables:: |= header text | cell text | more cell text |;
+
+(!) For more help, see HelpOnEditing or HelpOnCreoleSyntax.
 """),
     }
     edit_locking = 'warn 10' # None, 'warn <timeout mins>', 'lock <timeout mins>'
@@ -286,6 +349,8 @@ Lists: * bullets; 1., a. numbered items.
                #   if cfg.hacks.get('feature', False): <doit>
                # A non-existing hack key should ever mean False, None, "", [] or {}!
 
+    history_count = (100, 200) # (default_revisions_shown, max_revisions_shown)
+
     hosts_deny = []
 
     html_head = ''
@@ -295,89 +360,34 @@ Lists: * bullets; 1., a. numbered items.
     html_head_normal  = '''<meta name="robots" content="index,nofollow">\n'''
     html_pagetitle = None
 
+    interwikiname = None # our own interwikiname. choose wisely and never change!
     interwiki_preferred = [] # list of wiki names to show at top of interwiki list
 
     language_default = 'en'
     language_ignore_browser = False # ignore browser settings, use language_default
                                     # or user prefs
 
-    # ldap / active directory server URI
-    # use ldaps://server:636 url for ldaps,
-    # use  ldap://server for ldap without tls (and set ldap_start_tls to 0),
-    # use  ldap://server for ldap with tls (and set ldap_start_tls to 1 or 2).
-    ldap_uri = 'ldap://localhost'
-
-    # We can either use some fixed user and password for binding to LDAP.
-    # Be careful if you need a % char in those strings - as they are used as
-    # a format string, you have to write %% to get a single % in the end.
-    #ldap_binddn = 'binduser@example.org' # (AD)
-    #ldap_binddn = 'cn=admin,dc=example,dc=org' # (OpenLDAP)
-    #ldap_bindpw = 'secret'
-    # or we can use the username and password we got from the user:
-    #ldap_binddn = '%(username)s@example.org' # DN we use for first bind (AD)
-    #ldap_bindpw = '%(password)s' # password we use for first bind
-    # or we can bind anonymously (if that is supported by your directory).
-    # In any case, ldap_binddn and ldap_bindpw must be defined.
-    ldap_binddn = ''
-    ldap_bindpw = ''
-
-    # base DN we use for searching
-    #ldap_base = 'ou=SOMEUNIT,dc=example,dc=org'
-    ldap_base = ''
-
-    # scope of the search we do (2 == ldap.SCOPE_SUBTREE)
-    ldap_scope = 2 # we do not want to import ldap for everybody just for that
-
-    # LDAP REFERRALS
-    ldap_referrals = 0 # (0 needed for AD)
-
-    # ldap filter used for searching:
-    #ldap_filter = '(sAMAccountName=%(username)s)' # (AD)
-    ldap_filter = '(uid=%(username)s)' # (OpenLDAP)
-    # you can also do more complex filtering like:
-    # "(&(cn=%(username)s)(memberOf=CN=WikiUsers,OU=Groups,DC=example,DC=org))"
-
-    # some attribute names we use to extract information from LDAP:
-    ldap_givenname_attribute = None # ('givenName') ldap attribute we get the first name from
-    ldap_surname_attribute = None # ('sn') ldap attribute we get the family name from
-    ldap_aliasname_attribute = None # ('displayName') ldap attribute we get the aliasname from
-    ldap_email_attribute = None # ('mail') ldap attribute we get the email address from
-    ldap_email_callback = None # called to make up email address
-
-    ldap_coding = 'utf-8' # coding used for ldap queries and result values
-    ldap_timeout = 10 # how long we wait for the ldap server [s]
-    ldap_verbose = True # if True, put lots of LDAP debug info into the log
-
-    # TLS / SSL related defaults
-    ldap_start_tls = 0 # 0 = No, 1 = Try, 2 = Required
-    ldap_tls_cacertdir = ''
-    ldap_tls_cacertfile = ''
-    ldap_tls_certfile = ''
-    ldap_tls_keyfile = ''
-    ldap_tls_require_cert = 0 # 0 == ldap.OPT_X_TLS_NEVER (needed for self-signed certs)
-
-    ldap_bindonce = False # set to True to only do one bind.  Useful if
-                          # configured to bind as the user on the first attempt
+    logo_string = None # can be either just some text or a piece of html shown as "logo"
 
     log_reverse_dns_lookups = True  # if we do reverse dns lookups for logging hostnames
                                     # instead of just IPs
-    log_timing = False              # update <data_dir>/timing.log?
+    log_timing = False # log infos about timing of actions, good to analyze load conditions
 
-    mail_login = None # or "user pwd" if you need to use SMTP AUTH
-    mail_sendmail = None # "/usr/sbin/sendmail -t -i" to not use SMTP, but sendmail
-    mail_smarthost = None
     mail_from = None # u'Juergen Wiki <noreply@jhwiki.org>'
+    mail_login = None # "user pwd" if you need to use SMTP AUTH when using your mail server
+    mail_smarthost = None # your SMTP mail server
+    mail_sendmail = None # "/usr/sbin/sendmail -t -i" to not use SMTP, but sendmail
 
+    mail_import_secret = "" # a shared secret also known to the mail importer xmlrpc script
     mail_import_subpage_template = u"$from-$date-$subject" # used for mail import
     mail_import_pagename_search = ['subject', 'to', ] # where to look for target pagename (and in which order)
     mail_import_pagename_envelope = u"%s" # use u"+ %s/" to add "+ " and "/" automatically
     mail_import_pagename_regex = r'\[\[([^\]]*)\]\]' # how to find/extract the pagename from the subject
     mail_import_wiki_addrs = [] # the e-mail addresses for e-mails that should go into the wiki
-    mail_import_secret = ""
 
     # some dangerous mimetypes (we don't use "content-disposition: inline" for them when a user
     # downloads such attachments, because the browser might execute e.g. Javascript contained
-    # in the HTML and steal your moin cookie or do other nasty stuff) 
+    # in the HTML and steal your moin session cookie or do other nasty stuff)
     mimetypes_xss_protect = [
         'text/html',
         'application/x-shockwave-flash',
@@ -409,6 +419,13 @@ Lists: * bullets; 1., a. numbered items.
     navi_bar = [u'RecentChanges', u'FindPage', u'HelpContents', ]
     nonexist_qm = False
 
+    notification_bot_uri = None # uri of the jabber bot
+
+    # OpenID server support
+    openid_server_enabled = False
+    openid_server_restricted_users_group = None
+    openid_server_enable_user = False
+
     page_credits = [
         # Feel free to add other credits, but PLEASE do NOT change or remove
         # the following links - you help us by keeping them "as is":
@@ -423,18 +440,24 @@ Lists: * bullets; 1., a. numbered items.
         # if you don't need/want to check the html output, feel free to remove this one:
         '<a href="http://validator.w3.org/check?uri=referer" title="Click here to validate this page.">Valid HTML 4.01</a>',
         ]
+
+    # you can put some pieces of html at specific places into the theme output:
     page_footer1 = ''
     page_footer2 = ''
-
     page_header1 = ''
     page_header2 = ''
 
     page_front_page = u'HelpOnLanguages' # this will make people choose a sane config
     page_local_spelling_words = u'LocalSpellingWords'
-    page_category_regex = u'^Category[A-Z]'
-    page_dict_regex = u'[a-z0-9]Dict$'
-    page_group_regex = u'[a-z0-9]Group$'
-    page_template_regex = u'[a-z0-9]Template$'
+
+    # the following regexes should match the complete name when used in free text
+    # the group 'all' shall match all, while the group 'key' shall match the key only
+    # e.g. CategoryFoo -> group 'all' ==  CategoryFoo, group 'key' == Foo
+    # moin's code will add ^ / $ at beginning / end when needed
+    page_category_regex =  ur'(?P<all>Category(?P<key>(?!Template)\S+))'
+    page_dict_regex = ur'(?P<all>(?P<key>\S+)Dict)'
+    page_group_regex = ur'(?P<all>(?P<key>\S+)Group)'
+    page_template_regex = ur'(?P<all>(?P<key>\S+)Template)'
 
     page_license_enabled = False
     page_license_page = u'WikiLicense'
@@ -442,27 +465,28 @@ Lists: * bullets; 1., a. numbered items.
     # These icons will show in this order in the iconbar, unless they
     # are not relevant, e.g email icon when the wiki is not configured
     # for email.
-    page_iconbar = ["edit", "view", "diff", "info", "subscribe", "raw", "print", ]
+    page_iconbar = ["up", "edit", "view", "diff", "info", "subscribe", "raw", "print", ]
 
     # Standard buttons in the iconbar
     page_icons_table = {
-        # key           querystr dict, title, icon-key
-        'diff':        ({'action': 'diff'}, _("Diffs"), "diff"),
-        'info':        ({'action': 'info'}, _("Info"), "info"),
-        'edit':        ({'action': 'edit'}, _("Edit"), "edit"),
-        'unsubscribe': ({'action': 'unsubscribe'}, _("UnSubscribe"), "unsubscribe"),
-        'subscribe':   ({'action': 'subscribe'}, _("Subscribe"), "subscribe"),
-        'raw':         ({'action': 'raw'}, _("Raw"), "raw"),
-        'xml':         ({'action': 'show', 'mimetype': 'text/xml'}, _("XML"), "xml"),
-        'print':       ({'action': 'print'}, _("Print"), "print"),
-        'view':        ({}, _("View"), "view"),
+        # key           pagekey, querystr dict, title, icon-key
+        'diff':        ('page', {'action': 'diff'}, _("Diffs"), "diff"),
+        'info':        ('page', {'action': 'info'}, _("Info"), "info"),
+        'edit':        ('page', {'action': 'edit'}, _("Edit"), "edit"),
+        'unsubscribe': ('page', {'action': 'unsubscribe'}, _("UnSubscribe"), "unsubscribe"),
+        'subscribe':   ('page', {'action': 'subscribe'}, _("Subscribe"), "subscribe"),
+        'raw':         ('page', {'action': 'raw'}, _("Raw"), "raw"),
+        'xml':         ('page', {'action': 'show', 'mimetype': 'text/xml'}, _("XML"), "xml"),
+        'print':       ('page', {'action': 'print'}, _("Print"), "print"),
+        'view':        ('page', {}, _("View"), "view"),
+        'up':          ('page_parent_page', {}, _("Up"), "up"),
         }
 
 
     def password_checker(username, password):
         """ Check if a password is secure enough.
             We use a built-in check to get rid of the worst passwords.
-            
+
             We do NOT use cracklib / python-crack here any more because it is
             not thread-safe (we experienced segmentation faults when using it).
 
@@ -471,7 +495,9 @@ Lists: * bullets; 1., a. numbered items.
             @return: None if there is no problem with the password,
                      some string with an error msg, if the password is problematic.
         """
+
         try:
+            # in any case, do a very simple built-in check to avoid the worst passwords
             if len(password) < 6:
                 raise ValueError("Password too short.")
             if len(set(password)) < 4:
@@ -498,58 +524,85 @@ Lists: * bullets; 1., a. numbered items.
     password_checker = staticmethod(password_checker)
 
     quicklinks_default = [] # preload user quicklinks with this page list
+
     refresh = None # (minimum_delay, type), e.g.: (2, 'internal')
     rss_cache = 60 # suggested caching time for RecentChanges RSS, in seconds
+
+    search_results_per_page = 25
+
+    session_handler = session.DefaultSessionHandler()
+    session_id_handler = session.MoinCookieSessionIDHandler()
+
+    secrets = None  # if wiki admin does not set it, will get calculated from some cfg values
+
+    shared_intermap = None # can be string or list of strings (filenames)
+
+    show_hosts = True # show hostnames on RecentChanges / info/history action
+    show_interwiki = False # show our interwiki name (usually in front of the page name)
+    show_names = True # show editor names on RecentChanges / info/history action
+    show_section_numbers = 0 # enumerate sections (headlines) by default?
+    show_timings = False # show some timing stats (usually in the footer)
+    show_version = False # show moin version info / (C) (depends on theme)
+
     sistersites = [
         #('Self', 'http://localhost:8080/?action=sisterpages'),
         #('EmacsWiki', 'http://www.emacswiki.org/cgi-bin/test?action=sisterpages'),
         #('JspWiki', 'http://www.jspwiki.org/SisterSites.jsp'),
     ] # list of (sistersitename, sisterpagelistfetchurl)
-    shared_intermap = None # can be string or list of strings (filenames)
-    show_hosts = True
-    show_interwiki = False
-    show_login = True
-    show_names = True
-    show_section_numbers = 0
-    show_timings = False
-    show_version = False
+
     siteid = 'default'
+    sitename = u'Untitled Wiki' # Wiki identity
+
     stylesheets = [] # list of tuples (media, csshref) to insert after theme css, before user css
+
+    _subscribable_events = None # A list of event types that user can subscribe to
     subscribed_pages_default = [] # preload user subscribed pages with this page list
+    email_subscribed_events_default = [
+        PageChangedEvent.__name__,
+        PageRenamedEvent.__name__,
+        PageDeletedEvent.__name__,
+        PageCopiedEvent.__name__,
+        PageRevertedEvent.__name__,
+        FileAttachedEvent.__name__,
+    ]
+    jabber_subscribed_events_default = []
+
     superuser = [] # list of unicode user names that have super powers :)
-    supplementation_page = False
-    supplementation_page_name = u'Discussion'
-    supplementation_page_template = u'DiscussionTemplate'
+
+    supplementation_page = False # use supplementation pages (show a link in the theme)?
+    supplementation_page_name = u'Discussion' # name of suppl. subpage
+    supplementation_page_template = u'DiscussionTemplate' # name of template used to create suppl. pages
+
     surge_action_limits = {# allow max. <count> <action> requests per <dt> secs
         # action: (count, dt)
-        'all': (30, 30),
+        'all': (30, 30), # all requests (except cache/AttachFile action) count for this limit
+        'default': (30, 60), # default limit for actions without a specific limit
         'show': (30, 60),
         'recall': (10, 120),
         'raw': (20, 40),  # some people use this for css
-        'AttachFile': (90, 60),
         'diff': (30, 60),
         'fullsearch': (10, 120),
         'edit': (30, 300), # can be lowered after making preview different from edit
         'rss_rc': (1, 60),
-        'default': (30, 60),
+        # The following actions are often used for images - to avoid pages with lots of images
+        # (like photo galleries) triggering surge protection, we assign rather high limits:
+        'AttachFile': (90, 60),
+        'cache': (600, 30), # cache action is very cheap/efficient
     }
     surge_lockout_time = 3600 # secs you get locked out when you ignore warnings
 
-    textchas = None
+    textchas = None # a data structure with site-specific questions/answers, see HelpOnTextChas
     textchas_disabled_group = None # e.g. u'NoTextChasGroup' if you are a member of this group, you don't get textchas
 
     theme_default = 'modern'
     theme_force = False
 
-    traceback_show = 1
-    traceback_log_dir = None
+    traceback_show = True # if True, tracebacks are displayed in the web browser
+    traceback_log_dir = None # if set to a directory path, tracebacks are written to files there
 
-    trail_size = 5
+    trail_size = 5 # number of recently visited pagenames shown in the trail display
     tz_offset = 0.0 # default time zone offset in hours from UTC
 
-    user_autocreate = False # do we auto-create user profiles
-    user_email_unique = True # do we check whether a user's email is unique?
-
     # a regex of HTTP_USER_AGENTS that should be excluded from logging
     # and receive a FORBIDDEN for anything except viewing a page
     # list must not contain 'java' because of twikidraw wanting to save drawing uses this useragent
@@ -558,8 +611,11 @@ Lists: * bullets; 1., a. numbered items.
                   'microsoft.url.control|mirror| mj12bot|msnbot|msrbot|neomo|nutbot|omniexplorer|puf|robot|scooter|seekbot|'
                   'sherlock|slurp|sitecheck|snoopy|spider|teleport|twiceler|voilabot|voyager|webreaper|wget|yeti')
 
-    # Wiki identity
-    sitename = u'Untitled Wiki'
+    unzip_single_file_size = 2.0 * 1000 ** 2
+    unzip_attachments_space = 200.0 * 1000 ** 2
+    unzip_attachments_count = 101 # 1 zip file + 100 files contained in it
+
+    url_mappings = {}
 
     # url_prefix is DEPRECATED and not used any more by the code.
     # it confused many people by its name and default value of '/wiki' to the
@@ -581,10 +637,14 @@ Lists: * bullets; 1., a. numbered items.
     #url_prefix_action = 'action' # no leading or trailing '/'
     url_prefix_action = None # compatiblity
 
-    logo_string = None
-    interwikiname = None
+    # allow disabling certain userpreferences plugins
+    userprefs_disabled = []
 
-    url_mappings = {}
+    user_autocreate = False # do we auto-create user profiles
+    user_email_unique = True # do we check whether a user's email is unique?
+    user_jid_unique = True # do we check whether a user's email is unique?
+
+    user_homewiki = 'Self' # interwiki name for where user homepages are located
 
     user_checkbox_fields = [
         ('mailto_author', lambda _: _('Publish my email (not my wiki homepage) in author info')),
@@ -598,7 +658,6 @@ Lists: * bullets; 1., a. numbered items.
         ('show_fancy_diff', lambda _: _('Show fancy diffs')),
         ('wikiname_add_spaces', lambda _: _('Add spaces to displayed wiki names')),
         ('remember_me', lambda _: _('Remember login information')),
-        ('want_trivial', lambda _: _('Subscribe to trivial changes')),
 
         ('disabled', lambda _: _('Disable this account forever')),
         # if an account is disabled, it may be used for looking up
@@ -617,11 +676,10 @@ Lists: * bullets; 1., a. numbered items.
                               'show_fancy_diff':     1,
                               'wikiname_add_spaces': 0,
                               'remember_me':         1,
-                              'want_trivial':        0,
                              }
 
     # don't let the user change those
-    # user_checkbox_disable = ['disabled', 'want_trivial']
+    # user_checkbox_disable = ['disabled']
     user_checkbox_disable = []
 
     # remove those checkboxes:
@@ -632,9 +690,8 @@ Lists: * bullets; 1., a. numbered items.
     user_form_fields = [
         ('name', _('Name'), "text", "36", _("(Use FirstnameLastname)")),
         ('aliasname', _('Alias-Name'), "text", "36", ''),
-        ('password', _('Password'), "password", "36", ''),
-        ('password2', _('Password repeat'), "password", "36", _('(Only for password change or new account)')),
         ('email', _('Email'), "text", "36", ''),
+        ('jid', _('Jabber ID'), "text", "36", ''),
         ('css_url', _('User CSS URL'), "text", "40", _('(Leave it empty for disabling user CSS)')),
         ('edit_rows', _('Editor size'), "text", "3", ''),
     ]
@@ -645,6 +702,7 @@ Lists: * bullets; 1., a. numbered items.
         'password': '',
         'password2': '',
         'email': '',
+        'jid': '',
         'css_url': '',
         'edit_rows': "20",
     }
@@ -658,21 +716,12 @@ Lists: * bullets; 1., a. numbered items.
     user_form_remove = []
 
     # attributes we do NOT save to the userpref file
-    user_transient_fields = ['id', 'valid', 'may', 'auth_username', 'trusted', 'password', 'password2', 'auth_method', 'auth_attribs', ]
-
-    user_homewiki = 'Self' # interwiki name for where user homepages are located
-
-    unzip_single_file_size = 2.0 * 1000**2
-    unzip_attachments_space = 200.0 * 1000**2
-    unzip_attachments_count = 101 # 1 zip file + 100 files contained in it
+    user_transient_fields = ['id', 'valid', 'may', 'auth_username', 'password', 'password2', 'auth_method', 'auth_attribs', ]
 
     xapian_search = False
     xapian_index_dir = None
     xapian_stemming = False
     xapian_index_history = False
-    search_results_per_page = 10
-
-    SecurityPolicy = None
 
     def __init__(self, siteid):
         """ Init Config instance """
@@ -703,6 +752,13 @@ Lists: * bullets; 1., a. numbered items.
         self.cache.page_dict_regex = re.compile(self.page_dict_regex, re.UNICODE)
         self.cache.page_group_regex = re.compile(self.page_group_regex, re.UNICODE)
         self.cache.page_template_regex = re.compile(self.page_template_regex, re.UNICODE)
+
+        # the ..._regexact versions only match if nothing is left (exact match)
+        self.cache.page_category_regexact = re.compile(u'^%s$' % self.page_category_regex, re.UNICODE)
+        self.cache.page_dict_regexact = re.compile(u'^%s$' % self.page_dict_regex, re.UNICODE)
+        self.cache.page_group_regexact = re.compile(u'^%s$' % self.page_group_regex, re.UNICODE)
+        self.cache.page_template_regexact = re.compile(u'^%s$' % self.page_template_regex, re.UNICODE)
+
         self.cache.ua_spiders = self.ua_spiders and re.compile(self.ua_spiders, re.I)
 
         self._check_directories()
@@ -761,6 +817,26 @@ Lists: * bullets; 1., a. numbered items.
         # check if mail is possible and set flag:
         self.mail_enabled = (self.mail_smarthost is not None or self.mail_sendmail is not None) and self.mail_from
 
+        # check if jabber bot is available and set flag:
+        self.jabber_enabled = self.notification_bot_uri is not None
+
+        # if we are to use the jabber bot, instantiate a server object for future use
+        if self.jabber_enabled:
+
+            errmsg = "You must set a (long) secret string to send notifications!"
+            try:
+                if not self.secret:
+                    raise error.ConfigurationError(errmsg)
+            except AttributeError, err:
+                raise error.ConfigurationError(errmsg)
+
+            from xmlrpclib import Server
+            self.notification_server = Server(self.notification_bot_uri, )
+
+        if self.secrets is None:  # Note: this is 'secrets' (with s at the end), not 'secret' (as above)
+                                  # This stuff is already cleaned up in 1.8 repo...
+            self.secrets = self.calc_secrets()
+
         # Cache variables for the properties below
         self._iwid = self._iwid_full = self._meta_dict = None
 
@@ -779,6 +855,19 @@ Lists: * bullets; 1., a. numbered items.
             self.url_prefix_local = self.url_prefix_static
 
 
+    def calc_secrets(self):
+        """ make up some 'secret' using some config values """
+        varnames = ['data_dir', 'data_underlay_dir', 'language_default',
+                    'mail_smarthost', 'mail_from', 'page_front_page',
+                    'theme_default', 'sitename', 'logo_string',
+                    'interwikiname', 'user_homewiki', 'acl_rights_before', ]
+        secret = ''
+        for varname in varnames:
+            var = getattr(self, varname, None)
+            if isinstance(var, (str, unicode)):
+                secret += repr(var)
+        return secret
+
     def load_meta_dict(self):
         """ The meta_dict contains meta data about the wiki instance. """
         if getattr(self, "_meta_dict", None) is None:
@@ -796,6 +885,32 @@ Lists: * bullets; 1., a. numbered items.
     iwid = make_iwid_property("_iwid")
     iwid_full = make_iwid_property("_iwid_full")
 
+    # lazily load a list of events a user can subscribe to
+    def make_subscribable_events_prop():
+        def getter(self):
+            if getattr(self, "_subscribable_events", None) is None:
+                self._subscribable_events = events.get_subscribable_events()
+            return getattr(self, "_subscribable_events")
+
+        def setter(self, new_events):
+            self._subscribable_events = new_events
+
+        return property(getter, setter)
+    subscribable_events = make_subscribable_events_prop()
+
+    # lazily create a list of event handlers
+    def make_event_handlers_prop():
+        def getter(self):
+            if getattr(self, "_event_handlers", None) is None:
+                self._event_handlers = events.get_handlers(self)
+            return getattr(self, "_event_handlers")
+
+        def setter(self, new_handlers):
+            self._event_handlers = new_handlers
+
+        return property(getter, setter)
+    event_handlers = make_event_handlers_prop()
+
     def load_IWID(self):
         """ Loads the InterWikiID of this instance. It is used to identify the instance
             globally.
@@ -818,7 +933,7 @@ Lists: * bullets; 1., a. numbered items.
 
     def _config_check(self):
         """ Check namespace and warn about unknown names
-        
+
         Warn about names which are not used by DefaultConfig, except
         modules, classes, _private or __magic__ names.
 
@@ -840,13 +955,13 @@ configuration for typos before requesting support or reporting a bug.
 
     def _decode(self):
         """ Try to decode certain names, ignore unicode values
-        
-        Try to decode str using utf-8. If the decode fail, raise FatalError. 
+
+        Try to decode str using utf-8. If the decode fail, raise FatalError.
 
         Certain config variables should contain unicode values, and
         should be defined with u'text' syntax. Python decode these if
         the file have a 'coding' line.
-        
+
         This will allow utf-8 users to use simple strings using, without
         using u'string'. Other users will have to use u'string' for
         these names, because we don't know what is the charset of the
@@ -907,10 +1022,10 @@ file. It should match the actual charset of the configuration file.
             path_pages = os.path.join(path, "pages")
             if not (os.path.isdir(path_pages) and os.access(path_pages, mode)):
                 msg = '''
-%(attr)s "%(path)s" does not exists, or has incorrect ownership or
+%(attr)s "%(path)s" does not exist, or has incorrect ownership or
 permissions.
 
-Make sure the directory and the subdirectory pages are owned by the web
+Make sure the directory and the subdirectory "pages" are owned by the web
 server and are readable, writable and executable by the web server user
 and group.
 
@@ -942,7 +1057,7 @@ also the spelling of the directory name.
                     plugin_parent_dir = os.path.abspath(os.path.join(self.plugin_dir, '..'))
                     fp, path, info = imp.find_module('plugin', [plugin_parent_dir])
                     try:
-                        # Load the module and set in sys.modules             
+                        # Load the module and set in sys.modules
                         module = imp.load_module(name, fp, path, info)
                         sys.modules[self.siteid].plugin = module
                     finally:
@@ -979,6 +1094,6 @@ that the data/plugin directory has an __init__.py file.
         """ Make it possible to access a config object like a dict """
         return getattr(self, item)
 
-# remove the gettext pseudo function 
+# remove the gettext pseudo function
 del _
 
