@@ -64,7 +64,7 @@ class MinimalRequest(object):
         self._page_headings = {}
         self.result = []
 
-    def getText(self, text, formatted=True):
+    def getText(self, text, wiki=False, percent=False):
         return text
 
     def write(self, text):
@@ -728,7 +728,7 @@ test
         self.do(test, output)
 
     def testPreSuccess7(self):
-        py.test.skip('broken test')
+        py.test.skip("Broken.")
         test = ur"""
  * {{{
    test
