@@ -49,20 +49,20 @@ def search_box(type, macro):
 
     # Title search settings
     boxes = ''
-    button = _("Search Titles", formatted=False)
+    button = _("Search Titles")
 
     # Special code for fullsearch
     if type == "fullsearch":
         boxes = [
             u'<br>',
             u'<input type="checkbox" name="context" value="160" checked="checked">',
-            _('Display context of search results', formatted=False),
+            _('Display context of search results'),
             u'<br>',
             u'<input type="checkbox" name="case" value="1">',
-            _('Case-sensitive searching', formatted=False),
+            _('Case-sensitive searching'),
             ]
         boxes = u'\n'.join(boxes)
-        button = _("Search Text", formatted=False)
+        button = _("Search Text")
 
     # Format
     type = (type == "titlesearch")
@@ -97,7 +97,7 @@ def execute(macro, needle):
     # TODO: search should implement those errors message for clients
     elif needle.isspace():
         err = _('Please use a more selective search term instead of '
-                '{{{"%s"}}}', formatted=True) % needle
+                '{{{"%s"}}}', wiki=True) % needle
         return '<span class="error">%s</span>' % err
 
     needle = needle.strip()
