@@ -6,8 +6,8 @@
     The days are links to Wiki pages following this naming convention:
     BasePageName/year-month-day
 
-    @copyright: 2002-2005 by Thomas Waldmann <ThomasWaldmann@gmx.de>
-    @license: GNU GPL - see COPYING for details.
+    @copyright: 2002-2007 MoinMoin:ThomasWaldmann
+    @license: GNU GPL, see COPYING for details.
 
     Revisions:
     * first revision without a number (=1.0):
@@ -87,69 +87,72 @@
         * do a correct calculation of "today" using user's timezone
     * 2.2:
         * added template argument for specifying an edit template for new pages
+    * 2.3:
+        * adapted to moin 1.7 new macro parameter parsing
 
     Usage:
-        [[MonthCalendar(BasePage,year,month,monthoffset,monthoffset2,height6)]]
+        <<MonthCalendar(BasePage,year,month,monthoffset,monthoffset2,height6)>>
 
         each parameter can be empty and then defaults to currentpage or currentdate or monthoffset=0
 
     Samples (paste that to one of your pages for a first try):
 
 Calendar of current month for current page:
-[[MonthCalendar]]
+<<MonthCalendar>>
 
 Calendar of last month:
-[[MonthCalendar(,,,-1)]]
+<<MonthCalendar(,,,-1)>>
 
 Calendar of next month:
-[[MonthCalendar(,,,+1)]]
+<<MonthCalendar(,,,+1)>>
 
 Calendar of Page SampleUser, this years december:
-[[MonthCalendar(SampleUser,,12)]]
+<<MonthCalendar(SampleUser,,12)>>
 
 Calendar of current Page, this years december:
-[[MonthCalendar(,,12)]]
+<<MonthCalendar(,,12)>>
 
 Calendar of December, 2001:
-[[MonthCalendar(,2001,12)]]
+<<MonthCalendar(,2001,12)>>
 
 Calendar of the month two months after December, 2001
 (maybe doesn't make much sense, but is possible)
-[[MonthCalendar(,2001,12,+2)]]
+<<MonthCalendar(,2001,12,+2)>>
 
 Calendar of year 2002 (every month padded to height of 6):
 ||||||Year 2002||
-||[[MonthCalendar(,2002,1,,,1)]]||[[MonthCalendar(,2002,2,,,1)]]||[[MonthCalendar(,2002,3,,,1)]]||
-||[[MonthCalendar(,2002,4,,,1)]]||[[MonthCalendar(,2002,5,,,1)]]||[[MonthCalendar(,2002,6,,,1)]]||
-||[[MonthCalendar(,2002,7,,,1)]]||[[MonthCalendar(,2002,8,,,1)]]||[[MonthCalendar(,2002,9,,,1)]]||
-||[[MonthCalendar(,2002,10,,,1)]]||[[MonthCalendar(,2002,11,,,1)]]||[[MonthCalendar(,2002,12,,,1)]]||
+||<<MonthCalendar(,2002,1,,,1)>>||<<MonthCalendar(,2002,2,,,1)>>||<<MonthCalendar(,2002,3,,,1)>>||
+||<<MonthCalendar(,2002,4,,,1)>>||<<MonthCalendar(,2002,5,,,1)>>||<<MonthCalendar(,2002,6,,,1)>>||
+||<<MonthCalendar(,2002,7,,,1)>>||<<MonthCalendar(,2002,8,,,1)>>||<<MonthCalendar(,2002,9,,,1)>>||
+||<<MonthCalendar(,2002,10,,,1)>>||<<MonthCalendar(,2002,11,,,1)>>||<<MonthCalendar(,2002,12,,,1)>>||
 
 Current calendar of me, also showing entries of A and B:
-[[MonthCalendar(MyPage*TestUserA*TestUserB)]]
+<<MonthCalendar(MyPage*TestUserA*TestUserB)>>
 
 SubPage calendars:
-[[MonthCalendar(MyName/CalPrivate)]]
-[[MonthCalendar(MyName/CalBusiness)]]
-[[MonthCalendar(MyName/CalBusiness*MyName/CalPrivate)]]
+<<MonthCalendar(MyName/CalPrivate)>>
+<<MonthCalendar(MyName/CalBusiness)>>
+<<MonthCalendar(MyName/CalBusiness*MyName/CalPrivate)>>
 
 
 Anniversary Calendars: (no year data)
-[[MonthCalendar(Yearly,,,+1,,6,1)]]
+<<MonthCalendar(Yearly,,,+1,,6,1)>>
 
-This creates calendars of the format Yearly/MM-DD 
-By leaving out the year, you can set birthdays, and anniversaries in this 
+This creates calendars of the format Yearly/MM-DD
+By leaving out the year, you can set birthdays, and anniversaries in this
 calendar and not have to re-enter each year.
 
 This creates a calendar which uses MonthCalendarTemplate for directly editing
 nonexisting day pages:
-[[MonthCalendar(,,,,,,MonthCalendarTemplate)]]
+<<MonthCalendar(,,,,,,MonthCalendarTemplate)>>
 """
 
-Dependencies = ['namespace','time']
+Dependencies = ['namespace', 'time', ]
+
+import re, calendar, time
 
 from MoinMoin import wikiutil
 from MoinMoin.Page import Page
-import re, calendar, time
 
 # The following line sets the calendar to have either Sunday or Monday as
 # the first day of the week. Only SUNDAY or MONDAY (case sensitive) are
@@ -158,91 +161,47 @@ import re, calendar, time
 # XXX change here ----------------vvvvvv
 calendar.setfirstweekday(calendar.MONDAY)
 
-def cliprgb(r,g,b): # don't use 255!
-    if r < 0:   r=0
-    if r > 254: r=254
-    if b < 0:   b=0
-    if b > 254: b=254
-    if g < 0:   g=0
-    if g > 254: g=254
-    return r, g, b
+def cliprgb(r, g, b):
+    """ clip r,g,b values into range 0..254 """
+    def clip(x):
+        """ clip x value into range 0..254 """
+        if x < 0:
+            x = 0
+        elif x > 254:
+            x = 254
+        return x
+    return clip(r), clip(g), clip(b)
 
 def yearmonthplusoffset(year, month, offset):
-    month = month+offset
+    """ calculate new year/month from year/month and offset """
+    month += offset
     # handle offset and under/overflows - quick and dirty, yes!
     while month < 1:
-        month = month + 12
-        year = year - 1
+        month += 12
+        year -= 1
     while month > 12:
-        month = month - 12
-        year = year + 1
+        month -= 12
+        year += 1
     return year, month
 
-def parseargs(args, defpagename, defyear, defmonth, defoffset, defoffset2, defheight6, defanniversary, deftemplate):
-    strpagename = args.group('basepage')
-    if strpagename:
-        parmpagename = wikiutil.unquoteWikiname(strpagename)
-    else:
-        parmpagename = defpagename
+def parseargs(request, args, defpagename, defyear, defmonth, defoffset, defoffset2, defheight6, defanniversary, deftemplate):
+    """ parse macro arguments """
+    args = wikiutil.parse_quoted_separated(args, name_value=False)
+    args += [None] * 8 # fill up with None to trigger defaults
+    parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, parmanniversary, parmtemplate = args[:8]
+    parmpagename = wikiutil.get_unicode(request, parmpagename, 'pagename', defpagename)
+    parmyear = wikiutil.get_int(request, parmyear, 'year', defyear)
+    parmmonth = wikiutil.get_int(request, parmmonth, 'month', defmonth)
+    parmoffset = wikiutil.get_int(request, parmoffset, 'offset', defoffset)
+    parmoffset2 = wikiutil.get_int(request, parmoffset2, 'offset2', defoffset2)
+    parmheight6 = wikiutil.get_bool(request, parmheight6, 'height6', defheight6)
+    parmanniversary = wikiutil.get_bool(request, parmanniversary, 'anniversary', defanniversary)
+    parmtemplate = wikiutil.get_unicode(request, parmtemplate, 'template', deftemplate)
+
     # multiple pagenames separated by "*" - split into list of pagenames
     parmpagename = re.split(r'\*', parmpagename)
 
-    stryear = args.group('year')
-    if stryear:
-        parmyear = int(stryear)
-    else:
-        parmyear = defyear
-
-    strmonth = args.group('month')
-    if strmonth:
-        parmmonth = int(strmonth)
-    else:
-        parmmonth = defmonth
-    
-    stroffset = args.group('offset')
-    if stroffset:
-        parmoffset = int(stroffset)
-    else:
-        parmoffset = defoffset
-
-    stroffset2 = args.group('offset2')
-    if stroffset2:
-        parmoffset2 = int(stroffset2)
-    else:
-        parmoffset2 = defoffset2
-
-    strheight6 = args.group('height6')
-    if strheight6:
-        parmheight6 = int(strheight6)
-    else:
-        parmheight6 = defheight6
-
-    stranniversary = args.group('anniversary')
-    if stranniversary:
-            parmanniversary = int(stranniversary)
-    else:
-        parmanniversary = defanniversary
-
-    strtemplate = args.group('template')
-    if strtemplate:
-        parmtemplate = wikiutil.unquoteWikiname(strtemplate)
-    else:
-        parmtemplate = deftemplate
     return parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, parmanniversary, parmtemplate
-        
-# FIXME:                          vvvvvv is there a better way for matching a pagename ?
-_arg_basepage = r'\s*(?P<basepage>[^, ]+)?\s*'
-_arg_year = r',\s*(?P<year>\d+)?\s*'
-_arg_month = r',\s*(?P<month>\d+)?\s*'
-_arg_offset = r',\s*(?P<offset>[+-]?\d+)?\s*'
-_arg_offset2 = r',\s*(?P<offset2>[+-]?\d+)?\s*'
-_arg_height6 = r',\s*(?P<height6>[+-]?\d+)?\s*'
-_arg_anniversary =  r',\s*(?P<anniversary>[+-]?\d+)?\s*'
-_arg_template = r',\s*(?P<template>[^, ]+)?\s*' # XXX see basepage comment
-_args_re_pattern = r'^(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?$' % \
-                     (_arg_basepage,_arg_year,_arg_month, \
-                      _arg_offset,_arg_offset2,_arg_height6,_arg_anniversary,_arg_template)
-
 
 def execute(macro, text):
     request = macro.request
@@ -253,52 +212,54 @@ def execute(macro, text):
     if request.mode_getpagelinks:
         return ''
 
-    args_re = re.compile(_args_re_pattern)
-    
     currentyear, currentmonth, currentday, h, m, s, wd, yd, ds = request.user.getTime(time.time())
     thispage = formatter.page.page_name
     # does the url have calendar params (= somebody has clicked on prev/next links in calendar) ?
-    if macro.form.has_key('calparms'):
+    if 'calparms' in macro.form:
+        has_calparms = 1 # yes!
         text2 = macro.form['calparms'][0]
-        args2 = args_re.match(text2)
-        if not args2:
-            return ('<p><strong class="error">%s</strong></p>' % _('Invalid MonthCalendar calparms "%s"!')) % (text2,)
-        else:
-            has_calparms = 1 # yes!
+        try:
             cparmpagename, cparmyear, cparmmonth, cparmoffset, cparmoffset2, cparmheight6, cparmanniversary, cparmtemplate = \
-                parseargs(args2, thispage, currentyear, currentmonth, 0, 0, 0, 0, '')
+                parseargs(request, text2, thispage, currentyear, currentmonth, 0, 0, False, False, u'')
+        except (ValueError, TypeError), err:
+            return macro.format_error(err)
     else:
         has_calparms = 0
 
     if text is None: # macro call without parameters
+        text = u''
+
+    # parse and check arguments
+    try:
         parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, anniversary, parmtemplate = \
-            [thispage], currentyear, currentmonth, 0, 0, 0, 0, ''
-    else:
-        # parse and check arguments
-        args = args_re.match(text)
-        if not args:
-            return ('<p><strong class="error">%s</strong></p>' % _('Invalid MonthCalendar arguments "%s"!')) % (text,)
-        else:
-            parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, anniversary, parmtemplate = \
-                parseargs(args, thispage, currentyear, currentmonth, 0, 0, 0, 0, '')
+            parseargs(request, text, thispage, currentyear, currentmonth, 0, 0, False, False, u'')
+    except (ValueError, TypeError), err:
+        return macro.format_error(err)
 
     # does url have calendar params and is THIS the right calendar to modify (we can have multiple
     # calendars on the same page)?
     #if has_calparms and (cparmpagename,cparmyear,cparmmonth,cparmoffset) == (parmpagename,parmyear,parmmonth,parmoffset):
-    
+
     # move all calendars when using the navigation:
     if has_calparms and cparmpagename == parmpagename:
-        year,month = yearmonthplusoffset(parmyear, parmmonth, parmoffset + cparmoffset2)
+        year, month = yearmonthplusoffset(parmyear, parmmonth, parmoffset + cparmoffset2)
         parmoffset2 = cparmoffset2
         parmtemplate = cparmtemplate
     else:
-        year,month = yearmonthplusoffset(parmyear, parmmonth, parmoffset)
+        year, month = yearmonthplusoffset(parmyear, parmmonth, parmoffset)
+
+    if request.isSpiderAgent and abs(currentyear - year) > 1:
+        return '' # this is a bot and it didn't follow the rules (see below)
+    if currentyear == year:
+        attrs = {}
+    else:
+        attrs = {'rel': 'nofollow' } # otherwise even well-behaved bots will index forever
 
     # get the calendar
     monthcal = calendar.monthcalendar(year, month)
 
     # european / US differences
-    months = ('January','February','March','April','May','June','July','August','September','October','November','December')
+    months = ('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December')
     # Set things up for Monday or Sunday as the first day of the week
     if calendar.firstweekday() == calendar.MONDAY:
         wkend = (5, 6)
@@ -309,18 +270,19 @@ def execute(macro, text):
 
     colorstep = 85
     p = Page(request, thispage)
-    qpagenames = '*'.join(map(wikiutil.quoteWikinameURL, parmpagename))
+    qpagenames = '*'.join([wikiutil.quoteWikinameURL(pn) for pn in parmpagename])
     qtemplate = wikiutil.quoteWikinameURL(parmtemplate)
-    querystr = "calparms=%s,%d,%d,%d,%%d,%s" % (qpagenames, parmyear, parmmonth, parmoffset, qtemplate)
-    prevlink  = p.url(request, querystr % (parmoffset2 - 1), 0)
-    nextlink  = p.url(request, querystr % (parmoffset2 + 1), 0)
-    prevylink = p.url(request, querystr % (parmoffset2 - 12), 0)
-    nextylink = p.url(request, querystr % (parmoffset2 + 12), 0)
-    prevmonth = formatter.url(1, prevlink, 'cal-link') + '&lt;' + formatter.url(0)
-    nextmonth = formatter.url(1, nextlink, 'cal-link') + '&gt;' + formatter.url(0)
-    prevyear  = formatter.url(1, prevylink, 'cal-link') + '&lt;&lt;' + formatter.url(0)
-    nextyear  = formatter.url(1, nextylink, 'cal-link') + '&gt;&gt;' + formatter.url(0)
-    
+    querystr = "calparms=%%s,%d,%d,%d,%%d,%%s" % (parmyear, parmmonth, parmoffset)
+    prevlink = p.url(request, querystr % (qpagenames, parmoffset2 - 1, qtemplate))
+    nextlink = p.url(request, querystr % (qpagenames, parmoffset2 + 1, qtemplate))
+    prevylink = p.url(request, querystr % (qpagenames, parmoffset2 - 12, qtemplate))
+    nextylink = p.url(request, querystr % (qpagenames, parmoffset2 + 12, qtemplate))
+
+    prevmonth = formatter.url(1, prevlink, 'cal-link', **attrs) + '&lt;' + formatter.url(0)
+    nextmonth = formatter.url(1, nextlink, 'cal-link', **attrs) + '&gt;' + formatter.url(0)
+    prevyear = formatter.url(1, prevylink, 'cal-link', **attrs) + '&lt;&lt;' + formatter.url(0)
+    nextyear = formatter.url(1, nextylink, 'cal-link', **attrs) + '&gt;&gt;' + formatter.url(0)
+
     if parmpagename != [thispage]:
         pagelinks = ''
         r, g, b = (255, 0, 0)
@@ -334,16 +296,18 @@ def execute(macro, text):
         while st < l:
             ch = parmpagename[0][st:st+chstep]
             r, g, b = cliprgb(r, g, b)
-            pagelinks = pagelinks + '<a style="%s" href="%s">%s</a>' % \
-                ('background-color:#%02x%02x%02x;color:#000000;text-decoration:none' % \
-                    (r,g,b), Page(request, parmpagename[0]).url(request), ch)
+            link = Page(request, parmpagename[0]).link_to(request, ch,
+                        rel='nofollow',
+                        style='background-color:#%02x%02x%02x;color:#000000;text-decoration:none' % (r, g, b))
+            pagelinks = pagelinks + link
             r, g, b = (r, g+colorstep, b)
             st = st + chstep
         r, g, b = (255-colorstep, 255, 255-colorstep)
         for page in parmpagename[1:]:
-            pagelinks = pagelinks + '*<a style="%s" href="%s">%s</a>' % \
-                            ('background-color:#%02x%02x%02x;color:#000000;text-decoration:none' % \
-                                (r,g,b), Page(request, page).url(request), page)
+            link = Page(request, page).link_to(request, page,
+                        rel='nofollow',
+                        style='background-color:#%02x%02x%02x;color:#000000;text-decoration:none' % (r, g, b))
+            pagelinks = pagelinks + '*' + link
         showpagename = '   %s<BR>\n' % pagelinks
     else:
         showpagename = ''
@@ -367,12 +331,12 @@ def execute(macro, text):
             cssday = "cal-weekend"
         else:
             cssday = "cal-workday"
-        restd2.append('  <td class="%s" width="14%%">%s</td>\n' % (cssday, wday))
+        restd2.append('  <td class="%s">%s</td>\n' % (cssday, wday))
     restr2 = ' <tr>\n%s </tr>\n' % "".join(restd2)
- 
+
     if parmheight6:
         while len(monthcal) < 6:
-            monthcal = monthcal + [[0,0,0,0,0,0,0]]
+            monthcal = monthcal + [[0, 0, 0, 0, 0, 0, 0]]
 
     maketip_js = []
     restrn = []
@@ -389,7 +353,7 @@ def execute(macro, text):
                 else:
                     link = "%s/%4d-%02d-%02d" % (page, year, month, day)
                 daypage = Page(request, link)
-                if daypage.exists():
+                if daypage.exists() and request.user.may.read(link):
                     csslink = "cal-usedday"
                     query = {}
                     r, g, b, u = (255, 0, 0, 1)
@@ -399,14 +363,15 @@ def execute(macro, text):
                     for match in header1_re.finditer(daycontent):
                         if match:
                             title = match.group(1)
-                            title = wikiutil.escape(title).replace("'","\\'")
+                            title = wikiutil.escape(title).replace("'", "\\'")
                             titletext.append(title)
+                    link = wikiutil.escape(link).replace("'", "\\'")
                     tipname = link
                     tiptitle = link
                     tiptext = '<br>'.join(titletext)
                     maketip_js.append("maketip('%s','%s','%s');" % (tipname, tiptitle, tiptext))
-                    onmouse = {'onMouseOver': "tip('%s')" % tipname,
-                               'onMouseOut':  "untip()"}
+                    attrs = {'onMouseOver': "tip('%s')" % tipname,
+                             'onMouseOut': "untip()"}
                 else:
                     csslink = "cal-emptyday"
                     if parmtemplate:
@@ -416,7 +381,7 @@ def execute(macro, text):
                     r, g, b, u = (255, 255, 255, 0)
                     if wkday in wkend:
                         csslink = "cal-weekend"
-                    onmouse = {}
+                    attrs = {'rel': 'nofollow'}
                 for otherpage in parmpagename[1:]:
                     otherlink = "%s/%4d-%02d-%02d" % (otherpage, year, month, day)
                     otherdaypage = Page(request, otherlink)
@@ -428,7 +393,7 @@ def execute(macro, text):
                             r, g, b = (r, g+colorstep, b)
                 r, g, b = cliprgb(r, g, b)
                 style = 'background-color:#%02x%02x%02x' % (r, g, b)
-                fmtlink = formatter.url(1, daypage.url(request, query), csslink, **onmouse) + str(day) + formatter.url(0)
+                fmtlink = formatter.url(1, daypage.url(request, query), csslink, **attrs) + str(day) + formatter.url(0)
                 if day == currentday and month == currentmonth and year == currentyear:
                     cssday = "cal-today"
                     fmtlink = "<b>%s</b>" % fmtlink # for browser with CSS probs
@@ -437,19 +402,24 @@ def execute(macro, text):
                 restdn.append('  <td style="%s" class="%s">%s</td>\n' % (style, cssday, fmtlink))
         restrn.append(' <tr>\n%s </tr>\n' % "".join(restdn))
 
-    restable = '<table border="2" cellspacing="2" cellpadding="2">\n%s%s%s</table>\n'
+    restable = '<table border="2" cellspacing="2" cellpadding="2">\n<col width="14%%" span="7">%s%s%s</table>\n'
     restable = restable % (restr1, restr2, "".join(restrn))
 
-    result = """\
-<script language="JavaScript" type="text/javascript" src="%s/common/js/infobox.js"></script>
-<div id="infodiv" style="position:absolute; visibility:hidden; z-index:20; top:-999em; left:0px;"></div>
-<script language="JavaScript" type="text/javascript">
+    if maketip_js:
+        tip_js = '''<script language="JavaScript" type="text/javascript">
 <!--
 %s
 // -->
 </script>
-%s
-""" % (request.cfg.url_prefix, "\n".join(maketip_js), restable)
+''' % '\n'.join(maketip_js)
+    else:
+        tip_js = ''
+
+    result = """\
+<script type="text/javascript" src="%s/common/js/infobox.js"></script>
+<div id="%s" style="position:absolute; visibility:hidden; z-index:20; top:-999em; left:0px;"></div>
+%s%s
+""" % (request.cfg.url_prefix_static, formatter.make_id_unique('infodiv'), tip_js, restable)
     return formatter.rawHTML(result)
 
 # EOF
