@@ -2,21 +2,23 @@
 """
     MoinMoin - Package Installer
 
-    @copyright: 2005 by MoinMoin:AlexanderSchremmer
+    @copyright: 2005 MoinMoin:AlexanderSchremmer,
+                2007 MoinMoin:ReimarBauer
     @license: GNU GPL, see COPYING for details.
 """
 
-import os
-import sys
+import os, sys
 import zipfile
 
-from MoinMoin import config, wikiutil, caching
+from MoinMoin import config, wikiutil, caching, user
 from MoinMoin.Page import Page
 from MoinMoin.PageEditor import PageEditor
+from MoinMoin.logfile import editlog, eventlog
 
 MOIN_PACKAGE_FILE = 'MOIN_PACKAGE'
 MAX_VERSION = 1
 
+
 # Exceptions
 class PackageException(Exception):
     """ Raised if the package is broken. """
@@ -38,10 +40,29 @@ class RuntimeScriptException(ScriptException):
 class ScriptExit(Exception):
     """ Raised by the script commands when the script should quit. """
 
+def event_logfile(self, pagename, pagefile):
+    # add event log entry
+    eventtype = 'SAVENEW'
+    mtime_usecs = wikiutil.timestamp2version(os.path.getmtime(pagefile))
+    elog = eventlog.EventLog(self.request)
+    elog.add(self.request, eventtype, {'pagename': pagename}, 1, mtime_usecs)
+
+def edit_logfile_append(self, pagename, pagefile, rev, action, logname='edit-log', comment=u'', author=u"Scripting Subsystem"):
+    glog = editlog.EditLog(self.request, uid_override=author)
+    pagelog = Page(self.request, pagename).getPagePath(logname, use_underlay=0, isfile=1)
+    llog = editlog.EditLog(self.request, filename=pagelog,
+                               uid_override=author)
+    mtime_usecs = wikiutil.timestamp2version(os.path.getmtime(pagefile))
+    host = '::1'
+    extra = u''
+    glog.add(self.request, mtime_usecs, rev, action, pagename, host, comment)
+    llog.add(self.request, mtime_usecs, rev, action, pagename, host, extra, comment)
+    event_logfile(self, pagename, pagefile)
+
 # Parsing and (un)quoting for script files
-def packLine(list, separator="|"):
-    """ Packs a list into a string that is separated by `separator`. """
-    return '|'.join([x.replace('\\', '\\\\').replace(separator, '\\' + separator) for x in list])
+def packLine(items, separator="|"):
+    """ Packs a list of items into a string that is separated by `separator`. """
+    return '|'.join([item.replace('\\', '\\\\').replace(separator, '\\' + separator) for item in items])
 
 def unpackLine(string, separator="|"):
     """ Unpacks a string that was packed by packLine. """
@@ -91,11 +112,69 @@ class ScriptEngine:
         self.themename = None
         self.ignoreExceptions = False
         self.goto = 0
-        
+
         #Satisfy pylint
         self.msg = getattr(self, "msg", "")
         self.request = getattr(self, "request", None)
 
+    def do_addattachment(self, zipname, filename, pagename, author=u"Scripting Subsystem", comment=u""):
+        """
+        Installs an attachment
+
+        @param pagename: Page where the file is attached. Or in 2.0, the file itself.
+        @param zipname: Filename of the attachment from the zip file
+        @param filename: Filename of the attachment (just applicable for MoinMoin < 2.0)
+        """
+        if self.request.user.may.write(pagename):
+            _ = self.request.getText
+
+            attachments = Page(self.request, pagename).getPagePath("attachments", check_create=1)
+            filename = wikiutil.taintfilename(filename)
+            zipname = wikiutil.taintfilename(zipname)
+            target = os.path.join(attachments, filename)
+            page = PageEditor(self.request, pagename, do_editor_backup=0, uid_override=author)
+            rev = page.current_rev()
+            path = page.getPagePath(check_create=0)
+            if not os.path.exists(target):
+                self._extractToFile(zipname, target)
+                if os.path.exists(target):
+                    os.chmod(target, config.umask )
+                    action = 'ATTNEW'
+                    edit_logfile_append(self, pagename, path, rev, action, logname='edit-log',
+                                       comment=u'%(filename)s' % {"filename": filename}, author=author)
+                self.msg += u"%(filename)s attached \n" % {"filename": filename}
+            else:
+                self.msg += u"%(filename)s not attached \n" % {"filename": filename}
+        else:
+            self.msg += u"action add attachment: not enough rights - nothing done \n"
+
+    def do_delattachment(self, filename, pagename, author=u"Scripting Subsystem", comment=u""):
+        """
+        Removes an attachment
+
+        @param pagename: Page where the file is attached. Or in 2.0, the file itself.
+        @param filename: Filename of the attachment (just applicable for MoinMoin < 2.0)
+        """
+        if self.request.user.may.write(pagename):
+            _ = self.request.getText
+
+            attachments = Page(self.request, pagename).getPagePath("attachments", check_create=1)
+            filename = wikiutil.taintfilename(filename)
+            target = os.path.join(attachments, filename)
+            page = PageEditor(self.request, pagename, do_editor_backup=0, uid_override=author)
+            rev = page.current_rev()
+            path = page.getPagePath(check_create=0)
+            if os.path.exists(target):
+                os.remove(target)
+                action = 'ATTDEL'
+                edit_logfile_append(self, pagename, path, rev, action, logname='edit-log',
+                                    comment=u'%(filename)s' % {"filename": filename}, author=author)
+                self.msg += u"%(filename)s removed \n" % {"filename": filename}
+            else:
+                self.msg += u"%(filename)s does not exist \n" % {"filename": filename}
+        else:
+            self.msg += u"action delete attachment: not enough rights - nothing done \n"
+
     def do_print(self, *param):
         """ Prints the parameters into output of the script. """
         self.msg += '; '.join(param) + "\n"
@@ -119,7 +198,7 @@ class ScriptEngine:
         @param lines: lines to ignore
         """
         _ = self.request.getText
-        
+
         from MoinMoin.version import release
         version_int = [int(x) for x in version.split(".")]
         release = [int(x) for x in release.split(".")]
@@ -156,7 +235,7 @@ class ScriptEngine:
                                   wikiutil.taintfilename(target))
         theme_dir = os.path.dirname(theme_file)
         if not os.path.exists(theme_dir):
-            os.makedirs(theme_dir, 0777 & config.umask)
+            os.makedirs(theme_dir)
         self._extractToFile(filename, theme_file)
 
     def do_installplugin(self, filename, visibility, ptype, target):
@@ -200,11 +279,11 @@ class ScriptEngine:
                 raise RuntimeScriptException(_("Installation of '%(filename)s' failed.") % {
                     'filename': filename} + "\n" + package.msg)
         else:
-            raise RuntimeScriptException(_('The file %s is not a MoinMoin package file.' % filename))
+            raise RuntimeScriptException(_('The file %s is not a MoinMoin package file.') % filename)
 
         self.msg += package.msg
 
-    def do_addrevision(self, filename, pagename, author=u"Scripting Subsystem", comment=u"", trivial = u"No"):
+    def do_addrevision(self, filename, pagename, author=u"Scripting Subsystem", comment=u"", trivial=u"No"):
         """ Adds a revision to a page.
 
         @param filename: name of the file in this package
@@ -213,16 +292,44 @@ class ScriptEngine:
         @param comment:  comment related to this revision (optional)
         @param trivial:  boolean, if it is a trivial edit
         """
-        _ = self.request.getText
-        trivial = str2boolean(trivial)
+        if self.request.user.may.write(pagename):
+            _ = self.request.getText
+            trivial = str2boolean(trivial)
+            uid = user.getUserId(self.request, author)
+            theuser = user.User(self.request, uid)
+            save_user = self.request.user
+            self.request.user = theuser
+            page = PageEditor(self.request, pagename, do_editor_backup=0, uid_override=author)
+            try:
+                page.saveText(self.extract_file(filename).decode("utf-8"), 0, trivial=trivial, comment=comment)
+                self.msg += u"%(pagename)s added \n" % {"pagename": pagename}
+            except PageEditor.Unchanged:
+                pass
+            self.request.user = save_user
+            page.clean_acl_cache()
+        else:
+            self.msg += u"action add revision: not enough rights - nothing done \n"
 
-        page = PageEditor(self.request, pagename, do_editor_backup=0, uid_override=author)
-        try:
-            page.saveText(self.extract_file(filename).decode("utf-8"), 0, trivial=trivial, comment=comment)
-        except PageEditor.Unchanged:
-            pass
+    def do_renamepage(self, pagename, newpagename, author=u"Scripting Subsystem", comment=u"Renamed by the scripting subsystem."):
+        """ Renames a page.
 
-        page.clean_acl_cache()
+        @param pagename: name of the target page
+        @param newpagename: name of the new page
+        @param author:   user name of the editor (optional)
+        @param comment:  comment related to this revision (optional)
+        """
+        if self.request.user.may.write(pagename):
+            _ = self.request.getText
+            page = PageEditor(self.request, pagename, do_editor_backup=0, uid_override=author)
+            if not page.exists():
+                raise RuntimeScriptException(_("The page %s does not exist.") % pagename)
+            newpage = PageEditor(self.request, newpagename)
+            page.renamePage(newpage.page_name, comment=u"Renamed from '%s'" % (pagename))
+            self.msg += u'%(pagename)s renamed to %(newpagename)s\n' % {
+                            "pagename": pagename,
+                            "newpagename": newpagename}
+        else:
+            self.msg += u"action rename page: not enough rights - nothing done \n"
 
     def do_deletepage(self, pagename, comment="Deleted by the scripting subsystem."):
         """ Marks a page as deleted (like the DeletePage action).
@@ -230,12 +337,14 @@ class ScriptEngine:
         @param pagename: page to delete
         @param comment:  the related comment (optional)
         """
-        _ = self.request.getText
-        page = PageEditor(self.request, pagename, do_editor_backup=0)
-        if not page.exists():
-            raise RuntimeScriptException(_("The page %s does not exist.") % pagename)
-
-        page.deletePage(comment)
+        if self.request.user.may.write(pagename):
+            _ = self.request.getText
+            page = PageEditor(self.request, pagename, do_editor_backup=0)
+            if not page.exists():
+                raise RuntimeScriptException(_("The page %s does not exist.") % pagename)
+            page.deletePage(comment)
+        else:
+            self.msg += u"action delete page: not enough rights - nothing done \n"
 
     def do_replaceunderlay(self, filename, pagename):
         """
@@ -250,7 +359,7 @@ class ScriptEngine:
         pagedir = page.getPagePath(use_underlay=1, check_create=1)
 
         revdir = os.path.join(pagedir, 'revisions')
-        cfn = os.path.join(pagedir,'current')
+        cfn = os.path.join(pagedir, 'current')
 
         revstr = '%08d' % 1
         if not os.path.exists(revdir):
@@ -269,7 +378,7 @@ class ScriptEngine:
         except AttributeError:
             pass
         self.request.pages = {}
-        caching.CacheEntry(self.request, 'wikidicts', 'dicts_groups').remove()
+        caching.CacheEntry(self.request, 'wikidicts', 'dicts_groups', scope='wiki').remove()
         page.clean_acl_cache()
 
     def runScript(self, commands):
@@ -298,7 +407,7 @@ class ScriptEngine:
                 self.goto -= 1
                 continue
 
-            if line.startswith("#"):
+            if line.startswith("#") or len(line) == 0:
                 continue
             elements = unpackLine(line)
             fnname = elements[0].strip().lower()
@@ -361,7 +470,7 @@ class Package:
 
     def isPackage(self):
         """ Returns true if this package is recognised. """
-        raise NotImplementedException
+        raise NotImplementedError
 
 class ZipPackage(Package, ScriptEngine):
     """ A package that reads its files from a .zip file. """
@@ -424,8 +533,8 @@ Example:
         request_url = "localhost/"
 
     # Setup MoinMoin environment
-    from MoinMoin.request import RequestCLI
-    request = RequestCLI(url = 'localhost/')
+    from MoinMoin.request import request_cli
+    request = request_cli.Request(url=request_url)
     request.form = request.args = request.setup_args()
 
     package = ZipPackage(request, packagefile)
@@ -442,6 +551,6 @@ Example:
             print "Installation failed."
         if package.msg:
             print package.msg
-    
+
 if __name__ == '__main__':
     main()
