@@ -80,6 +80,14 @@ class TestParagraphs(ParserTestCase):
             result = self.parse(text)
             assert re.search(r'<p.*?>\s*Paragraph\s*', result)
 
+    def testStrangeP(self):
+        """ parser.wiki: empty line separates paragraphs """
+        result = self.parse("""<<BR>> <<BR>>
+
+foo ''bar'' baz.
+""")
+        assert re.search(r'foo <em>bar</em> baz', result)
+
 
 class TestHeadings(ParserTestCase):
     """ Test various heading problems """
@@ -229,12 +237,12 @@ class TestCloseInlineTestCase(ParserTestCase):
         py.test.skip("Broken")
         cases = (
             # test, expected
-            ("text'''text\n", r"<p>text<strong>text\s*</strong></p>"),
-            ("text''text\n", r"<p>text<em>text\s*</em></p>"),
-            ("text__text\n", r"<p>text<span class=\"u\">text\s*</span></p>"),
+            ("text__text\n", r'<p[^>]*>text<span class="u">text\s*</span></p>'),
+            ("text''text\n", r'<p[^>]*>text<em>text\s*</em></p>'),
+            ("text'''text\n", r'<p[^>]*>text<strong>text\s*</strong></p>'),
             ("text ''em '''em strong __em strong underline",
-             r"text <em>em <strong>em strong <span class=\"u\">em strong underline"
-             r"\s*</span></strong></em></p>"),
+             r'text <em>em <strong>em strong <span class="u">em strong underline'
+             r'\s*</span></strong></em></p>'),
             )
         for test, expected in cases:
             result = self.parse(test)
@@ -293,22 +301,22 @@ class TestEscapeHTML(ParserTestCase):
 
     def testEscapeInGetTextMacro(self):
         """ parser.wiki: escape html markup in GetText macro """
-        test = "text <<GetText(<escape-me>)>> text"
+        test = u"text <<GetText(<escape-me>)>> text"
         self._test(test)
 
     def testEscapeInGetTextFormatted(self):
         """ parser.wiki: escape html markup in getText formatted call """
-        test = self.request.getText('<escape-me>', formatted=1)
+        test = self.request.getText('<escape-me>', wiki=True)
         self._test(test)
 
     def testEscapeInGetTextFormatedLink(self):
         """ parser.wiki: escape html markup in getText formatted call with link """
-        test = self.request.getText('[[<escape-me>]]', formatted=1)
+        test = self.request.getText('[[<escape-me>]]', wiki=True)
         self._test(test)
 
     def testEscapeInGetTextUnFormatted(self):
         """ parser.wiki: escape html markup in getText non formatted call """
-        test = self.request.getText('<escape-me>', formatted=0)
+        test = self.request.getText('<escape-me>', wiki=False)
         self._test(test)
 
     def _test(self, test):
@@ -486,8 +494,6 @@ You can use {{{brackets}}}
     def testManyNestingPreBrackets(self):
         """ tests two nestings  ({{{ }}} and {{{ }}}) in one line for the wiki parser
         """
-        py.test.skip("Broken")
-
         raw = """{{{{
 Test {{{brackets}}} and test {{{brackets}}}
 }}}}"""
@@ -573,6 +579,23 @@ class TestTransclusionMarkup(ParserTestCase):
             result = self.needle.search(html).group(1)
             assert result == expected
 
+class TestMacrosInOneLine(ParserTestCase):
+    """ Test macro formatting """
+    text = 'AAA %s AAA'
+    needle = re.compile(text % r'(.+)')
+    _tests = (
+        # test                              expected
+        (u'<<Verbatim(A)>><<Verbatim(a)>>', 'Aa'),
+        (u'<<Verbatim(A)>> <<Verbatim(a)>>', 'A a'),
+        )
+
+    def testMultipleMacrosInOneLine(self):
+        """ parser.wiki: multiple macros in one line and no linebreak """
+        for test, expected in self._tests:
+            html = self.parse(self.text % test)
+            result = self.needle.search(html).group(1)
+            assert result == expected
+
 
 coverage_modules = ['MoinMoin.parser.text_moin_wiki']
 
