@@ -3,13 +3,15 @@
     MoinMoin - MoinMoin Wiki Markup Parser
 
     @copyright: 2000-2002 Juergen Hermann <jh@web.de>,
-                2006-2007 MoinMoin:ThomasWaldmann,
+                2006-2008 MoinMoin:ThomasWaldmann,
                 2007 by MoinMoin:ReimarBauer
     @license: GNU GPL, see COPYING for details.
 """
 
 import re
-import logging
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin import config, wikiutil, macro
 from MoinMoin.Page import Page
@@ -17,6 +19,7 @@ from MoinMoin.support.python_compatibility import rsplit, set
 
 Dependencies = ['user'] # {{{#!wiki comment ... }}} has different output depending on the user's profile settings
 
+
 class Parser:
     """
         Parse wiki format markup (and call the formatter to generate output).
@@ -277,7 +280,7 @@ class Parser:
     \^  # off
 )|(?P<sub>
     ,,  # subscript on
-    (?P<sub_text>[^,]{1,40})  # capture 1..40 chars of text
+    (?P<sub_text>.*?)  # capture the text
     ,,  # off
 )|(?P<tt>
     \{\{\{  # teletype on
@@ -415,13 +418,11 @@ class Parser:
 
         self.in_table = 0
         self.inhibit_p = 0 # if set, do not auto-create a <p>aragraph
-        self.titles = request._page_headings
 
         # holds the nesting level (in chars) of open lists
         self.list_indents = []
         self.list_types = []
 
-
     def _close_item(self, result):
         #result.append("<!-- close item begin -->\n")
         if self.in_table:
@@ -634,12 +635,15 @@ class Parser:
     _url_target_repl = _url_repl
     _url_scheme_repl = _url_repl
 
-
     def _transclude_description(self, desc, default_text=''):
         """ parse a string <desc> valid as transclude description (text, ...)
-            and return "formatted" content (we do not pass it through the text
-            formatter, but only through wikiutil.escape as we only use it as
-            alt/title text).
+            and return the description.
+
+            We do NOT use wikiutil.escape here because it is html specific (the
+            html formatter, if used, does this for all html attributes).
+
+            We do NOT call formatter.text here because it sometimes is just used
+            for some alt and/or title attribute, but not emitted as text.
 
             @param desc: the transclude description to parse
             @param default_text: use this text if parsing desc returns nothing.
@@ -650,7 +654,6 @@ class Parser:
                 desc = m.group('simple_text')
         else:
             desc = default_text
-        desc = wikiutil.escape(desc)
         return desc
 
     def _get_params(self, params, tag_attrs=None, acceptable_attrs=None, query_args=None):
@@ -669,15 +672,12 @@ class Parser:
             if acceptable_attrs is None:
                 acceptable_attrs = []
             for key, val in kw.items():
-                key = str(key) # we can't use unicode as key
+                # wikiutil.escape for key/val must be done by (html) formatter!
                 if key in acceptable_attrs:
-                    key = wikiutil.escape(key)
-                    val = unicode(val) # but for the value
-                    val = wikiutil.escape(val)
-                    tag_attrs[key] = val
+                    # tag attributes must be string type
+                    tag_attrs[str(key)] = val
                 elif key.startswith('&'):
                     key = key[1:]
-                    val = unicode(val)
                     query_args[key] = val
         return tag_attrs, query_args
 
@@ -721,7 +721,8 @@ class Parser:
                     if mt.major == 'text':
                         desc = self._transclude_description(desc, url)
                         return self.formatter.attachment_inlined(url, desc)
-                    elif mt.major == 'image':
+                    # destinguishs if browser need a plugin in place
+                    elif mt.major == 'image' and mt.minor in config.browser_supported_images:
                         desc = self._transclude_description(desc, url)
                         tag_attrs, query_args = self._get_params(params,
                                                                  tag_attrs={'alt': desc,
@@ -737,27 +738,24 @@ class Parser:
                                                                      tag_attrs={'title': desc, },
                                                                      acceptable_attrs=acceptable_attrs_object)
                             return (self.formatter.transclusion(1, data=href, type=mt.spoil(), **tag_attrs) +
-                                    self._transclude_description(desc, url) +
+                                    self.formatter.text(self._transclude_description(desc, url)) +
                                     self.formatter.transclusion(0))
                         else:
                             return (self.formatter.attachment_link(1, url) +
-                                    self._transclude_description(desc, url) +
+                                    self.formatter.text(self._transclude_description(desc, url)) +
                                     self.formatter.attachment_link(0))
 
                         #NOT USED CURRENTLY:
 
                         # use EmbedObject for other mimetypes
-                        from MoinMoin.macro.EmbedObject import EmbedObject
-                        from MoinMoin.action import AttachFile
                         if mt is not None:
-                            # reuse class tmp from Despam to define macro
-                            from MoinMoin.action.Despam import tmp
-                            macro = tmp()
+                            from MoinMoin import macro
                             macro.request = self.request
                             macro.formatter = self.request.html_formatter
+                            p = Parser("##\n", request)
+                            m = macro.Macro(p)
                             pagename = self.formatter.page.page_name
-                            href = AttachFile.getAttachUrl(pagename, url, self.request, escaped=1)
-                            return self.formatter.rawHTML(EmbedObject.embed(EmbedObject(macro, wikiutil.escape(url)), mt, href))
+                            return m.execute('EmbedObject', u'target=%s' % url)
                 elif scheme == 'drawing':
                     desc = self._transclude_description(desc, url)
                     return self.formatter.attachment_drawing(url, desc)
@@ -777,9 +775,9 @@ class Parser:
                                                              acceptable_attrs=acceptable_attrs_object)
                     if 'action' not in query_args:
                         query_args['action'] = 'content'
-                    url = Page(self.request, page_name_all).url(self.request, querystr=query_args, relative=False)
+                    url = Page(self.request, page_name_all).url(self.request, querystr=query_args)
                     return (self.formatter.transclusion(1, data=url, **tag_attrs) +
-                            self._transclude_description(desc, page_name_all) +
+                            self.formatter.text(self._transclude_description(desc, page_name_all)) +
                             self.formatter.transclusion(0))
                     #return u"Error: <<Include(%s,%s)>> emulation missing..." % (page_name, args)
                 else: # looks like a valid interwiki link
@@ -792,7 +790,7 @@ class Parser:
                         query_args['action'] = 'content' # XXX moin specific
                     url += '?%s' % wikiutil.makeQueryString(query_args)
                     return (self.formatter.transclusion(1, data=url, **tag_attrs) +
-                            self._transclude_description(desc, page_name) +
+                            self.formatter.text(self._transclude_description(desc, page_name)) +
                             self.formatter.transclusion(0))
                     #return u"Error: <<RemoteInclude(%s:%s,%s)>> still missing." % (wiki_name, page_name, args)
 
@@ -1066,13 +1064,13 @@ class Parser:
                 token = parser.get_token()
                 if token != '%':
                     wanted = '%'
-                    msg = _('Expected "%(wanted)s" after "%(key)s", got "%(token)s"', formatted=False) % {
+                    msg = _('Expected "%(wanted)s" after "%(key)s", got "%(token)s"') % {
                         'wanted': wanted, 'key': key, 'token': token}
                 else:
                     try:
                         dummy = int(key)
                     except ValueError:
-                        msg = _('Expected an integer "%(key)s" before "%(token)s"', formatted=False) % {
+                        msg = _('Expected an integer "%(key)s" before "%(token)s"') % {
                             'key': key, 'token': token}
                     else:
                         found = True
@@ -1082,7 +1080,7 @@ class Parser:
                 try:
                     dummy = int(arg)
                 except ValueError:
-                    msg = _('Expected an integer "%(arg)s" after "%(key)s"', formatted=False) % {
+                    msg = _('Expected an integer "%(arg)s" after "%(key)s"') % {
                         'arg': arg, 'key': key}
                 else:
                     found = True
@@ -1092,7 +1090,7 @@ class Parser:
                 try:
                     dummy = int(arg)
                 except ValueError:
-                    msg = _('Expected an integer "%(arg)s" after "%(key)s"', formatted=False) % {
+                    msg = _('Expected an integer "%(arg)s" after "%(key)s"') % {
                         'arg': arg, 'key': key}
                 else:
                     found = True
@@ -1115,10 +1113,11 @@ class Parser:
             elif key == '#':
                 arg = parser.get_token()
                 try:
-                    if len(arg) != 6: raise ValueError
+                    if len(arg) != 6:
+                        raise ValueError
                     dummy = int(arg, 16)
                 except ValueError:
-                    msg = _('Expected a color value "%(arg)s" after "%(key)s"', formatted=False) % {
+                    msg = _('Expected a color value "%(arg)s" after "%(key)s"') % {
                         'arg': arg, 'key': key}
                 else:
                     found = True
@@ -1129,7 +1128,7 @@ class Parser:
         attr, msg = wikiutil.parseAttributes(self.request, attrdef, '>', table_extension)
         if msg:
             msg = '<strong class="highlight">%s</strong>' % msg
-        #self.request.log("parseAttributes returned %r" % attr)
+        #logging.debug("parseAttributes returned %r" % attr)
         return attr, msg
 
     def _tableZ_repl(self, word, groups):
@@ -1178,27 +1177,16 @@ class Parser:
         else:
             return self.formatter.text(word)
 
-
     def _heading_repl(self, word, groups):
         """Handle section headings."""
-        import sha
+        heading_text = groups.get('heading_text', '')
         depth = min(len(groups.get('hmarker')), 5)
-
-        # FIXME: needed for Included pages but might still result in unpredictable results
-        # when included the same page multiple times
-        title_text = groups.get('heading_text', '')
-        pntt = self.formatter.page.page_name + title_text
-        self.titles.setdefault(pntt, 0)
-        self.titles[pntt] += 1
-
-        unique_id = ''
-        if self.titles[pntt] > 1:
-            unique_id = '-%d' % self.titles[pntt]
-        result = self._closeP()
-        result += self.formatter.heading(1, depth, id="head-"+sha.new(pntt.encode(config.charset)).hexdigest()+unique_id)
-
-        return (result + self.formatter.text(title_text) +
-                self.formatter.heading(0, depth))
+        return ''.join([
+            self._closeP(),
+            self.formatter.heading(1, depth, id=heading_text),
+            self.formatter.text(heading_text),
+            self.formatter.heading(0, depth),
+        ])
     _heading_text_repl = _heading_repl
 
     def _parser_repl(self, word, groups):
@@ -1226,7 +1214,7 @@ class Parser:
         elif parser_nothing is None:
             # there was something non-whitespace following the {{{
             parser_name = 'text'
-        
+
         self.setParser(parser_name)
         if not self.parser and parser_name:
             # loading the desired parser didn't work, retry a safe option:
@@ -1242,7 +1230,7 @@ class Parser:
                 self.parser_lines.append(word)
         else:
             self.in_pre = 'search_parser'
-        
+
         #logging.debug("_parser_repl: in_pre %r line %d" % (self.in_pre, self.lineno))
         return ''
     _parser_unique_repl = _parser_repl
@@ -1316,7 +1304,7 @@ class Parser:
         """Handle macros."""
         macro_name = groups.get('macro_name')
         macro_args = groups.get('macro_args')
-        self.inhibit_p = 0 # 1 fixes UserPreferences, 0 fixes paragraph formatting for macros
+        self.inhibit_p = 0 # 1 fixed macros like UserPreferences (in the past, gone now), 0 fixes paragraph formatting for macros
 
         # create macro instance
         if self.macro is None:
@@ -1562,4 +1550,3 @@ class Parser:
         except wikiutil.PluginMissingError:
             self.parser = None
 
-
