@@ -15,16 +15,16 @@ def run():
     user = "XmlRpc"
     password = "wrong"
     dsttrans = BasicAuthTransport(user, password)
-    mywiki = xmlrpclib.ServerProxy("http://enterprise.wikiwikiweb.de:8888/?action=xmlrpc2", transport=dsttrans)
+    mywiki = xmlrpclib.ServerProxy("http://master.moinmo.in/?action=xmlrpc2", transport=dsttrans)
 
     groupname = "TestGroup"
     groupdesc = "This is just a test."
-    groupmembers = ["TestUser1", "TestUser2",]
+    groupmembers = ["TestUser1", "TestUser2", ]
     print mywiki.UpdateGroup(groupname, groupdesc, groupmembers)
 
     groupname = "TestAclGroup"
     groupdesc = "This is just a test."
-    groupmembers = ["TestUser3",]
+    groupmembers = ["TestUser3", ]
     print mywiki.UpdateGroup(groupname, groupdesc, groupmembers, "All:read,write,delete,revert")
 
     del mywiki
@@ -33,11 +33,11 @@ def run():
     user = "XmlRpc"
     password = "completelywrong"
     dsttrans = BasicAuthTransport(user, password)
-    mywiki = xmlrpclib.ServerProxy("http://enterprise.wikiwikiweb.de:8888/?action=xmlrpc2", transport=dsttrans)
+    mywiki = xmlrpclib.ServerProxy("http://master.moinmo.in/?action=xmlrpc2", transport=dsttrans)
 
     groupname = "TestGroup"
     groupdesc = "This is just a test."
-    groupmembers = ["WrongUser1", "WrongUser2",]
+    groupmembers = ["WrongUser1", "WrongUser2", ]
     print mywiki.UpdateGroup(groupname, groupdesc, groupmembers)
 
 
