@@ -1,17 +1,19 @@
 # -*- coding: iso-8859-1 -*-
 """
     MoinMoin - search query parser
-    
+
     @copyright: 2005 MoinMoin:FlorianFesti,
                 2005 MoinMoin:NirSoffer,
                 2005 MoinMoin:AlexanderSchremmer,
-                2006-2007 MoinMoin:ThomasWaldmann,
+                2006-2008 MoinMoin:ThomasWaldmann,
                 2006 MoinMoin:FranzPletz
     @license: GNU GPL, see COPYING for details
 """
 
 import re
-import logging
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin import config, wikiutil
 from MoinMoin.search.results import Match, TitleMatch, TextMatch
@@ -29,6 +31,8 @@ except ImportError:
 class BaseExpression:
     """ Base class for all search terms """
 
+    _tag = ""
+
     def __init__(self):
         self.negated = 0
 
@@ -65,7 +69,7 @@ class BaseExpression:
 
     def costs(self):
         """ Return estimated time to calculate this term
-        
+
         Number is relative to other terms and has no real unit.
         It allows to do the fast searches first.
         """
@@ -98,6 +102,10 @@ class BaseExpression:
     def xapian_wanted(self):
         return False
 
+    def __unicode__(self):
+        neg = self.negated and '-' or ''
+        return u'%s%s"%s"' % (neg, self._tag, unicode(self._pattern))
+
 
 class AndExpression(BaseExpression):
     """ A term connecting several sub terms with a logical AND """
@@ -151,7 +159,7 @@ class AndExpression(BaseExpression):
                         break
                     elif t is True:
                         result = True
-                logging.debug("search: pageFilter AND returns %r" % result)
+                logging.debug("pageFilter AND returns %r" % result)
                 return result
             return filter
 
@@ -205,23 +213,28 @@ class AndExpression(BaseExpression):
                 not_terms.append(term.xapian_term(request, allterms))
 
         # prepare query for not negated terms
-        if len(terms) == 1:
+        if not terms:
+            t1 = None
+        elif len(terms) == 1:
             t1 = Query(terms[0])
         else:
             t1 = Query(Query.OP_AND, terms)
 
-        # negated terms?
+        # prepare query for negated terms
         if not not_terms:
-            # no, just return query for not negated terms
-            return t1
-
-        # yes, link not negated and negated terms' query with a AND_NOT query
-        if len(not_terms) == 1:
+            t2 = None
+        elif len(not_terms) == 1:
             t2 = Query(not_terms[0])
         else:
             t2 = Query(Query.OP_OR, not_terms)
 
-        return Query(Query.OP_AND_NOT, t1, t2)
+        if t1 and not t2:
+            return t1
+        elif t2 and not t1:
+            return Query(Query.OP_AND_NOT, Query(""), t2)  # Query("") == MatchAll
+        else:
+            # yes, link not negated and negated terms' query with a AND_NOT query
+            return Query(Query.OP_AND_NOT, t1, t2)
 
 
 class OrExpression(AndExpression):
@@ -251,7 +264,7 @@ class OrExpression(AndExpression):
                         return True
                     elif t is False:
                         result = False
-                logging.debug("search: pageFilter OR returns %r" % result)
+                logging.debug("pageFilter OR returns %r" % result)
                 return result
             return filter
 
@@ -290,7 +303,7 @@ class TextSearch(BaseExpression):
 
         @param pattern: pattern to search for, ascii string or unicode
         @param use_re: treat pattern as re of plain text, bool
-        @param case: do case sensitive search, bool 
+        @param case: do case sensitive search, bool
         """
         self._pattern = unicode(pattern)
         self.negated = 0
@@ -298,19 +311,20 @@ class TextSearch(BaseExpression):
         self.case = case
         self._build_re(self._pattern, use_re=use_re, case=case)
         self.titlesearch = TitleSearch(self._pattern, use_re=use_re, case=case)
+        self._tag = ''
+        if use_re:
+            self._tag += 're:'
+        if case:
+            self._tag += 'case:'
 
     def costs(self):
         return 10000
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
         return u"(%s)" % self.pattern
 
     def search(self, page):
-        logging.debug("search: TextSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
+        logging.debug("TextSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
         matches = []
 
         # Search in page name
@@ -322,24 +336,7 @@ class TextSearch(BaseExpression):
         # Search in page body
         body = page.get_raw_body()
         for match in self.search_re.finditer(body):
-            if page.request.cfg.xapian_stemming: # XXX if True fails fulltext searches for "bcd" if there is content "abcd"
-                # somewhere in regular word
-                if body[match.start()] not in config.chars_upper and \
-                        body[match.start()-1] in config.chars_lower:
-                    continue
-
-                post = 0
-                # XXX only do this for stemmd words. how?
-                #for c in body[match.end():]:
-                #    if c in config.chars_lower:
-                #        post += 1
-                #    else:
-                #        break
-
-                matches.append(TextMatch(start=match.start(),
-                        end=match.end()+post))
-            else:
-                matches.append(TextMatch(re_match=match))
+            matches.append(TextMatch(re_match=match))
 
         # Decide what to do with the results.
         if self.negated:
@@ -352,7 +349,7 @@ class TextSearch(BaseExpression):
                 result = matches
             else:
                 result = None
-        logging.debug("search: TextSearch returning %r" % result)
+        logging.debug("TextSearch returning %r" % result)
         return result
 
     def xapian_wanted(self):
@@ -408,7 +405,7 @@ class TitleSearch(BaseExpression):
 
         @param pattern: pattern to search for, ascii string or unicode
         @param use_re: treat pattern as re of plain text, bool
-        @param case: do case sensitive search, bool 
+        @param case: do case sensitive search, bool
         """
         self._pattern = unicode(pattern)
         self.negated = 0
@@ -416,13 +413,15 @@ class TitleSearch(BaseExpression):
         self.case = case
         self._build_re(self._pattern, use_re=use_re, case=case)
 
+        self._tag = 'title:'
+        if use_re:
+            self._tag += 're:'
+        if case:
+            self._tag += 'case:'
+
     def costs(self):
         return 100
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s!"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
         return u'' # do not highlight text with stuff from titlesearch,
                    # was: return u"(%s)" % self._pattern
@@ -432,34 +431,16 @@ class TitleSearch(BaseExpression):
         def filter(name):
             match = self.search_re.search(name)
             result = bool(self.negated) ^ bool(match)
-            logging.debug("search: pageFilter title returns %r (%r)" % (result, self.pattern))
+            logging.debug("pageFilter title returns %r (%r)" % (result, self.pattern))
             return result
         return filter
 
     def search(self, page):
         """ Get matches in page name """
-        logging.debug("search: TitleSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
+        logging.debug("TitleSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
         matches = []
         for match in self.search_re.finditer(page.page_name):
-            if page.request.cfg.xapian_stemming: # XXX if True this fails title searches for "foo" even if there is a page "foo"
-                # somewhere in regular word
-                if not self.case and \
-                        page.page_name[match.start()] not in config.chars_upper and \
-                        page.page_name[match.start()-1] in config.chars_lower:
-                    continue
-
-                post = 0
-                # XXX only do this for stemmd words. how?
-                #for c in page.page_name[match.end():]:
-                #    if c in config.chars_lower:
-                #        post += 1
-                #    else:
-                #        break
-
-                matches.append(TitleMatch(start=match.start(),
-                        end=match.end()+post))
-            else:
-                matches.append(TitleMatch(re_match=match))
+            matches.append(TitleMatch(re_match=match))
 
         if self.negated:
             if matches:
@@ -471,10 +452,9 @@ class TitleSearch(BaseExpression):
                 result = matches
             else:
                 result = None
-        logging.debug("search: TitleSearch returning %r" % result)
+        logging.debug("TitleSearch returning %r" % result)
         return result
 
-
     def xapian_wanted(self):
         return True # only easy regexps possible
 
@@ -542,7 +522,7 @@ class LinkSearch(BaseExpression):
 
         @param pattern: pattern to search for, ascii string or unicode
         @param use_re: treat pattern as re of plain text, bool
-        @param case: do case sensitive search, bool 
+        @param case: do case sensitive search, bool
         """
         # used for search in links
         self._pattern = pattern
@@ -554,6 +534,12 @@ class LinkSearch(BaseExpression):
         self.textsearch = TextSearch(self._textpattern, use_re=1, case=case)
         self._build_re(unicode(pattern), use_re=use_re, case=case)
 
+        self._tag = 'linkto:'
+        if use_re:
+            self._tag += 're:'
+        if case:
+            self._tag += 'case:'
+
     def _build_re(self, pattern, use_re=False, case=False):
         """ Make a regular expression out of a text pattern """
         flags = case and re.U or (re.I | re.U)
@@ -568,16 +554,12 @@ class LinkSearch(BaseExpression):
     def costs(self):
         return 5000 # cheaper than a TextSearch
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s!"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
         return u"(%s)" % self._textpattern
 
     def search(self, page):
         # Get matches in page links
-        logging.debug("search: LinkSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
+        logging.debug("LinkSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
         matches = []
         Found = True
 
@@ -607,7 +589,7 @@ class LinkSearch(BaseExpression):
                 result = matches
             else:
                 result = None
-        logging.debug("search: LinkSearch returning %r" % result)
+        logging.debug("LinkSearch returning %r" % result)
         return result
 
     def xapian_wanted(self):
@@ -646,7 +628,7 @@ class LanguageSearch(BaseExpression):
 
         @param pattern: pattern to search for, ascii string or unicode
         @param use_re: treat pattern as re of plain text, bool
-        @param case: do case sensitive search, bool 
+        @param case: do case sensitive search, bool
         """
         # iso language code, always lowercase
         self._pattern = pattern.lower()
@@ -656,18 +638,20 @@ class LanguageSearch(BaseExpression):
         self.xapian_called = False
         self._build_re(self._pattern, use_re=use_re, case=case)
 
+        self._tag = 'language:'
+        if use_re:
+            self._tag += 're:'
+        if case:
+            self._tag += 'case:'
+
     def costs(self):
         return 5000 # cheaper than a TextSearch
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s!"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
         return u""
 
     def search(self, page):
-        logging.debug("search: LanguageSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
+        logging.debug("LanguageSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
         match = False
         body = page.getPageHeader()
 
@@ -690,7 +674,7 @@ class LanguageSearch(BaseExpression):
                 result = [Match()] # represents "matched" (but we have nothing to show)
             else:
                 result = None
-        logging.debug("search: LanguageSearch returning %r" % result)
+        logging.debug("LanguageSearch returning %r" % result)
         return result
 
     def xapian_wanted(self):
@@ -730,20 +714,29 @@ class CategorySearch(TextSearch):
         TextSearch.__init__(self, *args, **kwargs)
         self.titlesearch = None
 
+        self._tag = 'category:'
+
     def _build_re(self, pattern, **kwargs):
+        """ match categories like this:
+            ... some page text ...
+            ----
+            ## optionally some comments, e.g. about possible categories:
+            ## CategoryFoo
+            CategoryTheRealAndOnly
+
+            Note: there might be multiple comment lines, but all real categories
+                  must be on a single line either directly below the ---- or
+                  directly below some comment lines.
+        """
         kwargs['use_re'] = True
         TextSearch._build_re(self,
-                r'(----(-*)(\r)?\n)(.*)\b%s\b' % pattern, **kwargs)
+                r'(?m)(^-----*\s*\r?\n)(^##.*\r?\n)*^(?!##)(.*)\b%s\b' % pattern, **kwargs)
 
     def costs(self):
         return 5000 # cheaper than a TextSearch
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s!"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
-        return ur'(\b%s\b)' % self._pattern
+        return u'(\\b%s\\b)' % self._pattern
 
     def xapian_wanted(self):
         return True # only easy regexps possible
@@ -783,7 +776,7 @@ class MimetypeSearch(BaseExpression):
 
         @param pattern: pattern to search for, ascii string or unicode
         @param use_re: treat pattern as re of plain text, bool
-        @param case: do case sensitive search, bool 
+        @param case: do case sensitive search, bool
         """
         self._pattern = pattern.lower()
         self.negated = 0
@@ -792,18 +785,25 @@ class MimetypeSearch(BaseExpression):
         self.xapian_called = False
         self._build_re(self._pattern, use_re=use_re, case=case)
 
+        self._tag = 'mimetype:'
+        if use_re:
+            self._tag += 're:'
+        if case:
+            self._tag += 'case:'
+
     def costs(self):
         return 5000 # cheaper than a TextSearch
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s!"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
         return u""
 
     def search(self, page):
-        return None
+        page_mimetype = u'text/%s' % page.pi['format']
+        matches = self.search_re.search(page_mimetype)
+        if matches and not self.negated or not matches and self.negated:
+            return [Match()]
+        else:
+            return None
 
     def xapian_wanted(self):
         return True # only easy regexps possible
@@ -843,7 +843,7 @@ class DomainSearch(BaseExpression):
 
         @param pattern: pattern to search for, ascii string or unicode
         @param use_re: treat pattern as re of plain text, bool
-        @param case: do case sensitive search, bool 
+        @param case: do case sensitive search, bool
         """
         self._pattern = pattern.lower()
         self.negated = 0
@@ -852,18 +852,20 @@ class DomainSearch(BaseExpression):
         self.xapian_called = False
         self._build_re(self._pattern, use_re=use_re, case=case)
 
+        self._tag = 'domain:'
+        if use_re:
+            self._tag += 're:'
+        if case:
+            self._tag += 'case:'
+
     def costs(self):
         return 5000 # cheaper than a TextSearch
 
-    def __unicode__(self):
-        neg = self.negated and '-' or ''
-        return u'%s!"%s"' % (neg, unicode(self._pattern))
-
     def highlight_re(self):
         return u""
 
     def search(self, page):
-        logging.debug("search: DomainSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
+        logging.debug("DomainSearch searching page %r for (negated = %r) %r" % (page.page_name, self.negated, self._pattern))
         checks = {'underlay': page.isUnderlayPage,
                   'standard': page.isStandardPage,
                   'system': lambda page=page: wikiutil.isSystemPage(page.request, page.page_name),
@@ -885,7 +887,7 @@ class DomainSearch(BaseExpression):
                 result = [Match()] # represents "matched" (but we have nothing to show)
             else:
                 result = None
-        logging.debug("search: DomainSearch returning %r" % result)
+        logging.debug("DomainSearch returning %r" % result)
         return result
 
     def xapian_wanted(self):
@@ -924,8 +926,7 @@ class DomainSearch(BaseExpression):
 
 class QueryParser:
     """
-    Converts a String into a tree of Query objects
-    using recursive top/down parsing
+    Converts a String into a tree of Query objects.
     """
 
     def __init__(self, **kw):
@@ -937,119 +938,128 @@ class QueryParser:
         self.titlesearch = kw.get('titlesearch', 0)
         self.case = kw.get('case', 0)
         self.regex = kw.get('regex', 0)
+        self._M = wikiutil.ParserPrefix('-')
+
+    def _analyse_items(self, items):
+        terms = AndExpression()
+        M = self._M
+        while items:
+            item = items[0]
+            items = items[1:]
+
+            if isinstance(item, unicode):
+                if item.lower() == 'or':
+                    sub = terms.subterms()
+                    if len(sub) >= 1:
+                        last = sub[-1]
+                        if last.__class__ == OrExpression:
+                            orexpr = last
+                        else:
+                            # Note: do NOT reduce "terms" when it has a single subterm only!
+                            # Doing that would break "-someterm" searches as we rely on AndExpression
+                            # doing a "MatchAll AND_NOT someterm" for that case!
+                            orexpr = OrExpression(terms)
+                        terms = AndExpression(orexpr)
+                    else:
+                        raise ValueError('Nothing to OR')
+                    remaining = self._analyse_items(items)
+                    if remaining.__class__ == OrExpression:
+                        for sub in remaining.subterms():
+                            orexpr.append(sub)
+                    else:
+                        orexpr.append(remaining)
+                    break
+                elif item.lower() == 'and':
+                    pass
+                else:
+                    # odd workaround; we should instead ignore this term
+                    # and reject expressions that contain nothing after
+                    # being parsed rather than rejecting an empty string
+                    # before parsing...
+                    if not item:
+                        raise ValueError("Term too short")
+                    regex = self.regex
+                    case = self.case
+                    if self.titlesearch:
+                        terms.append(TitleSearch(item, use_re=regex, case=case))
+                    else:
+                        terms.append(TextSearch(item, use_re=regex, case=case))
+            elif isinstance(item, tuple):
+                negate = item[0] == M
+                title_search = self.titlesearch
+                regex = self.regex
+                case = self.case
+                linkto = False
+                lang = False
+                category = False
+                mimetype = False
+                domain = False
+                while len(item) > 1:
+                    m = item[0]
+                    if m is None:
+                        raise ValueError("Invalid search prefix")
+                    elif m == M:
+                        negate = True
+                    elif "title".startswith(m):
+                        title_search = True
+                    elif "regex".startswith(m):
+                        regex = True
+                    elif "case".startswith(m):
+                        case = True
+                    elif "linkto".startswith(m):
+                        linkto = True
+                    elif "language".startswith(m):
+                        lang = True
+                    elif "category".startswith(m):
+                        category = True
+                    elif "mimetype".startswith(m):
+                        mimetype = True
+                    elif "domain".startswith(m):
+                        domain = True
+                    else:
+                        raise ValueError("Invalid search prefix")
+                    item = item[1:]
+
+                text = item[0]
+                if category:
+                    obj = CategorySearch(text, use_re=regex, case=case)
+                elif mimetype:
+                    obj = MimetypeSearch(text, use_re=regex, case=False)
+                elif lang:
+                    obj = LanguageSearch(text, use_re=regex, case=False)
+                elif linkto:
+                    obj = LinkSearch(text, use_re=regex, case=case)
+                elif domain:
+                    obj = DomainSearch(text, use_re=regex, case=False)
+                elif title_search:
+                    obj = TitleSearch(text, use_re=regex, case=case)
+                else:
+                    obj = TextSearch(text, use_re=regex, case=case)
+                obj.negated = negate
+                terms.append(obj)
+            elif isinstance(item, list):
+                # strip off the opening parenthesis
+                terms.append(self._analyse_items(item[1:]))
+
+        # Note: do NOT reduce "terms" when it has a single subterm only!
+        # Doing that would break "-someterm" searches as we rely on AndExpression
+        # doing a "MatchAll AND_NOT someterm" for that case!
+        return terms
 
     def parse_query(self, query):
         """ transform an string into a tree of Query objects """
         if isinstance(query, str):
             query = query.decode(config.charset)
-        self._query = query
-        result = self._or_expression()
-        if result is None:
-            result = BaseExpression()
-        return result
-
-    def _or_expression(self):
-        result = self._and_expression()
-        if self._query:
-            result = OrExpression(result)
-        while self._query:
-            q = self._and_expression()
-            if q:
-                result.append(q)
-        return result
-
-    def _and_expression(self):
-        result = None
-        while not result and self._query:
-            result = self._single_term()
-        term = self._single_term()
-        if term:
-            result = AndExpression(result, term)
-        else:
-            return result
-        term = self._single_term()
-        while term:
-            result.append(term)
-            term = self._single_term()
-        return result
-
-    def _single_term(self):
-        regex = (r'(?P<NEG>-?)\s*(' +              # leading '-'
-                 r'(?P<OPS>\(|\)|(or\b(?!$)))|' +  # or, (, )
-                 r'(?P<MOD>(\w+:)*)' +
-                 r'(?P<TERM>("[^"]+")|' +
-                 r"('[^']+')|([^\s\)]+)))")        # search word itself
-        self._query = self._query.strip()
-        match = re.match(regex, self._query, re.U)
-        if not match:
-            return None
-        self._query = self._query[match.end():]
-        ops = match.group("OPS")
-        if ops == '(':
-            result = self._or_expression()
-            if match.group("NEG"):
-                result.negate()
-            return result
-        elif ops == ')':
-            return None
-        elif ops == 'or':
-            return None
-        modifiers = match.group('MOD').split(":")[:-1]
-        text = match.group('TERM')
-        if self.isQuoted(text):
-            text = text[1:-1]
-
-        title_search = self.titlesearch
-        regex = self.regex
-        case = self.case
-        linkto = False
-        lang = False
-        category = False
-        mimetype = False
-        domain = False
-
-        for m in modifiers:
-            if "title".startswith(m):
-                title_search = True
-            elif "regex".startswith(m):
-                regex = True
-            elif "case".startswith(m):
-                case = True
-            elif "linkto".startswith(m):
-                linkto = True
-            elif "language".startswith(m):
-                lang = True
-            elif "category".startswith(m):
-                category = True
-            elif "mimetype".startswith(m):
-                mimetype = True
-            elif "domain".startswith(m):
-                domain = True
-
-        if category:
-            obj = CategorySearch(text, use_re=regex, case=case)
-        elif mimetype:
-            obj = MimetypeSearch(text, use_re=regex, case=False)
-        elif lang:
-            obj = LanguageSearch(text, use_re=regex, case=False)
-        elif linkto:
-            obj = LinkSearch(text, use_re=regex, case=case)
-        elif domain:
-            obj = DomainSearch(text, use_re=regex, case=False)
-        elif title_search:
-            obj = TitleSearch(text, use_re=regex, case=case)
-        else:
-            obj = TextSearch(text, use_re=regex, case=case)
-
-        if match.group("NEG"):
-            obj.negate()
-        return obj
-
-    def isQuoted(self, text):
-        # Empty string '' is not considered quoted
-        if len(text) < 3:
-            return False
-        return (text.startswith('"') and text.endswith('"') or
-                text.startswith("'") and text.endswith("'"))
-
-
+        try:
+            items = wikiutil.parse_quoted_separated_ext(query,
+                                                        name_value_separator=':',
+                                                        prefixes='-',
+                                                        multikey=True,
+                                                        brackets=('()', ),
+                                                        quotes='\'"')
+            logging.debug("parse_quoted_separated items: %r" % items)
+        except wikiutil.BracketError, err:
+            raise ValueError(str(err))
+        query = self._analyse_items(items)
+        logging.debug("analyse_items query: %r" % query)
+        return query
