@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 """
     MoinMoin - Text CAPTCHAs
-    
+
     This is just asking some (admin configured) questions and
     checking if the answer is as expected. It is up to the wiki
     admin to setup questions that a bot can not easily answer, but
@@ -14,6 +14,8 @@
     * roundtrip the question in some other way:
      * use safe encoding / encryption for the q
      * make sure a q/a pair in the POST is for the q in the GET before
+    * make some nice CSS
+    * make similar changes to GUI editor
 
     @copyright: 2007 by MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
@@ -21,7 +23,9 @@
 
 import re
 import random
-import logging
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin import wikiutil
 
@@ -50,10 +54,10 @@ class TextCha(object):
         textchas = cfg.textchas
         if textchas:
             lang = user.language or request.lang
-            #logging.debug(u"TextCha: user.language == '%s'." % lang)
+            logging.debug(u"TextCha: user.language == '%s'." % lang)
             if lang not in textchas:
                 lang = cfg.language_default
-                #logging.debug(u"TextCha: fallback to language_default == '%s'." % lang)
+                logging.debug(u"TextCha: fallback to language_default == '%s'." % lang)
                 if lang not in textchas:
                     logging.error(u"TextCha: The textchas do not have content for language_default == '%s'! Falling back to English." % lang)
                     lang = 'en'
@@ -66,12 +70,12 @@ class TextCha(object):
         if lang is None:
             return None
         else:
-            #logging.debug(u"TextCha: using lang = '%s'" % lang)
+            logging.debug(u"TextCha: using lang = '%s'" % lang)
             return textchas[lang]
 
     def _init_qa(self, question=None):
         """ Initialize the question / answer.
-        
+
          @param question: If given, the given question will be used.
                           If None, a new question will be generated.
         """
@@ -105,7 +109,7 @@ class TextCha(object):
                 },
                 'de': {}, # having no questions for 'de' means disabling textchas for 'de'
                 # ...
-            }             
+            }
         """
         return not not self.textchas # we don't want to return the dict
 
@@ -140,8 +144,8 @@ class TextCha(object):
     def render(self, form=None):
         """ Checks if textchas are enabled and returns HTML for one,
             or an empty string if they are not enabled.
-            
-            @return: unicode result html         
+
+            @return: unicode result html
         """
         if self.is_enabled():
             question, given_answer = self._extract_form_values(form)
@@ -166,3 +170,4 @@ class TextCha(object):
             return self.check_answer(given_answer)
         else:
             return True
+
