@@ -2,22 +2,31 @@
 """
     MoinMoin - User Accounts
 
-    @copyright: 2000-2004 by Jrgen Hermann <jh@web.de>
+    This module contains functions to access user accounts (list all users, get
+    some specific user). User instances are used to access the user profile of
+    some specific user (name, password, email, bookmark, trail, settings, ...).
+
+    Some related code is in the userform and userprefs modules.
+
+    TODO:
+    * code is a mixture of highlevel user stuff and lowlevel storage functions,
+      this has to get separated into:
+      * user object highlevel stuff
+      * storage code
+
+    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
+                2003-2007 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
 
-import os, time, sha, codecs
+# add names here to hide them in the cgitb traceback
+unsafe_names = ("id", "key", "val", "user_data", "enc_password", "recoverpass_key")
 
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
+import os, time, sha, codecs, hmac
 
-# Set pickle protocol, see http://docs.python.org/lib/node64.html
-PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
-
-from MoinMoin import config, caching, wikiutil
-from MoinMoin.util import filesys, timefuncs
+from MoinMoin import config, caching, wikiutil, i18n, events
+from MoinMoin.util import timefuncs, filesys, random_string
+from MoinMoin.wikiutil import url_quote_plus
 
 
 def getUserList(request):
@@ -27,53 +36,97 @@ def getUserList(request):
     @rtype: list
     @return: all user IDs
     """
-    import re, dircache
+    import re
     user_re = re.compile(r'^\d+\.\d+(\.\d+)?$')
-    files = dircache.listdir(request.cfg.user_dir)
+    files = filesys.dclistdir(request.cfg.user_dir)
     userlist = [f for f in files if user_re.match(f)]
     return userlist
 
+def get_by_filter(request, filter_func):
+    """ Searches for an user with a given filter function """
+    for uid in getUserList(request):
+        theuser = User(request, uid)
+        if filter_func(theuser):
+            return theuser
+
+def get_by_email_address(request, email_address):
+    """ Searches for an user with a particular e-mail address and returns it. """
+    filter_func = lambda user: user.valid and user.email.lower() == email_address.lower()
+    return get_by_filter(request, filter_func)
+
+def get_by_jabber_id(request, jabber_id):
+    """ Searches for an user with a perticular jabber id and returns it. """
+    filter_func = lambda user: user.valid and user.jid.lower() == jabber_id.lower()
+    return get_by_filter(request, filter_func)
+
+def _getUserIdByKey(request, key, search):
+    """ Get the user ID for a specified key/value pair.
+
+    This method must only be called for keys that are
+    guaranteed to be unique.
+
+    @param key: the key to look in
+    @param search: the value to look for
+    @return the corresponding user ID or None
+    """
+    if not search or not key:
+        return None
+    cfg = request.cfg
+    cachekey = '%s2id' % key
+    try:
+        _key2id = getattr(cfg.cache, cachekey)
+    except AttributeError:
+        arena = 'user'
+        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
+        try:
+            _key2id = cache.content()
+        except caching.CacheError:
+            _key2id = {}
+        setattr(cfg.cache, cachekey, _key2id)
+    uid = _key2id.get(search, None)
+    if uid is None:
+        for userid in getUserList(request):
+            u = User(request, id=userid)
+            if hasattr(u, key):
+                value = getattr(u, key)
+                if isinstance(value, list):
+                    for val in value:
+                        _key2id[val] = userid
+                else:
+                    _key2id[value] = userid
+        arena = 'user'
+        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
+        try:
+            cache.update(_key2id)
+        except caching.CacheError:
+            pass
+        uid = _key2id.get(search, None)
+    return uid
+
 
 def getUserId(request, searchName):
-    """
-    Get the user ID for a specific user NAME.
+    """ Get the user ID for a specific user NAME.
 
     @param searchName: the user name to look up
     @rtype: string
     @return: the corresponding user ID or None
     """
-    if not searchName:
-        return None
-    cfg = request.cfg
-    try:
-        _name2id = cfg._name2id
-    except AttributeError:
-        arena = 'user'
-        key = 'name2id'
-        cache = caching.CacheEntry(request, arena, key)
-        try:
-            _name2id = pickle.loads(cache.content())
-        except (pickle.UnpicklingError, IOError, EOFError, ValueError):
-            _name2id = {}
-        cfg._name2id = _name2id
-    id = _name2id.get(searchName, None)
-    if id is None:
-        for userid in getUserList(request):
-            name = User(request, id=userid).name
-            _name2id[name] = userid
-        cfg._name2id = _name2id
-        arena = 'user'
-        key = 'name2id'
-        cache = caching.CacheEntry(request, arena, key)
-        cache.update(pickle.dumps(_name2id, PICKLE_PROTOCOL))
-        id = _name2id.get(searchName, None)
-    return id
+    return _getUserIdByKey(request, 'name', searchName)
+
+
+def getUserIdByOpenId(request, openid):
+    """ Get the user ID for a specific OpenID.
+
+    @param openid: the openid to look up
+    @rtype: string
+    @return: the corresponding user ID or None
+    """
+    return _getUserIdByKey(request, 'openids', openid)
 
 
 def getUserIdentification(request, username=None):
-    """ 
-    Return user name or IP or '<unknown>' indicator.
-    
+    """ Return user name or IP or '<unknown>' indicator.
+
     @param request: the request object
     @param username: (optional) user name
     @rtype: string
@@ -112,7 +165,7 @@ def encodePassword(pwd, charset='utf-8'):
     pwd = '{SHA}' + base64.encodestring(pwd).rstrip()
     return pwd
 
-    
+
 def normalizeName(name):
     """ Make normalized user name
 
@@ -124,31 +177,30 @@ def normalizeName(name):
     in user names.
 
     Prevent using ':' and ',' which are reserved by acl.
-    
+
     @param name: user name, unicode
     @rtype: unicode
     @return: user name that can be used in acl lines
     """
-    name = name.replace('_', ' ') # we treat _ as a blank
-    username_allowedchars = "'@.-" # ' for names like O'Brian or email addresses.
-                                   # "," and ":" must not be allowed (ACL delimiters).
+    username_allowedchars = "'@.-_" # ' for names like O'Brian or email addresses.
+                                    # "," and ":" must not be allowed (ACL delimiters).
+                                    # We also allow _ in usernames for nicer URLs.
     # Strip non alpha numeric characters (except username_allowedchars), keep white space
     name = ''.join([c for c in name if c.isalnum() or c.isspace() or c in username_allowedchars])
 
-    # Normalize white space. Each name can contain multiple 
+    # Normalize white space. Each name can contain multiple
     # words separated with only one space.
     name = ' '.join(name.split())
 
     return name
 
-    
+
 def isValidName(request, name):
     """ Validate user name
 
     @param name: user name, unicode
     """
     normalized = normalizeName(name)
-    name = name.replace('_', ' ') # we treat _ as a blank
     return (name == normalized) and not wikiutil.isGroupPage(request, name)
 
 
@@ -156,7 +208,7 @@ def encodeList(items):
     """ Encode list of items in user data file
 
     Items are separated by '\t' characters.
-    
+
     @param items: list unicode strings
     @rtype: unicode
     @return: list encoded as unicode
@@ -167,14 +219,13 @@ def encodeList(items):
         if not item:
             continue
         line.append(item)
-        
+
     line = '\t'.join(line)
     return line
 
-
 def decodeList(line):
     """ Decode list of items from user data file
-    
+
     @param line: line containing list of items, encoded with encodeList
     @rtype: list of unicode strings
     @return: list of items in encoded in line
@@ -187,13 +238,48 @@ def decodeList(line):
         items.append(item)
     return items
 
-    
+def encodeDict(items):
+    """ Encode dict of items in user data file
+
+    Items are separated by '\t' characters.
+    Each item is key:value.
+
+    @param items: dict of unicode:unicode
+    @rtype: unicode
+    @return: dict encoded as unicode
+    """
+    line = []
+    for key, value in items.items():
+        item = u'%s:%s' % (key, value)
+        line.append(item)
+    line = '\t'.join(line)
+    return line
+
+def decodeDict(line):
+    """ Decode dict of key:value pairs from user data file
+
+    @param line: line containing a dict, encoded with encodeDict
+    @rtype: dict
+    @return: dict  unicode:unicode items
+    """
+    items = {}
+    for item in line.split('\t'):
+        item = item.strip()
+        if not item:
+            continue
+        key, value = item.split(':', 1)
+        items[key] = value
+    return items
+
+
 class User:
-    """A MoinMoin User"""
+    """ A MoinMoin User """
 
     def __init__(self, request, id=None, name="", password=None, auth_username="", **kw):
         """ Initialize User object
-        
+
+        TODO: when this gets refactored, use "uid" not builtin "id"
+
         @param request: the request object
         @param id: (optional) user ID
         @param name: (optional) user name
@@ -204,23 +290,20 @@ class User:
                               default: 'internal'
         @keyword auth_attribs: tuple of user object attribute names that are
                                determined by auth method and should not be
-                               changed by UserPreferences form, default: ().
+                               changeable by preferences, default: ().
                                First tuple element was used for authentication.
         """
         self._cfg = request.cfg
         self.valid = 0
-        self.trusted = 0
         self.id = id
         self.auth_username = auth_username
         self.auth_method = kw.get('auth_method', 'internal')
         self.auth_attribs = kw.get('auth_attribs', ())
-                                       
+        self.bookmarks = {} # interwikiname: bookmark
+
         # create some vars automatically
-        for tuple in self._cfg.user_form_fields:
-            key = tuple[0]
-            default = self._cfg.user_form_defaults.get(key, '')
-            setattr(self, key, default)
-       
+        self.__dict__.update(self._cfg.user_form_defaults)
+
         if name:
             self.name = name
         elif auth_username: # this is needed for user_autocreate
@@ -230,28 +313,31 @@ class User:
         for key, label in self._cfg.user_checkbox_fields:
             setattr(self, key, self._cfg.user_checkbox_defaults.get(key, 0))
 
+        self.recoverpass_key = ""
+
         self.enc_password = ""
         if password:
-            if password.startswith('{SHA}'):
-                self.enc_password = password
-            else:
-                try:
-                    self.enc_password = encodePassword(password)
-                except UnicodeError:
-                    pass # Should never happen
+            try:
+                self.enc_password = encodePassword(password)
+            except UnicodeError:
+                pass # Should never happen
 
         #self.edit_cols = 80
         self.tz_offset = int(float(self._cfg.tz_offset) * 3600)
         self.language = ""
+        self.real_language = "" # In case user uses "Browser setting". For language-statistics
+        self._stored = False
         self.date_fmt = ""
         self.datetime_fmt = ""
-        self.quicklinks = []
-        self.subscribed_pages = []
+        self.quicklinks = self._cfg.quicklinks_default
+        self.subscribed_pages = self._cfg.subscribed_pages_default
+        self.email_subscribed_events = self._cfg.email_subscribed_events_default
+        self.jabber_subscribed_events = self._cfg.jabber_subscribed_events_default
         self.theme_name = self._cfg.theme_default
         self.editor_default = self._cfg.editor_default
         self.editor_ui = self._cfg.editor_ui
         self.last_saved = str(time.time())
-        
+
         # attrs not saved to profile
         self._request = request
         self._trail = []
@@ -264,8 +350,6 @@ class User:
                 check_pass = 1
         if self.id:
             self.load_from_id(check_pass)
-            if self.name == self.auth_username:
-                self.trusted = 1
         elif self.name:
             self.id = getUserId(self._request, self.name)
             if self.id:
@@ -281,15 +365,14 @@ class User:
         else:
             from MoinMoin.security import Default
             self.may = Default(self)
-        
-        from MoinMoin.i18n.meta import languages
-        if self.language and not languages.has_key(self.language):
+
+        if self.language and not self.language in i18n.wikiLanguages():
             self.language = 'en'
 
     def __repr__(self):
-        return "<%s.%s at 0x%x name:%r id:%s valid:%r>" % (
+        return "<%s.%s at 0x%x name:%r valid:%r>" % (
             self.__class__.__module__, self.__class__.__name__,
-            id(self), self.name, self.id, self.valid)
+            id(self), self.name, self.valid)
 
     def make_id(self):
         """ make a new unique user id """
@@ -297,7 +380,7 @@ class User:
         # and some other things identifying remote users, then we could also
         # use it reliably in edit locking
         from random import randint
-        return "%s.%d" % (str(time.time()), randint(0,65535))
+        return "%s.%d" % (str(time.time()), randint(0, 65535))
 
     def create_or_update(self, changed=False):
         """ Create or update a user profile
@@ -307,25 +390,18 @@ class User:
         if self._cfg.user_autocreate:
             if not self.valid and not self.disabled or changed: # do we need to save/update?
                 self.save() # yes, create/update user profile
-                                
+
     def __filename(self):
         """ Get filename of the user's file on disk
-        
+
         @rtype: string
         @return: full path and filename of user account file
         """
         return os.path.join(self._cfg.user_dir, self.id or "...NONE...")
 
-    def __bookmark_filename(self):
-        if self._cfg.interwikiname:
-            return (self.__filename() + "." + self._cfg.interwikiname +
-                    ".bookmark")
-        else:
-            return self.__filename() + ".bookmark"
-
     def exists(self):
         """ Do we have a user account for this user?
-        
+
         @rtype: bool
         @return: true, if we have a user account
         """
@@ -338,7 +414,7 @@ class User:
 
         This loads all member variables, except "id" and "valid" and
         those starting with an underscore.
-        
+
         @param check_pass: If 1, then self.enc_password must match the
                            password in the user account file.
         """
@@ -355,7 +431,16 @@ class User:
                 key, val = line.strip().split('=', 1)
                 if key not in self._cfg.user_transient_fields and key[0] != '_':
                     # Decode list values
-                    if key in ['quicklinks', 'subscribed_pages']:
+                    if key.endswith('[]'):
+                        key = key[:-2]
+                        val = decodeList(val)
+                    # Decode dict values
+                    elif key.endswith('{}'):
+                        key = key[:-2]
+                        val = decodeDict(val)
+                    # for compatibility reading old files, keep these explicit
+                    # we will store them with [] appended
+                    elif key in ['quicklinks', 'subscribed_pages', 'subscribed_events']:
                         val = decodeList(val)
                     user_data[key] = val
             except ValueError:
@@ -371,14 +456,12 @@ class User:
                 return
             # Check for a valid password, possibly changing encoding
             valid, changed = self._validatePassword(user_data)
-            if not valid: 
+            if not valid:
                 return
-            else:
-                self.trusted = 1
 
         # Remove ignored checkbox values from user data
         for key, label in self._cfg.user_checkbox_fields:
-            if user_data.has_key(key) and key in self._cfg.user_checkbox_disable:
+            if key in user_data and key in self._cfg.user_checkbox_disable:
                 del user_data[key]
 
         # Copy user data into user object
@@ -393,7 +476,7 @@ class User:
             if hasattr(self, attr):
                 delattr(self, attr)
                 changed = 1
-        
+
         # make sure checkboxes are boolean
         for key, label in self._cfg.user_checkbox_fields:
             try:
@@ -411,6 +494,10 @@ class User:
         if not self.disabled:
             self.valid = 1
 
+        # Mark this user as stored so saves don't send
+        # the "user created" event
+        self._stored = True
+
         # If user data has been changed, save fixed user data.
         if changed:
             self.save()
@@ -456,12 +543,12 @@ class User:
 
         # Get the clear text password from the form (require non empty
         # password)
-        password = self._request.form.get('password',[None])[0]
+        password = self._request.form.get('password', [None])[0]
         if not password:
-            return False, False 
-                
+            return False, False
+
         # First get all available pre13 charsets on this system
-        pre13 = ['iso-8859-1', 'iso-8859-2', 'euc-jp', 'gb2312', 'big5',]
+        pre13 = ['iso-8859-1', 'iso-8859-2', 'euc-jp', 'gb2312', 'big5', ]
         available = []
         for charset in pre13:
             try:
@@ -469,7 +556,7 @@ class User:
                 available.append(charset)
             except LookupError:
                 pass # missing on this system
-                
+
         # Now try to match the password
         for charset in available:
             # Try to encode, failure is expected
@@ -488,6 +575,11 @@ class User:
         # No encoded password match, this must be wrong password
         return False, False
 
+    def persistent_items(self):
+        """ items we want to store into the user profile """
+        return [(key, value) for key, value in vars(self).items()
+                    if key not in self._cfg.user_transient_fields and key[0] != '_']
+
     def save(self):
         """ Save user account data to user account file on disk.
 
@@ -498,37 +590,61 @@ class User:
             return
 
         user_dir = self._cfg.user_dir
-        filesys.makeDirs(user_dir)
+        if not os.path.exists(user_dir):
+            os.makedirs(user_dir)
 
         self.last_saved = str(time.time())
 
         # !!! should write to a temp file here to avoid race conditions,
         # or even better, use locking
-        
+
         data = codecs.open(self.__filename(), "w", config.charset)
         data.write("# Data saved '%s' for id '%s'\n" % (
             time.strftime(self._cfg.datetime_fmt, time.localtime(time.time())),
             self.id))
-        attrs = vars(self).items()
+        attrs = self.persistent_items()
         attrs.sort()
         for key, value in attrs:
-            if key not in self._cfg.user_transient_fields and key[0] != '_':
-                # Encode list values
-                if key in ['quicklinks', 'subscribed_pages']:
-                    value = encodeList(value)
-                line = u"%s=%s\n" % (key, unicode(value))
-                data.write(line)
+            # Encode list values
+            if isinstance(value, list):
+                key += '[]'
+                value = encodeList(value)
+            # Encode dict values
+            elif isinstance(value, dict):
+                key += '{}'
+                value = encodeDict(value)
+            line = u"%s=%s\n" % (key, unicode(value))
+            data.write(line)
         data.close()
 
+        arena = 'user'
+        key = 'name2id'
+        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
+        try:
+            del self._request.cfg.cache.name2id
+        except:
+            pass
+        key = 'openid2id'
+        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
+        try:
+            del self._request.cfg.cache.openid2id
+        except:
+            pass
+
         if not self.disabled:
             self.valid = 1
 
+        if not self._stored:
+            self._stored = True
+            event = events.UserCreatedEvent(self._request, self)
+            events.send_event(event)
+
     # -----------------------------------------------------------------
     # Time and date formatting
 
     def getTime(self, tm):
         """ Get time in user's timezone.
-        
+
         @param tm: time (UTC UNIX timestamp)
         @rtype: int
         @return: tm tuple adjusted for user's timezone
@@ -562,28 +678,27 @@ class User:
 
     def setBookmark(self, tm):
         """ Set bookmark timestamp.
-        
+
         @param tm: timestamp
         """
         if self.valid:
-            bm_fn = self.__bookmark_filename()
-            bmfile = open(bm_fn, "w")
-            bmfile.write(str(tm)+"\n")
-            bmfile.close()
+            interwikiname = unicode(self._cfg.interwikiname or '')
+            bookmark = unicode(tm)
+            self.bookmarks[interwikiname] = bookmark
+            self.save()
 
     def getBookmark(self):
         """ Get bookmark timestamp.
-        
+
         @rtype: int
         @return: bookmark timestamp or None
         """
         bm = None
-        bm_fn = self.__bookmark_filename()
-
-        if self.valid and os.path.exists(bm_fn):
+        interwikiname = unicode(self._cfg.interwikiname or '')
+        if self.valid:
             try:
-                bm = long(open(bm_fn, 'r').readline()) # must be long for py 2.2
-            except (OSError, ValueError):
+                bm = int(self.bookmarks[interwikiname])
+            except (ValueError, KeyError):
                 pass
         return bm
 
@@ -593,13 +708,13 @@ class User:
         @rtype: int
         @return: 0 on success, 1 on failure
         """
-        bm_fn = self.__bookmark_filename()
+        interwikiname = unicode(self._cfg.interwikiname or '')
         if self.valid:
-            if os.path.exists(bm_fn):
-                try:
-                    os.unlink(bm_fn)
-                except OSError:
-                    return 1
+            try:
+                del self.bookmarks[interwikiname]
+            except KeyError:
+                return 1
+            self.save()
             return 0
         return 1
 
@@ -608,36 +723,36 @@ class User:
 
     def getSubscriptionList(self):
         """ Get list of pages this user has subscribed to
-        
+
         @rtype: list
         @return: pages this user has subscribed to
         """
         return self.subscribed_pages
-        
+
     def isSubscribedTo(self, pagelist):
         """ Check if user subscription matches any page in pagelist.
-        
+
         The subscription list may contain page names or interwiki page
         names. e.g 'Page Name' or 'WikiName:Page_Name'
-        
-        TODO: check if its fast enough when calling with many users
-        from page.getSubscribersList()
-        
+
+        TODO: check if it's fast enough when getting called for many
+              users from page.getSubscribersList()
+
         @param pagelist: list of pages to check for subscription
         @rtype: bool
         @return: if user is subscribed any page in pagelist
         """
         if not self.valid:
             return False
-        
-        import re        
+
+        import re
         # Create a new list with both names and interwiki names.
         pages = pagelist[:]
         if self._cfg.interwikiname:
             pages += [self._interWikiName(pagename) for pagename in pagelist]
         # Create text for regular expression search
         text = '\n'.join(pages)
-        
+
         for pattern in self.getSubscriptionList():
             # Try simple match first
             if pattern in pages:
@@ -662,15 +777,20 @@ class User:
         @type pagename: unicode
         @rtype: bool
         @return: if page was subscribed
-        """        
+        """
         if self._cfg.interwikiname:
             pagename = self._interWikiName(pagename)
-        
+
         if pagename not in self.subscribed_pages:
             self.subscribed_pages.append(pagename)
             self.save()
+
+            # Send a notification
+            from MoinMoin.events import SubscribedToPageEvent, send_event
+            e = SubscribedToPageEvent(self._request, pagename, self.name)
+            send_event(e)
             return True
-        
+
         return False
 
     def unsubscribe(self, pagename):
@@ -686,9 +806,6 @@ class User:
         must check if the user is still subscribed to the page after we
         try to remove names from the list.
 
-        TODO: should we remove non-interwiki subscription? what if the
-        user want to subscribe to the same page in multiple wikis?
-
         @param pagename: name of the page to subscribe
         @type pagename: unicode
         @rtype: bool
@@ -699,16 +816,16 @@ class User:
         if pagename in self.subscribed_pages:
             self.subscribed_pages.remove(pagename)
             changed = True
-        
-        interWikiName = self._interWikiName(pagename)        
+
+        interWikiName = self._interWikiName(pagename)
         if interWikiName and interWikiName in self.subscribed_pages:
             self.subscribed_pages.remove(interWikiName)
             changed = True
-    
+
         if changed:
             self.save()
         return not self.isSubscribedTo([pagename])
-        
+
     # -----------------------------------------------------------------
     # Quicklinks
 
@@ -722,26 +839,26 @@ class User:
 
     def isQuickLinkedTo(self, pagelist):
         """ Check if user quicklink matches any page in pagelist.
-        
+
         @param pagelist: list of pages to check for quicklinks
         @rtype: bool
         @return: if user has quicklinked any page in pagelist
         """
         if not self.valid:
             return False
-            
+
         for pagename in pagelist:
             if pagename in self.quicklinks:
                 return True
             interWikiName = self._interWikiName(pagename)
             if interWikiName and interWikiName in self.quicklinks:
                 return True
-        
+
         return False
 
     def addQuicklink(self, pagename):
-        """ Adds a page to the user quicklinks 
-        
+        """ Adds a page to the user quicklinks
+
         If the wiki has an interwiki name, all links are saved as
         interwiki names. If not, as simple page name.
 
@@ -769,10 +886,10 @@ class User:
         return changed
 
     def removeQuicklink(self, pagename):
-        """ Remove a page from user quicklinks 
-        
+        """ Remove a page from user quicklinks
+
         Remove both interwiki and simple name from quicklinks.
-        
+
         @param pagename: page name
         @type pagename: unicode
         @rtype: bool
@@ -785,47 +902,43 @@ class User:
             changed = True
         if pagename in self.quicklinks:
             self.quicklinks.remove(pagename)
-            changed = True        
-        
+            changed = True
+
         if changed:
             self.save()
         return changed
 
     def _interWikiName(self, pagename):
-        """ Return the inter wiki name of a page name 
+        """ Return the inter wiki name of a page name
 
         @param pagename: page name
         @type pagename: unicode
         """
         if not self._cfg.interwikiname:
             return None
-            
-        # Interwiki links must use _ e.g Wiki:Main_Page
-        pagename = pagename.replace(" ", "_")
+
         return "%s:%s" % (self._cfg.interwikiname, pagename)
 
     # -----------------------------------------------------------------
     # Trail
 
-    def addTrail(self, pagename):
+    def _wantTrail(self):
+        return (not self.valid and self._request.session.is_stored  # anon session
+                or self.valid and (self.show_page_trail or self.remember_last_visit))  # logged-in session
+
+    def addTrail(self, page):
         """ Add page to trail.
-        
-        @param pagename: the page name to add to the trail
+
+        @param page: the page (object) to add to the trail
         """
-        # TODO: acquire lock here, so multiple processes don't clobber
-        # each one trail.
-
-        if self.valid and (self.show_page_trail or self.remember_last_visit):
+        if self._wantTrail():
             # load trail if not known
-            self.getTrail()      
+            self.getTrail()
 
+            pagename = page.page_name
             # Add only existing pages that the user may read
-            if self._request:
-                from MoinMoin.Page import Page
-                page = Page(self._request, pagename)
-                if not (page.exists() and
-                        self._request.user.may.read(page.page_name)):
-                    return
+            if not (page.exists() and self._request.user.may.read(pagename)):
+                return
 
             # Save interwiki links internally
             if self._cfg.interwikiname:
@@ -841,38 +954,19 @@ class User:
             self._trail.append(pagename)
             self.saveTrail()
 
-        # TODO: release lock here
-            
     def saveTrail(self):
-        """ Save trail file
-
-        Save using one write call, which should be fine in most cases,
-        but will fail in rare cases without real file locking.
-        """
-        data = '\n'.join(self._trail) + '\n'
-        path = self.__filename() + ".trail"
-        try:
-            file = codecs.open(path, "w", config.charset)
-            try:
-                file.write(data)
-            finally:
-                file.close()
-        except (IOError, OSError), err:
-            self._request.log("Can't save trail file: %s" % str(err))
+        """ Save trail into session """
+        if not self._request.session.is_new:
+            self._request.session['trail'] = self._trail
 
     def getTrail(self):
         """ Return list of recently visited pages.
-        
+
         @rtype: list
         @return: pages in trail
         """
-        if self.valid and (self.show_page_trail or self.remember_last_visit) \
-                and not self._trail \
-                and os.path.exists(self.__filename() + ".trail"):
-            try:
-                trail = codecs.open(self.__filename() + ".trail", 'r', config.charset).readlines()
-            except (OSError, ValueError):
-                trail = []
+        if not self._trail and self._wantTrail():
+            trail = self._request.session.get('trail', [])
             trail = [t.strip() for t in trail]
             trail = [t for t in trail if t]
             self._trail = trail[-self._cfg.trail_size:]
@@ -883,10 +977,16 @@ class User:
     # Other
 
     def isCurrentUser(self):
+        """ Check if this user object is the user doing the current request """
         return self._request.user.name == self.name
 
     def isSuperUser(self):
-        superusers = self._request.cfg.superuser
+        """ Check if this user is superuser """
+        request = self._request
+        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1' and request.user and request.user.valid:
+            # the DesktopEdition gives any local user superuser powers
+            return True
+        superusers = request.cfg.superuser
         assert isinstance(superusers, (list, tuple))
         return self.valid and self.name and self.name in superusers
 
@@ -896,73 +996,99 @@ class User:
         host = self.isCurrentUser() and self._cfg.show_hosts and self._request.remote_addr
         return host or _("<unknown>")
 
+    def wikiHomeLink(self):
+        """ Return wiki markup usable as a link to the user homepage,
+            it doesn't matter whether it already exists or not.
+        """
+        wikiname, pagename = wikiutil.getInterwikiHomePage(self._request, self.name)
+        if wikiname == 'Self':
+            if wikiutil.isStrictWikiname(self.name):
+                markup = pagename
+            else:
+                markup = '[[%s]]' % pagename
+        else:
+            markup = '[[%s:%s]]' % (wikiname, pagename)
+        return markup
+
     def signature(self):
-        """ Return user signature using markup
-        
-        Users sign with a link to their homepage, or with text if they
-        don't have one. The text may be parsed as a link if it's using
-        CamelCase. Visitors return their host address.
-        
+        """ Return user signature using wiki markup
+
+        Users sign with a link to their homepage.
+        Visitors return their host address.
+
         TODO: The signature use wiki format only, for example, it will
         not create a link when using rst format. It will also break if
         we change wiki syntax.
         """
-        if not self.name:
-            return self.host()
-        
-        wikiname, pagename = wikiutil.getInterwikiHomePage(self._request,
-                                                           self.name)
-        if wikiname == 'Self':
-            if not wikiutil.isStrictWikiname(self.name):
-                markup = '["%s"]' % pagename
-            else:
-                markup = pagename
+        if self.name:
+            return self.wikiHomeLink()
         else:
-            markup = '%s:%s' % (wikiname, pagename.replace(" ","_")) 
-        return markup
+            return self.host()
+
+    def generate_recovery_token(self):
+        key = random_string(64, "abcdefghijklmnopqrstuvwxyz0123456789")
+        msg = str(int(time.time()))
+        h = hmac.new(key, msg, sha).hexdigest()
+        self.recoverpass_key = key
+        self.save()
+        return msg + '-' + h
+
+    def apply_recovery_token(self, tok, newpass):
+        key = self.recoverpass_key
+        parts = tok.split('-')
+        if len(parts) != 2:
+            return False
+        try:
+            stamp = int(parts[0])
+        except ValueError:
+            return False
+        # only allow it to be valid for twelve hours
+        if stamp + 12*60*60 < time.time():
+            return False
+        # check hmac
+        h = hmac.new(self.recoverpass_key, str(stamp), sha).hexdigest()
+        if h != parts[1]:
+            return False
+        self.recoverpass_key = ""
+        self.enc_password = encodePassword(newpass)
+        self.save()
+        return True
 
     def mailAccountData(self, cleartext_passwd=None):
-        from MoinMoin.util import mail
-        from MoinMoin.wikiutil import getSysPage
+        """ Mail a user who forgot his password a message enabling
+            him to login again.
+        """
+        from MoinMoin.mail import sendmail
+        from MoinMoin.wikiutil import getLocalizedPage
         _ = self._request.getText
 
-        if not self.enc_password: # generate pw if there is none yet
-            from random import randint
-            import base64
-    
-            charset = 'utf-8'
-            pwd = "%s%d" % (str(time.time()), randint(0, 65535))
-            pwd = pwd.encode(charset)
-
-            pwd = sha.new(pwd).digest()
-            pwd = '{SHA}%s' % base64.encodestring(pwd).rstrip()
-    
-            self.enc_password = pwd
-            self.save()
+        tok = self.generate_recovery_token()
 
         text = '\n' + _("""\
 Login Name: %s
 
-Login Password: %s
+Password recovery token: %s
 
-Login URL: %s/%s?action=login
-""", formatted=False) % (
-                        self.name, self.enc_password, self._request.getBaseURL(), getSysPage(self._request, 'UserPreferences').page_name)
+Password reset URL: %s/?action=recoverpass&name=%s&token=%s
+""") % (
+                        self.name,
+                        tok,
+                        self._request.getBaseURL(),
+                        url_quote_plus(self.name),
+                        tok, )
 
         text = _("""\
-Somebody has requested to submit your account data to this email address.
+Somebody has requested to email you a password recovery token.
 
-If you lost your password, please use the data below and just enter the
-password AS SHOWN into the wiki's password form field (use copy and paste
-for that).
-
-After successfully logging in, it is of course a good idea to set a new and known password.
-""", formatted=False) + text
+If you lost your password, please go to the password reset URL below or
+go to the password recovery page again and enter your username and the
+recovery token.
+""") + text
 
 
         subject = _('[%(sitename)s] Your wiki account data',
-                    formatted=False) % {'sitename': self._cfg.sitename or "Wiki"}
-        mailok, msg = mail.sendmail(self._request, [self.email], subject,
+                ) % {'sitename': self._cfg.sitename or "Wiki"}
+        mailok, msg = sendmail.sendmail(self._request, [self.email], subject,
                                     text, mail_from=self._cfg.mail_from)
-        return msg
+        return mailok, msg
 
