@@ -2,7 +2,7 @@
 """
     MoinMoin - Supporting function for Python magic
 
-    @copyright: 2002 by Jrgen Hermann <jh@web.de>
+    @copyright: 2002 Juergen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
 
@@ -10,9 +10,6 @@
 ### Module import / Plugins
 #############################################################################
 
-import sys
-
-
 def isImportable(module):
     """ Check whether a certain module is available.
     """
@@ -30,7 +27,7 @@ def getPackageModules(packagefile):
     import os, re
 
     packagedir = os.path.dirname(packagefile)
-    
+
     in_plugin_dir = lambda dir, ops=os.path.split: ops(ops(dir)[0])[1] == "plugin"
 
     moinmodule = __import__('MoinMoin')
@@ -39,14 +36,14 @@ def getPackageModules(packagefile):
     if not in_plugin_dir(packagedir) and hasattr(moinmodule, '__loader__'):
         pyre = re.compile(r"^([^_].*)\.py(?:c|o)$")
         zipfiles = moinmodule.__loader__._files
-        dirlist = [file[0].replace(r'/', '\\').split('\\')[-1]
-                   for file in zipfiles.values() if packagedir in file[0]]
+        dirlist = [entry[0].replace(r'/', '\\').split('\\')[-1]
+                   for entry in zipfiles.values() if packagedir in entry[0]]
     else:
         pyre = re.compile(r"^([^_].*)\.py$")
         dirlist = os.listdir(packagedir)
 
-    pyfiles = filter(None, map(pyre.match, dirlist))
-    modules = map(lambda x: x.group(1), pyfiles)
+    matches = [pyre.match(fn) for fn in dirlist]
+    modules = [match.group(1) for match in matches if match]
 
     modules.sort()
     return modules
@@ -59,7 +56,7 @@ def importName(modulename, name):
     names only in runtime.
 
     Any error raised here must be handled by the caller.
-    
+
     @param modulename: full qualified mudule name, e.g. x.y.z
     @param name: name to import from modulename
     @rtype: any object
@@ -80,17 +77,17 @@ def makeThreadSafe(function, lock=None):
     @param function: function to make thread safe
     @param lock: threading.Lock instance or None
     @rtype: function
-    @retrun: function decorated with locking
+    @return: function decorated with locking
     """
     if lock is None:
         import threading
         lock = threading.Lock()
-    
+
     def decorated(*args, **kw):
         lock.acquire()
         try:
             return function(*args, **kw)
         finally:
             lock.release()
-            
+
     return decorated
