@@ -21,7 +21,7 @@
     Also, the URL http://moinmo.in/HelpOnConfiguration has
     a list of config options.
 
-    ** Please do not use this file for a wiki farm. Use the sample file 
+    ** Please do not use this file for a wiki farm. Use the sample file
     from the wikifarm directory instead! **
 """
 
@@ -38,7 +38,7 @@ class Config(DefaultConfig):
     # Wiki logo. You can use an image, text or both. [Unicode]
     # For no logo or text, use '' - the default is to show the sitename.
     # See also url_prefix setting below!
-    logo_string = u'<img src="/moin_static163/common/moinmoin.png" alt="MoinMoin Logo">'
+    logo_string = u'<img src="/moin_static171/common/moinmoin.png" alt="MoinMoin Logo">'
 
     # name of entry page / front page [Unicode], choose one of those:
 
@@ -76,12 +76,12 @@ class Config(DefaultConfig):
 
     # The URL prefix we use to access the static stuff (img, css, js).
     # NOT touching this is maybe the best way to handle this setting as moin
-    # uses a good internal default (something like '/moin_static163' for moin
-    # version 1.6.3).
+    # uses a good internal default (something like '/moin_static171' for moin
+    # version 1.7.1).
     # For Twisted and standalone server, the default will automatically work.
     # For others, you should make a matching server config (e.g. an Apache
     # Alias definition pointing to the directory with the static stuff).
-    #url_prefix_static = '/moin_static163'
+    #url_prefix_static = '/moin_static171'
 
 
     # Security ----------------------------------------------------------
@@ -142,23 +142,26 @@ class Config(DefaultConfig):
 
     # Language options --------------------------------------------------
 
-    # See http://moinmo.in/ConfigMarket for configuration in 
+    # See http://moinmo.in/ConfigMarket for configuration in
     # YOUR language that other people contributed.
 
     # The main wiki language, set the direction of the wiki pages
     language_default = 'en'
 
+    # the following regexes should match the complete name when used in free text
+    # the group 'all' shall match all, while the group 'key' shall match the key only
+    # e.g. CategoryFoo -> group 'all' ==  CategoryFoo, group 'key' == Foo
+    # moin's code will add ^ / $ at beginning / end when needed
     # You must use Unicode strings here [Unicode]
-    page_category_regex = u'^Category[A-Z]'
-    page_dict_regex = u'[a-z]Dict$'
-    page_form_regex = u'[a-z]Form$'
-    page_group_regex = u'[a-z]Group$'
-    page_template_regex = u'[a-z]Template$'
+    page_category_regex = ur'(?P<all>Category(?P<key>\S+))'
+    page_dict_regex = ur'(?P<all>(?P<key>\S+)Dict)'
+    page_group_regex = ur'(?P<all>(?P<key>\S+)Group)'
+    page_template_regex = ur'(?P<all>(?P<key>\S+)Template)'
 
     # Content options ---------------------------------------------------
 
     # Show users hostnames in RecentChanges
-    show_hosts = 1                  
+    show_hosts = 1
 
     # Enable graphical charts, requires gdchart.
     #chart_options = {'width': 600, 'height': 300}
