@@ -93,12 +93,12 @@ class FarmConfig(DefaultConfig):
 
     # The URL prefix we use to access the static stuff (img, css, js).
     # NOT touching this is maybe the best way to handle this setting as moin
-    # uses a good internal default (something like '/moin_static162' for moin
-    # version 1.6.2).
+    # uses a good internal default (something like '/moin_static171' for moin
+    # version 1.7.1).
     # For Twisted and standalone server, the default will automatically work.
     # For others, you should make a matching server config (e.g. an Apache
     # Alias definition pointing to the directory with the static stuff).
-    #url_prefix_static = '/moin_static162'
+    #url_prefix_static = '/moin_static171'
 
 
     # Security ----------------------------------------------------------
@@ -154,17 +154,21 @@ class FarmConfig(DefaultConfig):
 
     # Language options --------------------------------------------------
 
-    # See http://moinmo.in/ConfigMarket for configuration in 
+    # See http://moinmo.in/ConfigMarket for configuration in
     # YOUR language that other people contributed.
 
     # The main wiki language, set the direction of the wiki pages
     language_default = 'en'
 
+    # the following regexes should match the complete name when used in free text
+    # the group 'all' shall match all, while the group 'key' shall match the key only
+    # e.g. CategoryFoo -> group 'all' ==  CategoryFoo, group 'key' == Foo
+    # moin's code will add ^ / $ at beginning / end when needed
     # You must use Unicode strings here [Unicode]
-    page_category_regex = u'^Category[A-Z]'
-    page_dict_regex = u'[a-z]Dict$'
-    page_group_regex = u'[a-z]Group$'
-    page_template_regex = u'[a-z]Template$'
+    page_category_regex = ur'(?P<all>Category(?P<key>\S+))'
+    page_dict_regex = ur'(?P<all>(?P<key>\S+)Dict)'
+    page_group_regex = ur'(?P<all>(?P<key>\S+)Group)'
+    page_template_regex = ur'(?P<all>(?P<key>\S+)Template)'
 
     # Content options ---------------------------------------------------
 
