@@ -3,38 +3,44 @@
 
     Run this server with mointwisted script on Linux or Mac OS X, or
     mointwisted.cmd on Windows.
-    
+
     @copyright: 2004-2005 Thomas Waldmann, Oliver Graf, Nir Soffer
     @license: GNU GPL, see COPYING for details.
 """
 
-# System path configuration
+import sys, os
 
-import sys
+# a) Configuration of Python's code search path
+#    If you already have set up the PYTHONPATH environment variable for the
+#    stuff you see below, you don't need to do a1) and a2).
 
-# Path of the directory where wikiconfig.py is located.
-# YOU NEED TO CHANGE THIS TO MATCH YOUR SETUP.
-sys.path.insert(0, '/path/to/wikiconfig')
+# a1) Path of the directory where the MoinMoin code package is located.
+#     Needed if you installed with --prefix=PREFIX or you didn't use setup.py.
+#sys.path.insert(0, 'PREFIX/lib/python2.3/site-packages')
 
-# Path to MoinMoin package, needed if you installed with --prefix=PREFIX
-# or if you did not use setup.py
-## sys.path.insert(0, 'PREFIX/lib/python2.3/site-packages')
+# a2) Path of the directory where wikiconfig.py / farmconfig.py is located.
+#     See wiki/config/... for some sample config files.
+#sys.path.insert(0, '/path/to/wikiconfigdir')
+#sys.path.insert(0, '/path/to/farmconfigdir')
 
-# Path to the directory where farmconfig is located (if different).
-## sys.path.insert(0, '/path/to/farmconfig')
+# b) Configuration of moin's logging
+#    If you have set up MOINLOGGINGCONF environment variable, you don't need this!
+#    You also don't need this if you are happy with the builtin defaults.
+#    See wiki/config/logging/... for some sample config files.
+#from MoinMoin import log
+#log.load_config('/path/to/logging_configuration_file')
 
 # Debug mode - show detailed error reports
-## import os
-## os.environ['MOIN_DEBUG'] = '1'
+#os.environ['MOIN_DEBUG'] = '1'
+
 
 from MoinMoin.server.server_twisted import TwistedConfig, makeApp
 
 
 class Config(TwistedConfig):
 
-    # Server name
-    # Used to create .log, .pid and .prof files
-    name = 'mointwisted'
+    # Server name - used to create .pid and .prof files
+    name = 'moin'
 
     # Path to moin shared files (default '/usr/share/moin/wiki/htdocs')
     docs = '/usr/share/moin/htdocs'
@@ -42,7 +48,7 @@ class Config(TwistedConfig):
     # URL prefix for the static stuff (used to access stuff in docs) - you
     # usually should not need to change this because Twisted moin uses
     # matching defaults for here and for wikiconfig.py:
-    #url_prefix_static = '/moin_static162'
+    #url_prefix_static = '/moin_static171'
 
     # The server will run with as this user and group (default 'www-data')
     user = 'www-data'
@@ -53,7 +59,7 @@ class Config(TwistedConfig):
     port = 8080
 
     # Interfaces (default [''])
-    # The interfaces the server will listen to. 
+    # The interfaces the server will listen to.
     # [''] - listen to all interfaces defined on the server
     # ['red.wikicolors.org', 'blue.wikicolors.org'] - listen to some
     # If '' is in the list, other ignored.
@@ -72,9 +78,6 @@ class Config(TwistedConfig):
     # which analyze them and display statistics.
     ## logPath_twisted = 'mointwisted.log'
 
-    # moin log file
-    ## logPath = 'moin.log'
-
     # Memory profile (default commented)
     # Useful only if you are a developer or interested in moin memory usage
     ## from MoinMoin.util.profile import TwistedProfiler
