@@ -8,7 +8,8 @@ This is NOT intended for internet or server or multiuser use due to relaxed secu
 import sys, os
 from MoinMoin.config.multiconfig import DefaultConfig
 
-class Config(DefaultConfig):
+
+class LocalConfig(DefaultConfig):
     # vvv DON'T TOUCH THIS EXCEPT IF YOU KNOW WHAT YOU DO vvv
     moinmoin_dir = os.path.abspath(os.path.normpath(os.path.dirname(sys.argv[0])))
     data_dir = os.path.join(moinmoin_dir, 'wiki', 'data')
@@ -18,11 +19,28 @@ class Config(DefaultConfig):
     acl_rights_default = u"All:read,write,delete,revert,admin"
     surge_action_limits = None # no surge protection
     sitename = u'MoinMoin DesktopEdition'
-    logo_string = u'<img src="/moin_static163/common/moinmoin.png" alt="MoinMoin Logo">'
+    logo_string = u'<img src="/moin_static171/common/moinmoin.png" alt="MoinMoin Logo">'
     page_front_page = u'FrontPage' # change to some better value
-    page_credits = [
-        '<a href="http://moinmo.in/">MoinMoin DesktopEdition Powered</a>',
-        '<a href="http://moinmo.in/Python">Python Powered</a>',
-    ]
     # ^^^ DON'T TOUCH THIS EXCEPT IF YOU KNOW WHAT YOU DO ^^^
 
+
+    # Add your configuration items here.
+
+
+
+# DEVELOPERS! Do not add your configuration items there,
+# you could accidentally commit them! Instead, create a
+# wikiconfig_local.py file containing this:
+#
+# from wikiconfig import LocalConfig
+#
+# class Config(LocalConfig):
+#     configuration_item_1 = 'value1'
+#
+
+try:
+    from wikiconfig_local import Config
+except ImportError, err:
+    if not str(err).endswith('wikiconfig_local'):
+        raise
+    Config = LocalConfig
